







;         ciforth : a generic I86 ISO FORTH by HCC FIG
; $Id: ciarm.gnr,v 6.120 2022/03/31 15:10:56 albert Exp $
; Copyright (2012): Albert van der Horst/Frans van der Markt by GNU Public License
;
;HCC FIG Holland : Hobby Computer Club, Forth Interest Group  Holland
; ;   ciforth $Revision: 6.120 $
;
;
;
;
; If this is a configured assembly file, it should be accompanied with configured
; documentation (texinfo, ps, html.)
; WITHOUT THE DOCUMENTATION: GIVE UP! GET THE REAL THING!
; You have a configured system, if there are NO curly brackets on the next line.
;                          
;
; Configuration of this particular version:
; 64-bits
; running under Linux  ;  with native forth I/O 

; Normally ciforth doesn't observe ISO >IN.
; Contains :
; (there may be no items here.)
;        Security words
;         Loadable words, i.e. all of ISO CORE, more than is needed
;           for a self contained kernel.
; A field in the header to point to source
;
;
; This is a NASM version of ciforth created by ``m4'' from the generic listing.
; It can be assembled using ``nasm'' obtainable via :
; Source: ftp://ftp.us.kernel.org/pub/software/devel/nasm/source/
; URL: http://www.cryogen.com/Nasm/


; This version can be assembled on a Linux system by
;   nasm forth.asm -felf -o forth.o
;   ld -N forth.o -o forth
;And to get the smallest possible executable(optional):
;   strip forth -s -R .data
 


; For assembling on other systems where nasm is available see the
; documentation of nasm.

_COMMENTED(
        A generic version of ISO FORTH for ARM processors
                M4_SUPPLIER

                in cooperation with
                HCC Forth user group
                The Netherlands
                www.forth.hccnet.nl

              based on
              FIG-FORTH
   implemented by:  Charlie Krajewski
                    205 ( BIG ) Blue Rd.
                    Middletown, CT  06457

  The listing has been made possible by the
  prior work of:
               Thomas Newman, Hayward, Ca.

 : other_acknowledgements
         John_Cassidy
         Kim_Harris
         George_Flammer
         Robert_D._Villwock ;

 : for tools
         Richard M. Stallman
         Linus Torvalds

No one who programs with FORTH can afford to be without:
  "Starting Forth  by Leo Brodie" and "Thinking Forth by Leo Brodie".
   Both out of print, but saved on the internet.

This Forth is a descendant in the 300+ (RCS)- generations from fig-Forth.

For nostalgic reasons the following comment has never been removed:
   Although there is much to be said for typing in your own
   listing and getting it running, there is much to be said
   not typing in your own listing.  If you feel that 100+
   pages of plinking is nutty, contact me for availability
   of a disc with source & executable files.  Obtainable at
   a bargain basement price, prepare yourself for bargain
   basement support.

All publications of the FORTH Interest Group are public domain.
They may be further distributed by the inclusion of this
credit notice:
               This publication has been made available by:

               FORTH Interest Group
               P.O. Box 1105
               San Carlos, Ca.  94070
[I feel obliged to keep this last one in (AH). Note that although it is
based on fig-Forth no stone is left unturned.]
)         ;
; ########################################################################################
;                       PREPARATION (no code)
; ########################################################################################

FIGREL  EQU     5       ; FIG RELEASE #
FIGREV  EQU     0       ; FIG REVISION #
USRVER  EQU     0      ; USER VERSION NUMBER, a digit now
;
;      VERY ELEMENTARY .
CW      EQU     8    ; Size of a cell in Forth, not in the bootcode.
ERRORSCREEN EQU     48    ; Screen where the error messages start.
;
;      MEMORY LAYOUT.
; Normally this is specified at the m4 configuration level.
; For a configured system these values can be changed at this single place. 
NBUF    EQU     16   ; No. of buffers, or screens 
KBBUF   EQU     1024      ; Data bytes per disk buffer
US      EQU     0x40*CW  ; User variable space, eats into next data stack, if any.
EM      EQU     (0x4000000)     ; Where the memory ends w.r.t. ORIG. 1) 
EMP     EQU     (EM-1)/0x1000+1 ; Number of pages.
TAS     EQU     0x100000    ;Size of unitialised area per task.
STACKSIZE   EQU TAS/4  ;For both stacks.  but for now
TIBSIZE     EQU TAS/4  ;TIB size. 
;


;
;      ASCII CHARACTER EQUIVALENTS
;
ABL     EQU     ' '     ; SPACE
ACR     EQU     0x0D     ; CR
AMS     EQU     '-'          ; MINUS SIGN 
ASO     EQU     '['          ; SQUARE BRACKET OPEN 
ASC     EQU     ']'          ; SQUARE BRACKET CLOSE 
ADOT    EQU     '.'          ; PERIOD
ALF     EQU     0x0A     ; LINE FEED, USED INTERNALLY AS LINE ENDER 
AFF     EQU     0x0C     ; FORM FEED
BELL    EQU     0x07     ; ^G
BSIN    EQU     0x08     ; INPUT DELETE CHARACTER
BSOUT   EQU     0x08     ; OUTPUT BACKSPACE ( ^H )
;
;      HEADER RELATED EQUATES
;      dea is the dictionary entry address 
B_DUMMY   EQU     0x01     ; dea is dummy, from namespace link
B_INVIS   EQU     0x02     ; dea is invisible, "smudged".
B_IMMED   EQU     0x04     ; dea is a immediate.
B_DENOT   EQU     0x08     ; dea is a denotation.
C_HOFFSET EQU     0       ; Offsets of code field in cells, w.r.t. dea
D_HOFFSET EQU     1       ; Same for data field
F_HOFFSET EQU     2       ; Same for flag field
L_HOFFSET EQU     3       ; Same for link field
N_HOFFSET EQU     4       ; Same for name field
S_HOFFSET EQU     5       ; Same for source field
PH_OFFSET EQU     5+1   ; Past header field: Start of data area. 
BD_OFFSET EQU     5+1+1 ; Start of BODY for CREATEd word.
HEADSIZE  EQU     (CW*(PH_OFFSET))  ;In cells, only to clean up source.
;
 ;  


BPS     EQU     512             ;Bytes/sector, common to all of MSDOS
SPB     EQU     KBBUF/BPS       ;Sectors per buffer 
;







; Bring the names in line with what Forth want immediately. 

; ------------------------------------------------------------      
;   Start of constants stolen from C.                                 
; ------------------------------------------------------------      

SEEK_SET	EQU	0x0
SEEK_CUR	EQU	0x1
TCGETS	EQU	0x5401
TCSETS	EQU	0x5402
ECHO	EQU	0x8
EAGAIN	EQU	0xb
EINTR	EQU	0x4
EPIPE	EQU	0x20
VMIN	EQU	0x6
VTIME	EQU	0x5
ICANON	EQU	0x2
O_RDWR	EQU	0x2
O_RDONLY	EQU	0x0
O_WRONLY	EQU	0x1
O_CREAT	EQU	0x40
O_TRUNC	EQU	0x200
O_NONBLOCK	EQU	0x800
WEXITED	EQU	0x4
P_PID	EQU	0x1
AT_FDCWD	EQU	0xffffff9c
AT_REMOVEDIR	EQU	0x200
MAP_SHARED	EQU	0x1
MAP_PRIVATE	EQU	0x2
SIZE_TERMIO	EQU	0x3c

; Numbers of system calls. See "Linux kernel Internals" Appendix A. 
; By M.Beck, H. Boehme e.a. Addison Wesley.                         
; or in practice                                                    
; https://github.com/torvalds/linux/tree/master/arch/x86/entry/syscalls
; The system calls themselves are extensively documented in chapter 
; 2 of the man pages, e.g. "man 2 exit"
exit	EQU	0x5d
openat	EQU	0x38
close	EQU	0x39
unlinkat	EQU	0x23
mmap	EQU	0xde
chdir	EQU	0x31
read	EQU	0x3f
pselect6	EQU	0x48
write	EQU	0x40
ioctl	EQU	0x1d
lseek	EQU	0x3e
execve	EQU	0xdd
waitid	EQU	0x5f
clone	EQU	0xdc
kill	EQU	0x81
times	EQU	0x99
; ------------------------------------------------------------      
;   End of constants stolen from C.                                 
; ------------------------------------------------------------      

select EQU pselect6     ; Can you believe this ?

RAWIO           EQU     (ECHO | ICANON)

; 


_main:



; allocatable, executable 
; Must be normal .text or otherwise linking with c-lib fails. 
; .text is normally not writable but linker -N option takes care of that. 
        _TEXT_
        ;
        GLOBAL  _start       ; Entry point.
        GLOBAL  start        ; Entry point.
_ORIG:
_start:      ; Entry point.
; 









NOBOOT:         ; Skip till here if not booting.


ENDBOOT:












; ########################################################################################
;                       FORTH startup CODE + restarts + debug 
; ########################################################################################
;


; 4.15.3.2
start:
;
COLD_ENTRY:

        ADR   X4, USINI
        MOV     X0, SP              ;Remember ARGS.
        STR     X0, [X4, #CW*(31)]
;


; Initialisation of the Forth engine.
        ADR     X4, USINI
        LDR     X0, [X4, #CW*(2)]  ;Parameter stack
        MOV     X13, X0           ;Parameter stack
        LDR     X12, [X4, #CW*(3)]  ;Return stack
        ADR     X11, COLD + CW*(PH_OFFSET)    ;Interpreter pointer
      LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  



; 

;
;
; ########################################################################################
;                       FORTH ITSELF
; ########################################################################################
;
_COMMENTED(
   FORTH REGISTERS
   The names under FORTH are used in the generic source.

   FORTH   ARM      FORTH PRESERVATION RULES
   -----   ----     ----- ------------ -----
dnl Note how the following is to pass through m4
           R0-R3   Used within and cluttered by Forth words.
                    across FORTH words.
                    The first of those is {WOR} (probably)

   {WOR}   WOR      Working register.  When entering a word
                    via its code field the DEA is passed in {WOR}.

            R0      Scratch register.  Used to pass data from
                    FORTH words, see label APUSH or macro {_APUSH}

            R1      General register.  Used when two data items are passed
                    from FORTH words, see label DPUSH or macro {_DPUSH}

   {HIP}   HIP      High level Interpreter Pointer.  Must be preserved

   {RPO}   RPO      Return stack pointer.  Must be preserved across
                    FORTH words.

   {SPO}   SPO      Parameter stack pointer.  Must be preserved
                    across FORTH words.

           PC       Machine Process Counter, sometimes used
                    for relative addressing

This is the ARM asignment, not valid for 64 bit.

   R13     SP      ARM stack pointer.
                   In thumb this can be used with pop and push.

   R14     RL      Machine Link Register for calls.
                    Can be used within Forth words, with care.

  R15     PC        Machine Process Counter (Program counter)
                    Cannot be used within Forth words.



            The remaining registers are free to use.
            R0-R3 are the normal Forth work registers.
            Within assembler words WOR can be used too,
            as it has served its purpose.

            Here are some  useful code sequences.
            For next:
                    _NEXT
            For push single and next:
                    _PUSH
            For push double and next:
                    _2PUSH



----------------------------------------------------------
)         ;
_COMMENTED(
---------------------------------------------

   COMMENT CONVENTIONS
   ------- -----------

   =       IS EQUAL TO
   <-      ASSIGNMENT

  NAME        =  Address of name
  (NAME)      =  Contents of name
  DEA         =  DICTIONARY ENTRY ADDRESS
                 fields have fixed offsets from DEA
  CFA         =  CODE FIELD ADDRESS : a pointer to executable code
  DFA         =  DATA FIELD ADDRESS : a pointer to
                        data/high level code/ DOES> pointer
  FFA         =  FLAG FIELD ADDRESS: contains flags
  LFA         =  LINK FIELD ADDRESS: a pointer to the next DEA
  NFA         =  NAME FIELD ADDRESS: a pointer to a name buffer
  _SOURCEFIELD_({SFA =  SOURCE FIELD ADDRESS: a pointer into source code})
  PHA         =  POST HEADER ADDRESS: After all fields

  S1          =  Parameter stack - 1st cell
  S2          =  Parameter stack - 2nd cell
  RP1          =  Return stack    - 1st cell
  R2          =  Return stack    - 2nd cell

  LSB         =  Least significant bit
  MSB         =  Most  significant bit
  LB          =  Low byte
  HB          =  High byte
  LC          =  Low  cell
  HC          =  High cell

------------------------------------------------------------
) ; 
        ;

;
; 
; There may be no jumps to NEXT at all 
; The label NEXT1 is rarely relevant (for _OLDDEBUG_) 
DPUSH:  STR     X1, [X13,#-(CW*(1))]!      ; Fall through.
APUSH:  STR     X0, [X13,#-(CW*(1))]!
NEXT:
;
        LDR     X0, [X11],#+(CW*(1))                  ; R0 <- (HIP) 
NEXT1:
        LDR     X1, [X0]
        BR      X1     ; TO `CFA'
;
;       Dictionary starts here.


;  ************
;  *   ONLY   *
;  ************
;
        ALIGN    8
N_ONLY:
        DQ      4
        DB      "ONLY"
        ALIGN    8
ONLY:
        DQ    DODOE
        DQ    ONLY+HEADSIZE
        DQ    0x0
        DQ    0
        DQ    N_ONLY
        DQ    0

        DQ      DOVOC
        DQ      FORTH     ; NEXT NAMESPACE 
ONLYBODY:

        ALIGN    8

        DQ    0x0
        DQ    0
        DQ    B_DUMMY
        DQ    STARTOFONLY
        DQ    (ZERO+(CW*(D_HOFFSET)))
        DQ    0

;

;  *******************
;  *   ENVIRONMENT   *
;  *******************
;
        ALIGN    8
N_ENV:
        DQ      11
        DB      "ENVIRONMENT"
        ALIGN    8
ENV:
        DQ    DODOE
        DQ    ENV+HEADSIZE
        DQ    0x0
        DQ    ONLY
        DQ    N_ENV
        DQ    0

        DQ      DOVOC
        DQ      ONLY       ; NEXT NAMESPACE 

        ALIGN    8

        DQ    0x0
        DQ    0
        DQ    B_DUMMY
        DQ    STARTOFENVIRONMENT
        DQ    (ZERO+(CW*(D_HOFFSET)))
        DQ    0

;

;  ************
;  *   COLD   *
;  ************
;
        ALIGN    8
N_COLD:
        DQ      4
        DB      "COLD"
        ALIGN    8
COLD:
        DQ    DOCOL
        DQ    COLD+HEADSIZE
        DQ    0x0
        DQ    ENV
        DQ    N_COLD
        DQ    0

; Fill user area for single task.
        DQ      LIT, USINI      ; Fails if SP/RP are not usable!
        DQ      LIT, USINI+(CW*(1)), FETCH
        DQ      LIT, US
        DQ      LCMOVE
        DQ      INIT
        DQ      ONE            ; Sign on wanted.


        DQ      ZERO, LIT, TCGETS, TERMIO
        DQ      LIT, ioctl, XOS, ZLESS
        DQ      ZBRAN
        DQ      COLD1-$-CW
        ; Not a terminal. Cannot set the terminal, get rid of "OK".
        DQ      DROP, ZERO      ; Sign on suppressed.
        DQ      LIT,TASK  ; Suppress OK for pipes.
        DQ      LIT,OK
        DQ      LIT, (CW*(3)), LCMOVE
COLD1:
 ; 

        DQ      SKIP
         DQ      8
SB0: DB      "/dev/mem"
       ALIGN    8
        DQ      LIT, SB0
        DQ      LIT, 8
        DQ      LIT, 0x00181002
        DQ      OFILE, LOR, DEVMEM, STOR
;
       DQ      OPTIONS
        DQ      ZBRAN
        DQ      COLD5-$-CW

        DQ      SIGNON    ; Suppressed for scripting! Or any options.
COLD5:
        DQ      ABORT
        DQ      BYE     ; In case of turnkey programs.
        DQ      SEMIS   ; Unnecessary, but helpful for decompilation.
;

;  ************
;  *   USER   *
;  ************
;
        ALIGN    8
N_USER:
        DQ      4
        DB      "USER"
        ALIGN    8
USER:
        DQ    DOCOL
        DQ    USER+HEADSIZE
        DQ    0x0
        DQ    COLD
        DQ    N_USER
        DQ    0

        DQ      LCONST
        DQ      PSCOD
DOUSE:  LDR     X3,[X0, # + (CW*((D_HOFFSET)))] ;PFA  
        ADR     X0, USINI
        LDR     X0,[X0]
        ADD     X0,X0,X3      ;ADDR OF VARIABLE
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  

;

;  ***************
;  *   +ORIGIN   *
;  ***************
;
        ALIGN    8
N_PORIG:
        DQ      7
        DB      "+ORIGIN"
        ALIGN    8
PORIG:
        DQ    DOCOL
        DQ    PORIG+HEADSIZE
        DQ    0x0
        DQ    USER
        DQ    N_PORIG
        DQ    0

        DQ      LIT
        DQ      USINI
        DQ      PLUS
        DQ      SEMIS
;
;      Initialisation block for user variables through DOC-LINK
;       <<<<< must be in same order as user variables >>>>>
; To be fig compatible, there should be two vectors here  
; for the cold and warm start.
;
; ------------------------ Forth entry code, hooks for restart -------------
; FIXME. You want to jump here from -2 CELLS +USINI / -4 CELLS +USINI 
        ADR     X4, COLD
        MOV     X0, X4
        LDR     X1, [X0]
        BR      X1          ;Hope stacks are still okay.
        ALIGN    8
; Make it possible to revector WARM. You can even make it low level. 
        ADR      X4, WARM
        MOV      X0,X4
        LDR     X1, [X0]          ;Hope stacks are still okay.
        BR      X1
        ALIGN    8
USINI:  DQ      STRUSA
        DQ      STRUSA  ; INIT (U0) user area of the main task 1
        DQ      INITS0  ; INIT (S0)         2
        DQ      INITR0  - (CW*(1))
                        ; INIT (R0)         3
        DQ      STRTIB  ; INIT (TIB)        4
        DQ      BSIN    ; RUBOUT: get rid of latest char 5
        DQ      0       ; AVAILABLE         6
        DQ      1       ; INIT (WARNING)     7
        DQ      INITDP  ;      INIT (FENCE)  8
DPA:    DQ      INITDP  ;      INIT (DP)     9
        DQ      ENV ;       INIT (VOC-LINK) 10
        DQ      39+16        ; INIT (MAX-USER) 
;
        DQ      0, 0            ; WHERE             12 13 
        DQ      0, 0            ;REMAINDER   14 15 

        RESB    US-($ - USINI)        ; All user can be initialised.
;
;      <<<<< end of data used by cold start >>>>>
;


;  **************
;  *   VMA-IO   *
;  **************
;
        ALIGN    8
N_VMAIO:
        DQ      6
        DB      "VMA-IO"
        ALIGN    8
VMAIO:
        DQ    DOVAR
        DQ    VMAIO+HEADSIZE
        DQ    0x0
        DQ    PORIG
        DQ    N_VMAIO
        DQ    0

        DQ 0xB2000000
;

;  *******************
;  *   PERIPHERALS   *
;  *******************
;
        ALIGN    8
N_PERIPHERALS:
        DQ      11
        DB      "PERIPHERALS"
        ALIGN    8
PERIPHERALS:
        DQ    DOVAR
        DQ    PERIPHERALS+HEADSIZE
        DQ    0x0
        DQ    VMAIO
        DQ    N_PERIPHERALS
        DQ    0

        DQ 0x03000000
;

;  ***************
;  *   DEV-MEM   *
;  ***************
;
        ALIGN    8
N_DEVMEM:
        DQ      7
        DB      "DEV-MEM"
        ALIGN    8
DEVMEM:
        DQ    DOVAR
        DQ    DEVMEM+HEADSIZE
        DQ    0x0
        DQ    PERIPHERALS
        DQ    N_DEVMEM
        DQ    0

        DQ 0
;

;  ***************
;  *   MMAP-IO   *
;  ***************
;
        ALIGN    8
N_MMAPIO:
        DQ      7
        DB      "MMAP-IO"
        ALIGN    8
MMAPIO:
        DQ    DOCOL
        DQ    MMAPIO+HEADSIZE
        DQ    0x0
        DQ    DEVMEM
        DQ    N_MMAPIO
        DQ    0

; The libc mmap definition, not necessarily documenting the system call! 
; mmap( void *addr, size_t length, int prot, int flags, int fd,
; off_t offset );     see man mmap

      DQ     DEVMEM, FETCH, LDUP, ZERO, GREAT
      DQ      ZBRAN
        DQ      MMAP2-$-CW
      DQ     TOR
      DQ     VMAIO, FETCH
      DQ     LIT, 0x010000000 ; length of mapped area
      DQ     LIT, 3                  ; protection-bits
      DQ     LIT, MAP_SHARED         ; flags
      DQ     FROMR
      DQ     PERIPHERALS, FETCH

      DQ     ZERO                ; dummy 
      DQ     LIT, mmap, XOS7
MMAP2:
      DQ     VMAIO, STOR
      DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
;
;

;

;  *************
;  *   ABORT   *
;  *************
;
        ALIGN    8
N_ABORT:
        DQ      5
        DB      "ABORT"
        ALIGN    8
ABORT:
        DQ    DOCOL
        DQ    ABORT+HEADSIZE
        DQ    0x0
        DQ    MMAPIO
        DQ    N_ABORT
        DQ    0

        DQ      SZERO, FETCH, SPSTO
        DQ      ZERO, HANDLER, STOR
        DQ      QUIT
        DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
;
;

;  ************
;  *   WARM   *
;  ************
;
        ALIGN    8
N_WARM:
        DQ      4
        DB      "WARM"
        ALIGN    8
WARM:
        DQ    DOCOL
        DQ    WARM+HEADSIZE
        DQ    0x0
        DQ    ABORT
        DQ    N_WARM
        DQ    0

        DQ      INIT
        DQ      SIGNON
        DQ      ABORT
        DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
;

;  ************
;  *   NOOP   *
;  ************
;
        ALIGN    8
N_NOOP:
        DQ      4
        DB      "NOOP"
        ALIGN    8
NOOP:
        DQ    NOOP+HEADSIZE
        DQ    NOOP+HEADSIZE
        DQ    0x0
        DQ    WARM
        DQ    N_NOOP
        DQ    0

       LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
; 
;

;  ***********
;  *   LIT   *
;  ***********
;
        ALIGN    8
N_LIT:
        DQ      3
        DB      "LIT"
        ALIGN    8
LIT:
        DQ    LIT+HEADSIZE
        DQ    LIT+HEADSIZE
        DQ    0x0
        DQ    NOOP
        DQ    N_LIT
        DQ    0

        LDR     X0, [X11],#+(CW*(1))                  ; R0 <- (HIP) 
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ; TO TOP OF STACK
;

;todo EXECUTE
;  ***************
;  *   EXECUTE   *
;  ***************
;
        ALIGN    8
N_EXEC:
        DQ      7
        DB      "EXECUTE"
        ALIGN    8
EXEC:
        DQ    EXEC+HEADSIZE
        DQ    EXEC+HEADSIZE
        DQ    0x0
        DQ    LIT
        DQ    N_EXEC
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; GET XT
        LDR    X1, [X0]
        BR     X1  ;(IP) <- (PFA)

;

;  ***************
;  *   RECURSE   *
;  ***************
;
        ALIGN    8
N_RECURSE:
        DQ      7
        DB      "RECURSE"
        ALIGN    8
RECURSE:
        DQ    DOCOL
        DQ    RECURSE+HEADSIZE
        DQ    B_IMMED
        DQ    EXEC
        DQ    N_RECURSE
        DQ    0

        DQ      LATEST, COMMA
        DQ      SEMIS
;

;  **************
;  *   BRANCH   *
;  **************
;
        ALIGN    8
N_BRAN:
        DQ      6
        DB      "BRANCH"
        ALIGN    8
BRAN:
        DQ    (SKIP+HEADSIZE)
        DQ    BRAN+HEADSIZE
        DQ    0x0
        DQ    RECURSE
        DQ    N_BRAN
        DQ    0

;

;  ************
;  *   SKIP   *
;  ************
;
        ALIGN    8
N_SKIP:
        DQ      4
        DB      "SKIP"
        ALIGN    8
SKIP:
        DQ    SKIP+HEADSIZE
        DQ    SKIP+HEADSIZE
        DQ    0x0
        DQ    BRAN
        DQ    N_SKIP
        DQ    0

BRAN1:
        LDR     X0, [X11],#+(CW*(1))                  ; R0 <- (HIP) 

        SUB     X0,X0, #1
        
         ORR     X0,X0, #(CW-1) 
        ADD     X0,X0, #1
;
        ADD     X11,X11,X0
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***************
;  *   0BRANCH   *
;  ***************
;
        ALIGN    8
N_ZBRAN:
        DQ      7
        DB      "0BRANCH"
        ALIGN    8
ZBRAN:
        DQ    ZBRAN+HEADSIZE
        DQ    ZBRAN+HEADSIZE
        DQ    0x0
        DQ    SKIP
        DQ    N_ZBRAN
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; GET STACK VALUE
        ANDS     X0,X0,X0   ; ZERO?
        BEQ      BRAN1   ; YES, BRANCH
        ADD      X11,X11,# + (CW*(1))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***************
;  *   (+LOOP)   *
;  ***************
;
        ALIGN    8
N_XPLOOP:
        DQ      7
        DB      "(+LOOP)"
        ALIGN    8
XPLOOP:
        DQ    XPLOOP+HEADSIZE
        DQ    XPLOOP+HEADSIZE
        DQ    0x0
        DQ    ZBRAN
        DQ    N_XPLOOP
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ; GET LOOP VALUE
        LDR     X2,[X12]  ; INDEX = INDEX + INCR
        ADD     X2,X2,X3
        STR     X2,[X12]  ; GET NEW INDEX
        LDR     X4, [X12, #+(CW*(1))]
        SUB     X2,X2,X4  ; COMPARE WITH LIMIT
        EOR     X2,X2,X3  ; GENERATE SIGN
        ANDS    X2,X2,X2  ; TEST SIGN
        EOR     X0,X0,X0  ; DEFAULT
        BMI     XPO1     ; KEEP LOOPING
        SUB     X0,X0, #1      ;TRUE
XPO1: STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   (DO)   *
;  ************
;
        ALIGN    8
N_XDO:
        DQ      4
        DB      "(DO)"
        ALIGN    8
XDO:
        DQ    XDO+HEADSIZE
        DQ    XDO+HEADSIZE
        DQ    0x0
        DQ    XPLOOP
        DQ    N_XDO
        DQ    0

        LDR     X2, [X11],#+(CW*(1))                  ; R0 <- (HIP) 
        ADD     X2,X2,X11  ;Make absolute
        LDR     X1, [X13],#+(CW*(1))           ; INITIAL INDEX VALUE
        LDR     X3, [X13],#+(CW*(1))           ; LIMIT VALUE
        
         MOV     X0, X12
         MOV     X12, X13
         MOV     X13, X0
   ; GET RETURN STACK, USES REGISTER R0 ! 
        STR     X2, [X13,#-(CW*(1))]!      ; Target location.
        STR     X3, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        
         MOV     X0, X12
         MOV     X12, X13
         MOV     X13, X0
   ; GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   (?DO)   *
;  *************
;
        ALIGN    8
N_XQDO:
        DQ      5
        DB      "(?DO)"
        ALIGN    8
XQDO:
        DQ    XQDO+HEADSIZE
        DQ    XQDO+HEADSIZE
        DQ    0x0
        DQ    XDO
        DQ    N_XQDO
        DQ    0

        LDR     X2, [X11],#+(CW*(1))                  ; R0 <- (HIP) 
        ADD     X2,X2,X11  ;Make absolute
        LDR     X1, [X13],#+(CW*(1))      ; INITIAL INDEX VALUE
        LDR     X3, [X13],#+(CW*(1))      ; LIMIT VALUE
        CMP     X1,X3
        BGE     QXDO1
        
         MOV     X0, X12
         MOV     X12, X13
         MOV     X13, X0
   ; GET RETURN STACK, USES REGISTER R0 ! 
        STR     X2, [X13,#-(CW*(1))]!      ; Target location.
        STR     X3, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        
         MOV     X0, X12
         MOV     X12, X13
         MOV     X13, X0
   ; GET PARAMETER STACK, USES REGISTER R0 ! 
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
QXDO1:  MOV     X11,X2
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   I   *
;  *********
;
        ALIGN    8
N_IDO:
        DQ      1
        DB      "I"
        ALIGN    8
IDO:
        DQ    IDO+HEADSIZE
        DQ    IDO+HEADSIZE
        DQ    0x0
        DQ    XQDO
        DQ    N_IDO
        DQ    0

        LDR     X0,[X12] ; GET INDEX VALUE
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ; TO PARAMETER STACK
;

;  *********
;  *   J   *
;  *********
;
        ALIGN    8
N_JDO:
        DQ      1
        DB      "J"
        ALIGN    8
JDO:
        DQ    JDO+HEADSIZE
        DQ    JDO+HEADSIZE
        DQ    0x0
        DQ    IDO
        DQ    N_JDO
        DQ    0

        LDR     X0,[X12,#+(CW*(3))] ; GET INDEX VALUE
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ; TO PARAMETER STACK
;

;  **************
;  *   UNLOOP   *
;  **************
;
        ALIGN    8
N_UNLOOP:
        DQ      6
        DB      "UNLOOP"
        ALIGN    8
UNLOOP:
        DQ    UNLOOP+HEADSIZE
        DQ    UNLOOP+HEADSIZE
        DQ    0x0
        DQ    JDO
        DQ    N_UNLOOP
        DQ    0
      
        ADD X12,X12,#(CW*(3))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   DIGIT   *
;  *************
;
        ALIGN    8
N_DIGIT:
        DQ      5
        DB      "DIGIT"
        ALIGN    8
DIGIT:
        DQ    DIGIT+HEADSIZE
        DQ    DIGIT+HEADSIZE
        DQ    0x0
        DQ    UNLOOP
        DQ    N_DIGIT
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;NUMBER BASE
        LDR     X0, [X13],#+(CW*(1))      ;ASCII DIGIT
        AND     X0,X0,#0xFF
        AND     X1,X1,#0xFF
        SUBS    X0,X0,#'0'
        BLT     DIGI2   ;NUMBER ERROR
        CMP     X0,#9
        BLE     DIGI1   ;NUMBER = 0 THRU 9
        SUB     X0,X0,#7
        CMP     X0,#10   ;NUMBER 'A' THRU 'Z'?
        BLT     DIGI2   ;NO
DIGI1:  CMP     X0,X1   ; COMPARE NUMBER TO BASE
        BGE     DIGI2   ;NUMBER ERROR
        MOV     X1,X0   ;NEW BINARY NUMBER
        
        MOV    X0,#-1  ;TRUE FLAG  
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ;ADD TO STACK
;   NUMBER ERROR
DIGI2:  MOV    X0, #0 ;FALSE FLAG
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   ~MATCH   *
;  **************
;
        ALIGN    8
N_NMATCH:
        DQ      6
        DB      "~MATCH"
        ALIGN    8
NMATCH:
        DQ    DOCOL
        DQ    NMATCH+HEADSIZE
        DQ    0x0
        DQ    DIGIT
        DQ    N_NMATCH
        DQ    0

;
        DQ      TOR                           ; Remember dea
        DQ      OVER
        DQ      RR, TNFA, FETCH, SFET
        DQ      CORA
        DQ      FROMR, SWAP                   ; Leave flag on top of dea.
        DQ      SEMIS                            ;
;

;  **************
;  *   ?BLANK   *
;  **************
;
        ALIGN    8
N_QBL:
        DQ      6
        DB      "?BLANK"
        ALIGN    8
QBL:
        DQ    DOCOL
        DQ    QBL+HEADSIZE
        DQ    0x0
        DQ    NMATCH
        DQ    N_QBL
        DQ    0

        DQ      LBL, ONEP, LESS
        DQ      SEMIS
;

;  ************
;  *   PP@@   *
;  ************
;
        ALIGN    8
N_PPFET:
        DQ      4
        DB      "PP@@"
        ALIGN    8
PPFET:
        DQ    DOCOL
        DQ    PPFET+HEADSIZE
        DQ    0x0
        DQ    QBL
        DQ    N_PPFET
        DQ    0

        DQ      SRC, CELLP, TFET
        DQ      OVER, EQUAL
        DQ      ZBRAN
        DQ      PPFET1-$-CW
        DQ      ZERO
        DQ      BRAN
        DQ      PPFET2-$-CW
PPFET1:
        DQ      LDUP
        
        DQ      CFET
        DQ      ONE, PP, PSTOR
PPFET2:
        DQ      SEMIS
;

;  ************
;  *   NAME   *
;  ************
;
        ALIGN    8
N_LNAME:
        DQ      4
        DB      "NAME"
        ALIGN    8
LNAME:
        DQ    DOCOL
        DQ    LNAME+HEADSIZE
        DQ    0x0
        DQ    PPFET
        DQ    N_LNAME
        DQ    0

        DQ      X
PWORD0: DQ      DROP
        DQ      PPFET, QBL
        DQ      OVER, SRC, CELLP, FETCH, LSUB ; At end?
        DQ      LAND, ZEQU
        DQ      ZBRAN
        DQ      PWORD0-$-CW

        DQ      X
PWORD1: DQ      DROP
        DQ      PPFET, QBL
        DQ      ZBRAN
        DQ      PWORD1-$-CW

        DQ      OVER, LSUB
        
        DQ      SEMIS
;

;  *************
;  *   PARSE   *
;  *************
;
        ALIGN    8
N_PARSE:
        DQ      5
        DB      "PARSE"
        ALIGN    8
PARSE:
        DQ    DOCOL
        DQ    PARSE+HEADSIZE
        DQ    0x0
        DQ    LNAME
        DQ    N_PARSE
        DQ    0

        DQ      SRC, CELLP, TFET
        DQ      OVER, LSUB
        
        DQ      ROT, SSLAS, TSWAP
        DQ      ZEQU
        DQ      ZBRAN
        DQ      PARSE8-$-CW
        DQ      DROP, SRC, CELLP, FETCH
PARSE8: 
        DQ      PP, STOR
        DQ SEMIS
;

;  ***********
;  *   SRC   *
;  ***********
;
        ALIGN    8
N_SRC:
        DQ      3
        DB      "SRC"
        ALIGN    8
SRC:
        DQ    DOUSE
        DQ    (CW*(27))
        DQ    0x0
        DQ    PARSE
        DQ    N_SRC
        DQ    0
      ; And 28 and 29.
;

;  **************
;  *   SOURCE   *
;  **************
;
        ALIGN    8
N_SOURCE:
        DQ      6
        DB      "SOURCE"
        ALIGN    8
SOURCE:
        DQ    DOCOL
        DQ    SOURCE+HEADSIZE
        DQ    0x0
        DQ    SRC
        DQ    N_SOURCE
        DQ    0

        DQ      SRC, TFET
        DQ      SWAP, OVER, LSUB
        DQ      SEMIS
;

;

;  **********
;  *   CR   *
;  **********
;
        ALIGN    8
N_CR:
        DQ      2
        DB      "CR"
        ALIGN    8
CR:
        DQ    DOCOL
        DQ    CR+HEADSIZE
        DQ    0x0
        DQ    SOURCE
        DQ    N_CR
        DQ    0

        DQ      LIT,ALF
        DQ      EMIT
        DQ      ZERO, LOUT, STOR
        DQ      SEMIS
;

;  *************
;  *   CMOVE   *
;  *************
;
        ALIGN    8
N_LCMOVE:
        DQ      5
        DB      "CMOVE"
        ALIGN    8
LCMOVE:
        DQ    LCMOVE+HEADSIZE
        DQ    LCMOVE+HEADSIZE
        DQ    0x0
        DQ    CR
        DQ    N_LCMOVE
        DQ    0

        MOV     X8,X11   ;save 
        LDR     X2, [X13],#+(CW*(1))      ;count
        LDR     X1, [X13],#+(CW*(1))      ;dest
        LDR     X3, [X13],#+(CW*(1))      ;source
        CMP X2, #0
        BEQ MOVRP2
MOVREP: LDRB    W0, [X3],#+1
        STRB    W0, [X1],#+1
        SUBS    X2,X2, #1
        BNE     MOVREP
MOVRP2: MOV     X11,X8   ;get back 
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   MOVE   *
;  ************
;
        ALIGN    8
N_LMOVE:
        DQ      4
        DB      "MOVE"
        ALIGN    8
LMOVE:
        DQ    LMOVE+HEADSIZE
        DQ    LMOVE+HEADSIZE
        DQ    0x0
        DQ    LCMOVE
        DQ    N_LMOVE
        DQ    0

        LDR     X2, [X13],#+(CW*(1))      ;count,  R2
        LDR     X1, [X13],#+(CW*(1))      ;dest,   R1
        LDR     X3, [X13],#+(CW*(1))      ;source, R3
        ANDS    X3,X3,X3   ; test R3,R1,R2 for 0 
        BEQ     MOVE3
        ANDS    X1,X1,X1
        BEQ     MOVE3
        ANDS    X2,X2,X2
        BEQ     MOVE3
        CMP     X3,X1
        BHI     MOVE2
        ADD     X1,X1,X2
        ADD     X3,X3,X2
        SUB     X1,X1, #1
        SUB     X3,X3, #1
MOVE1:
        LDRB    W0, [X3],#-1
        STRB    W0, [X1],#-1
        SUBS    X2,X2, #1
        BNE     MOVE1
        B     MOVE3
MOVE2:
        LDRB    W0, [X3],#+1
        STRB    W0, [X1],#+1
        SUBS    X2,X2, #1
        BNE     MOVE2
MOVE3:
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   UM*   *
;  ***********
;
        ALIGN    8
N_USTAR:
        DQ      3
        DB      "UM*"
        ALIGN    8
USTAR:
        DQ    USTAR+HEADSIZE
        DQ    USTAR+HEADSIZE
        DQ    0x0
        DQ    LMOVE
        DQ    N_USTAR
        DQ    0

        LDR     X2, [X13],#+(CW*(1))
        LDR     X3, [X13],#+(CW*(1))
 ; end thumb  )
        MUL      X1,X3,X2          ;Signed *, 64 bits result 
        UMULH    X0,X3,X2          ;Signed *, 64 bits result 
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ;STORE DOUBLE CELL
;

;  **************
;  *   UM/MOD   *
;  **************
;
        ALIGN    8
N_USLAS:
        DQ      6
        DB      "UM/MOD"
        ALIGN    8
USLAS:
        DQ    USLAS+HEADSIZE
        DQ    USLAS+HEADSIZE
        DQ    0x0
        DQ    USTAR
        DQ    N_USLAS
        DQ    0

        LDR     X2, [X13],#+(CW*(1))      ;DIVISOR
        LDR     X1, [X13],#+(CW*(1))     ;MSW OF DIVIDEND
        LDR     X0, [X13],#+(CW*(1))      ;LSW OF DIVIDEND
        BL UMDIV
        STR     X1, [X13,#-(CW*(1))]!    ;remainder
        STR     X4, [X13,#-(CW*(1))]!     ;quotient
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  

_COMMENTED(
_BITS_32{(  division of unsigned-64/unsigned-32})
_BITS_64{(  division of unsigned-128/unsigned-64})
  input:
  div  = R2  = divisor
  modh = R1 = MSW of dividend
  modl = R0  = LSW of dividend
  output:
  quot = R0  = quotient
  rem = R1 = remainder

  Internally R3 is used as a mask, with 1 bit set.
  mask = R3

) 
UMDIV:  STR     X30, [X13,#-(CW*(1))]!
        MOV X3, #1
        LSL X3,X3, #(CW*8-1)  ; init mask with highest bit set
        MOV X4, #0              ; init quot
        CMP X1, X2             ; test modh - div
        BLO UMDIV1              ; modh < div
        ; overflow condition (divide by zero) - show max numbers
        ASR  X4, X3, #(CW*8-1) ; all bits set
        MOV  X1, X4

; 
        B  UMDIV3             ; return

UMDIV1:
        ADDS X0,X0, X0         ;   double precision shift (modh, modl)
        ADCS X1,X1, X1      ;   ADD with carry and set flags again !
        BCS  UMDIV4
        CMP  X2, X1            ; test div - modh
        BHI  UMDIV2             ; div >  modh ?
UMDIV4:
        ADD  X4,X4, X3         ; add single pecision mask
        SUB  X1,X1, X2       ; subtract single precision div
UMDIV2:
        LSR X3,X3, #1          ; shift mask one bit to the right
        ANDS  X3,X3,X3
        BNE  UMDIV1
UMDIV3:
        ; now R0 and R1 contain quotient and remainder
                        LDR     X2, [X13],#+(CW*(1))
        BR X2
; 

;

;  ***********
;  *   AND   *
;  ***********
;
        ALIGN    8
N_LAND:
        DQ      3
        DB      "AND"
        ALIGN    8
LAND:
        DQ    LAND+HEADSIZE
        DQ    LAND+HEADSIZE
        DQ    0x0
        DQ    USLAS
        DQ    N_LAND
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X3, [X13],#+(CW*(1))
        AND     X0,X0,X3
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   OR   *
;  **********
;
        ALIGN    8
N_LOR:
        DQ      2
        DB      "OR"
        ALIGN    8
LOR:
        DQ    LOR+HEADSIZE
        DQ    LOR+HEADSIZE
        DQ    0x0
        DQ    LAND
        DQ    N_LOR
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; (S1) <- (S1) OR (S2)
        LDR     X3, [X13],#+(CW*(1))
        ORR      X0,X0,X3
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   XOR   *
;  ***********
;
        ALIGN    8
N_LXOR:
        DQ      3
        DB      "XOR"
        ALIGN    8
LXOR:
        DQ    LXOR+HEADSIZE
        DQ    LXOR+HEADSIZE
        DQ    0x0
        DQ    LOR
        DQ    N_LXOR
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; (S1) <- (S1) XOR (S2)
        LDR     X3, [X13],#+(CW*(1))
        EOR     X0,X0,X3
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   INVERT   *
;  **************
;
        ALIGN    8
N_INVERT:
        DQ      6
        DB      "INVERT"
        ALIGN    8
INVERT:
        DQ    INVERT+HEADSIZE
        DQ    INVERT+HEADSIZE
        DQ    0x0
        DQ    LXOR
        DQ    N_INVERT
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; (S1) <- (S1) XOR (S2)
        MOV X1,#0
        SUB X1,X1,#1
        EOR X0,X0,X1
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   DSP@   *
;  ************
;
        ALIGN    8
N_SPFET:
        DQ      4
        DB      "DSP@"
        ALIGN    8
SPFET:
        DQ    SPFET+HEADSIZE
        DQ    SPFET+HEADSIZE
        DQ    0x0
        DQ    INVERT
        DQ    N_SPFET
        DQ    0

        MOV     X0,X13   ; (S1) <- (SP)
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   DSP!   *
;  ************
;
        ALIGN    8
N_SPSTO:
        DQ      4
        DB      "DSP!"
        ALIGN    8
SPSTO:
        DQ    SPSTO+HEADSIZE
        DQ    SPSTO+HEADSIZE
        DQ    0x0
        DQ    SPFET
        DQ    N_SPSTO
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        MOV     X13,X0        ;RESET PARAM STACK POINTER
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   DEPTH   *
;  *************
;
        ALIGN    8
N_DEPTH:
        DQ      5
        DB      "DEPTH"
        ALIGN    8
DEPTH:
        DQ    DOCOL
        DQ    DEPTH+HEADSIZE
        DQ    0x0
        DQ    SPSTO
        DQ    N_DEPTH
        DQ    0

        DQ      SZERO, FETCH
        DQ      SPFET
        DQ      LSUB
        DQ      LIT, CW, SLASH
        DQ      ONEM
        DQ      SEMIS
;

;  ***********
;  *   CLS   *
;  ***********
;
        ALIGN    8
N_CLS:
        DQ      3
        DB      "CLS"
        ALIGN    8
CLS:
        DQ    DOCOL
        DQ    CLS+HEADSIZE
        DQ    0x0
        DQ    DEPTH
        DQ    N_CLS
        DQ    0

        DQ      SZERO, FETCH
        DQ      SPSTO
        DQ      SEMIS
;

;  ************
;  *   RSP@   *
;  ************
;
        ALIGN    8
N_RPFET:
        DQ      4
        DB      "RSP@"
        ALIGN    8
RPFET:
        DQ    RPFET+HEADSIZE
        DQ    RPFET+HEADSIZE
        DQ    0x0
        DQ    CLS
        DQ    N_RPFET
        DQ    0
      ;(S1) <- (RP)
        STR     X12, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   RSP!   *
;  ************
;
        ALIGN    8
N_RPSTO:
        DQ      4
        DB      "RSP!"
        ALIGN    8
RPSTO:
        DQ    RPSTO+HEADSIZE
        DQ    RPSTO+HEADSIZE
        DQ    0x0
        DQ    RPFET
        DQ    N_RPSTO
        DQ    0

        LDR     X12, [X13],#+(CW*(1))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   EXIT   *
;  ************
;
        ALIGN    8
N_EXIT:
        DQ      4
        DB      "EXIT"
        ALIGN    8
EXIT:
        DQ    EXIT+HEADSIZE
        DQ    EXIT+HEADSIZE
        DQ    0x0
        DQ    RPSTO
        DQ    N_EXIT
        DQ    0

        LDR     X11, [X12],#+(CW*(1))  ;(IP) <- (RP1)
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   CO   *
;  **********
;
        ALIGN    8
N_CO:
        DQ      2
        DB      "CO"
        ALIGN    8
CO:
        DQ    CO+HEADSIZE
        DQ    CO+HEADSIZE
        DQ    0x0
        DQ    EXIT
        DQ    N_CO
        DQ    0

        LDR     X1,[X12]
        STR     X11, [X12]
        MOV     X11, X1
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   (;)   *
;  ***********
;
        ALIGN    8
N_SEMIS:
        DQ      3
        DB      "(;)"
        ALIGN    8
SEMIS:
        DQ    (EXIT+HEADSIZE)
        DQ    SEMIS+HEADSIZE
        DQ    0x0
        DQ    CO
        DQ    N_SEMIS
        DQ    0

;

;  *************
;  *   LEAVE   *
;  *************
;
        ALIGN    8
N_LLEAV:
        DQ      5
        DB      "LEAVE"
        ALIGN    8
LLEAV:
        DQ    DOCOL
        DQ    LLEAV+HEADSIZE
        DQ    0x0
        DQ    SEMIS
        DQ    N_LLEAV
        DQ    0
  ;LIMIT <- INDEX
        DQ      UNLOOP
        DQ      SEMIS
;

;  **********
;  *   >R   *
;  **********
;
        ALIGN    8
N_TOR:
        DQ      2
        DB      ">R"
        ALIGN    8
TOR:
        DQ    TOR+HEADSIZE
        DQ    TOR+HEADSIZE
        DQ    0x0
        DQ    LLEAV
        DQ    N_TOR
        DQ    0
        ; (RP1) <- (S1)
        LDR     X3, [X13],#+(CW*(1))      ;GET STACK PARAMETER
        SUB     X12, X12, #(CW*(1))  ;MOVE RETURN STACK DOWN
        STR     X3, [X12]            ;ADD TO RETURN STACK
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   R>   *
;  **********
;
        ALIGN    8
N_FROMR:
        DQ      2
        DB      "R>"
        ALIGN    8
FROMR:
        DQ    FROMR+HEADSIZE
        DQ    FROMR+HEADSIZE
        DQ    0x0
        DQ    TOR
        DQ    N_FROMR
        DQ    0
      ;(S1) <- (RP1)
        LDR     X0,[X12] ; GET RETURN STACK VALUE
        ADD     X12,X12,# (CW*(1))
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   RDROP   *
;  *************
;
        ALIGN    8
N_RDROP:
        DQ      5
        DB      "RDROP"
        ALIGN    8
RDROP:
        DQ    RDROP+HEADSIZE
        DQ    RDROP+HEADSIZE
        DQ    0x0
        DQ    FROMR
        DQ    N_RDROP
        DQ    0
      ;(S1) <- (RP1)
        ADD X12,X12,#(CW*(1))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   R@   *
;  **********
;
        ALIGN    8
N_RR:
        DQ      2
        DB      "R@"
        ALIGN    8
RR:
        DQ    (IDO+HEADSIZE)
        DQ    RR+HEADSIZE
        DQ    0x0
        DQ    RDROP
        DQ    N_RR
        DQ    0

;

;  **********
;  *   0=   *
;  **********
;
        ALIGN    8
N_ZEQU:
        DQ      2
        DB      "0="
        ALIGN    8
ZEQU:
        DQ    ZEQU+HEADSIZE
        DQ    ZEQU+HEADSIZE
        DQ    0x0
        DQ    RR
        DQ    N_ZEQU
        DQ    0

        LDR     X2, [X13],#+(CW*(1))
        MOV    X0, #0   ;FALSE
        CMP     X2, #0
        BNE ZEQ1
        SUB     X0,X0, #1          ;
ZEQ1:   STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   0<   *
;  **********
;
        ALIGN    8
N_ZLESS:
        DQ      2
        DB      "0<"
        ALIGN    8
ZLESS:
        DQ    ZLESS+HEADSIZE
        DQ    ZLESS+HEADSIZE
        DQ    0x0
        DQ    ZEQU
        DQ    N_ZLESS
        DQ    0

        LDR     X1, [X13],#+(CW*(1))
        MOV    X0,#0    ;FALSE
        ANDS    X1,X1,X1   ;SET FLAGS
        BPL     ZLESS1
        SUB     X0,X0, #1      ;TRUE
ZLESS1: STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   +   *
;  *********
;
        ALIGN    8
N_PLUS:
        DQ      1
        DB      "+"
        ALIGN    8
PLUS:
        DQ    PLUS+HEADSIZE
        DQ    PLUS+HEADSIZE
        DQ    0x0
        DQ    ZLESS
        DQ    N_PLUS
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ;(S1) <- (S1) + (S2)
        LDR     X3, [X13],#+(CW*(1))
        ADD     X0,X0,X3
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   D+   *
;  **********
;
        ALIGN    8
N_DPLUS:
        DQ      2
        DB      "D+"
        ALIGN    8
DPLUS:
        DQ    DPLUS+HEADSIZE
        DQ    DPLUS+HEADSIZE
        DQ    0x0
        DQ    PLUS
        DQ    N_DPLUS
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; YHC
        LDR     X1, [X13],#+(CW*(1))      ; YLC
        LDR     X3, [X13],#+(CW*(1))      ; XHC
        LDR     X2, [X13],#+(CW*(1))      ; XLC
        ADDS    X1,X1,X2
        ADCS    X0,X0,X3   ; SLC
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   NEGATE   *
;  **************
;
        ALIGN    8
N_NEGATE:
        DQ      6
        DB      "NEGATE"
        ALIGN    8
NEGATE:
        DQ    NEGATE+HEADSIZE
        DQ    NEGATE+HEADSIZE
        DQ    0x0
        DQ    DPLUS
        DQ    N_NEGATE
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        MVN X0, X0  ; bitwise NOT 
        ADD X0,X0,#1
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***************
;  *   DNEGATE   *
;  ***************
;
        ALIGN    8
N_DNEGA:
        DQ      7
        DB      "DNEGATE"
        ALIGN    8
DNEGA:
        DQ    DNEGA+HEADSIZE
        DQ    DNEGA+HEADSIZE
        DQ    0x0
        DQ    NEGATE
        DQ    N_DNEGA
        DQ    0

        LDR     X0, [X13],#+(CW*(1))   ; MSW
        LDR     X1, [X13],#+(CW*(1))   ; LSW
        MVN X0, X0  ; invert bits 
        MVN X1, X1
        
        ADDS X1, X1, #1  ; make 2's complement 
        ADC X0, X0, XZR  ; add possible carry    
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
        ;
;

;  ************
;  *   OVER   *
;  ************
;
        ALIGN    8
N_OVER:
        DQ      4
        DB      "OVER"
        ALIGN    8
OVER:
        DQ    OVER+HEADSIZE
        DQ    OVER+HEADSIZE
        DQ    0x0
        DQ    DNEGA
        DQ    N_OVER
        DQ    0

        LDR     X1, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        STR     X0, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   DROP   *
;  ************
;
        ALIGN    8
N_DROP:
        DQ      4
        DB      "DROP"
        ALIGN    8
DROP:
        DQ    DROP+HEADSIZE
        DQ    DROP+HEADSIZE
        DQ    0x0
        DQ    OVER
        DQ    N_DROP
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   NIP   *
;  ***********
;
        ALIGN    8
N_NIP:
        DQ      3
        DB      "NIP"
        ALIGN    8
NIP:
        DQ    NIP+HEADSIZE
        DQ    NIP+HEADSIZE
        DQ    0x0
        DQ    DROP
        DQ    N_NIP
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X3, [X13],#+(CW*(1))
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   2DROP   *
;  *************
;
        ALIGN    8
N_TDROP:
        DQ      5
        DB      "2DROP"
        ALIGN    8
TDROP:
        DQ    TDROP+HEADSIZE
        DQ    TDROP+HEADSIZE
        DQ    0x0
        DQ    NIP
        DQ    N_TDROP
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   SWAP   *
;  ************
;
        ALIGN    8
N_SWAP:
        DQ      4
        DB      "SWAP"
        ALIGN    8
SWAP:
        DQ    SWAP+HEADSIZE
        DQ    SWAP+HEADSIZE
        DQ    0x0
        DQ    TDROP
        DQ    N_SWAP
        DQ    0

        LDR     X1, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   DUP   *
;  ***********
;
        ALIGN    8
N_LDUP:
        DQ      3
        DB      "DUP"
        ALIGN    8
LDUP:
        DQ    LDUP+HEADSIZE
        DQ    LDUP+HEADSIZE
        DQ    0x0
        DQ    SWAP
        DQ    N_LDUP
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        STR     X0, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   2DUP   *
;  ************
;
        ALIGN    8
N_TDUP:
        DQ      4
        DB      "2DUP"
        ALIGN    8
TDUP:
        DQ    TDUP+HEADSIZE
        DQ    TDUP+HEADSIZE
        DQ    0x0
        DQ    LDUP
        DQ    N_TDUP
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X1, [X13],#+(CW*(1))
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   2SWAP   *
;  *************
;
        ALIGN    8
N_TSWAP:
        DQ      5
        DB      "2SWAP"
        ALIGN    8
TSWAP:
        DQ    TSWAP+HEADSIZE
        DQ    TSWAP+HEADSIZE
        DQ    0x0
        DQ    TDUP
        DQ    N_TSWAP
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X2, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LDR     X1, [X13],#+(CW*(1))
        STR     X2, [X13,#-(CW*(1))]!
        STR     X3, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   2OVER   *
;  *************
;
        ALIGN    8
N_TOVER:
        DQ      5
        DB      "2OVER"
        ALIGN    8
TOVER:
        DQ    TOVER+HEADSIZE
        DQ    TOVER+HEADSIZE
        DQ    0x0
        DQ    TSWAP
        DQ    N_TOVER
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X2, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LDR     X1, [X13],#+(CW*(1))
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        STR     X2, [X13,#-(CW*(1))]!
        STR     X3, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   +!   *
;  **********
;
        ALIGN    8
N_PSTOR:
        DQ      2
        DB      "+!"
        ALIGN    8
PSTOR:
        DQ    PSTOR+HEADSIZE
        DQ    PSTOR+HEADSIZE
        DQ    0x0
        DQ    TOVER
        DQ    N_PSTOR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDRESS
        LDR     X0, [X13],#+(CW*(1))      ;INCREMENT
        LDR     X2, [X3]
        ADD     X2,X2, X0
        STR     X2, [X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   TOGGLE   *
;  **************
;
        ALIGN    8
N_TOGGL:
        DQ      6
        DB      "TOGGLE"
        ALIGN    8
TOGGL:
        DQ    TOGGL+HEADSIZE
        DQ    TOGGL+HEADSIZE
        DQ    0x0
        DQ    PSTOR
        DQ    N_TOGGL
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ;BIT PATTERN
        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X1, [X3]
        EOR     X1,X1, X0
        STR     X1, [X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   @   *
;  *********
;
        ALIGN    8
N_FETCH:
        DQ      1
        DB      "@"
        ALIGN    8
FETCH:
        DQ    FETCH+HEADSIZE
        DQ    FETCH+HEADSIZE
        DQ    0x0
        DQ    TOGGL
        DQ    N_FETCH
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X0,[X3]
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   P@   *
;  **********
;
        ALIGN    8
N_PFETCH:
        DQ      2
        DB      "P@"
        ALIGN    8
PFETCH:
        DQ    PFETCH+HEADSIZE
        DQ    PFETCH+HEADSIZE
        DQ    0x0
        DQ    FETCH
        DQ    N_PFETCH
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X0,[X3]
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;


;  **********
;  *   L@   *
;  **********
;
        ALIGN    8
N_LFETCH:
        DQ      2
        DB      "L@"
        ALIGN    8
LFETCH:
        DQ    LFETCH+HEADSIZE
        DQ    LFETCH+HEADSIZE
        DQ    0x0
        DQ    PFETCH
        DQ    N_LFETCH
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        EOR     X0,X0,X0
        LDR     W0,[X3]
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
;
;


;  ***********
;  *   BIC   *
;  ***********
;
        ALIGN    8
N_BITCLR:
        DQ      3
        DB      "BIC"
        ALIGN    8
BITCLR:
        DQ    BITCLR+HEADSIZE
        DQ    BITCLR+HEADSIZE
        DQ    0x0
        DQ    LFETCH
        DQ    N_BITCLR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LDR     W2,[X3]
        BIC     X2,X2,X0
        STR     W2,[X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   BIS   *
;  ***********
;
        ALIGN    8
N_BITSET:
        DQ      3
        DB      "BIS"
        ALIGN    8
BITSET:
        DQ    BITSET+HEADSIZE
        DQ    BITSET+HEADSIZE
        DQ    0x0
        DQ    BITCLR
        DQ    N_BITSET
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LDR     W2,[X3]
        ORR     X2,X2,X0
        STR     W2,[X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
;

;  **********
;  *   C@   *
;  **********
;
        ALIGN    8
N_CFET:
        DQ      2
        DB      "C@"
        ALIGN    8
CFET:
        DQ    CFET+HEADSIZE
        DQ    CFET+HEADSIZE
        DQ    0x0
        DQ    BITSET
        DQ    N_CFET
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDRB    W0,[X3]
        
         AND     X0,X0, #0xFF
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   2@   *
;  **********
;
        ALIGN    8
N_TFET:
        DQ      2
        DB      "2@"
        ALIGN    8
TFET:
        DQ    TFET+HEADSIZE
        DQ    TFET+HEADSIZE
        DQ    0x0
        DQ    CFET
        DQ    N_TFET
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0,[X3]                      ;MSW
        LDR     X1,[X3,# + (CW*(1))]        ;LSW
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   !   *
;  *********
;
        ALIGN    8
N_STOR:
        DQ      1
        DB      "!"
        ALIGN    8
STOR:
        DQ    STOR+HEADSIZE
        DQ    STOR+HEADSIZE
        DQ    0x0
        DQ    TFET
        DQ    N_STOR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
        STR     X0, [X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   P!   *
;  **********
;
        ALIGN    8
N_PRTSTR:
        DQ      2
        DB      "P!"
        ALIGN    8
PRTSTR:
        DQ    PRTSTR+HEADSIZE
        DQ    PRTSTR+HEADSIZE
        DQ    0x0
        DQ    STOR
        DQ    N_PRTSTR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
        STR     X0, [X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;


;  **********
;  *   L!   *
;  **********
;
        ALIGN    8
N_LSTOR:
        DQ      2
        DB      "L!"
        ALIGN    8
LSTOR:
        DQ    LSTOR+HEADSIZE
        DQ    LSTOR+HEADSIZE
        DQ    0x0
        DQ    PRTSTR
        DQ    N_LSTOR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
        STR     W0, [X3]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
;

;  **********
;  *   C!   *
;  **********
;
        ALIGN    8
N_CSTOR:
        DQ      2
        DB      "C!"
        ALIGN    8
CSTOR:
        DQ    CSTOR+HEADSIZE
        DQ    CSTOR+HEADSIZE
        DQ    0x0
        DQ    LSTOR
        DQ    N_CSTOR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
        STRB         W0, [X3]  ;Store byte
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   2!   *
;  **********
;
        ALIGN    8
N_TSTOR:
        DQ      2
        DB      "2!"
        ALIGN    8
TSTOR:
        DQ    TSTOR+HEADSIZE
        DQ    TSTOR+HEADSIZE
        DQ    0x0
        DQ    CSTOR
        DQ    N_TSTOR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))      ;ADDR
        LDR     X0, [X13],#+(CW*(1))      ;MSW
        STR     X0, [X3]
        LDR     X0, [X13],#+(CW*(1))      ;LSW
        STR     X0, [X3, # +(CW*(1))]
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   WITHIN   *
;  **************
;
        ALIGN    8
N_WITHIN:
        DQ      6
        DB      "WITHIN"
        ALIGN    8
WITHIN:
        DQ    DOCOL
        DQ    WITHIN+HEADSIZE
        DQ    0x0
        DQ    TSTOR
        DQ    N_WITHIN
        DQ    0

        DQ      OVER, LSUB, TOR
        DQ      LSUB, FROMR
        DQ      ULESS
        DQ      SEMIS
;
;

;  *********
;  *   :   *
;  *********
;
        ALIGN    8
N_COLON:
        DQ      1
        DB      ":"
        ALIGN    8
COLON:
        DQ    DOCOL
        DQ    COLON+HEADSIZE
        DQ    0x0
        DQ    WITHIN
        DQ    N_COLON
        DQ    0

        DQ      SCSP
        DQ      LNAME
        DQ      PCREAT
        DQ      LATEST, HIDDEN
        DQ      RBRAC
        DQ      PSCOD
DOCOL:  STR     X11, [X12,#-(CW*(1))]!
        LDR     X11,[X0, #+(CW*(D_HOFFSET))]  ;(IP) <- (PFA)

; 

        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  


;

;  *********
;  *   ;   *
;  *********
;
        ALIGN    8
N_SEMI:
        DQ      1
        DB      ";"
        ALIGN    8
SEMI:
        DQ    DOCOL
        DQ    SEMI+HEADSIZE
        DQ    B_IMMED
        DQ    COLON
        DQ    N_SEMI
        DQ    0

        DQ      QCSP
        DQ      LIT, SEMIS, COMMA
        DQ      LATEST, HIDDEN
        DQ      LBRAC
        DQ      SEMIS
;

;  ****************
;  *   CONSTANT   *
;  ****************
;
        ALIGN    8
N_LCONST:
        DQ      8
        DB      "CONSTANT"
        ALIGN    8
LCONST:
        DQ    DOCOL
        DQ    LCONST+HEADSIZE
        DQ    0x0
        DQ    SEMI
        DQ    N_LCONST
        DQ    0

        DQ      LNAME
        DQ      PCREAT
        DQ      LATEST, TDFA, STOR
        DQ      PSCOD
DOCON:  LDR     X0,[X0,#+(CW*((D_HOFFSET)))] ;GET DATA FROM PFA
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ****************
;  *   VARIABLE   *
;  ****************
;
        ALIGN    8
N_VAR:
        DQ      8
        DB      "VARIABLE"
        ALIGN    8
VAR:
        DQ    DOCOL
        DQ    VAR+HEADSIZE
        DQ    0x0
        DQ    LCONST
        DQ    N_VAR
        DQ    0

        DQ      LDATA
        DQ      ZERO, COMMA
        DQ      SEMIS
;

;  ************
;  *   DATA   *
;  ************
;
        ALIGN    8
N_LDATA:
        DQ      4
        DB      "DATA"
        ALIGN    8
LDATA:
        DQ    DOCOL
        DQ    LDATA+HEADSIZE
        DQ    0x0
        DQ    VAR
        DQ    N_LDATA
        DQ    0

        DQ      LNAME
        DQ      PCREAT
        DQ      PSCOD
DOVAR:  LDR     X0,[X0,#+(CW*((D_HOFFSET)))]  ;(R0) <- PFA
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   _   *
;  *********
;
        ALIGN    8
N_X:
        DQ      1
        DB      "_"
        ALIGN    8
X:
        DQ    X+HEADSIZE
        DQ    X+HEADSIZE
        DQ    0x0
        DQ    LDATA
        DQ    N_X
        DQ    0

        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1   ;Whatever happens to be in X0, i.e. the dea of ``_''.
;

;  *********
;  *   0   *
;  *********
;
        ALIGN    8
N_ZERO:
        DQ      1
        DB      "0"
        ALIGN    8
ZERO:
        DQ    DOCON
        DQ    0
        DQ    0x0
        DQ    X
        DQ    N_ZERO
        DQ    0

; The data field of 0 is used for an empty string.
;

;  *********
;  *   1   *
;  *********
;
        ALIGN    8
N_ONE:
        DQ      1
        DB      "1"
        ALIGN    8
ONE:
        DQ    DOCON
        DQ    1
        DQ    0x0
        DQ    ZERO
        DQ    N_ONE
        DQ    0

;

;  *********
;  *   2   *
;  *********
;
        ALIGN    8
N_TWO:
        DQ      1
        DB      "2"
        ALIGN    8
TWO:
        DQ    DOCON
        DQ    2
        DQ    0x0
        DQ    ONE
        DQ    N_TWO
        DQ    0

;

;  **********
;  *   BL   *
;  **********
;
        ALIGN    8
N_LBL:
        DQ      2
        DB      "BL"
        ALIGN    8
LBL:
        DQ    DOCON
        DQ    ABL
        DQ    0x0
        DQ    TWO
        DQ    N_LBL
        DQ    0

;

;
;  **********
;  *   $@   *
;  **********
;
        ALIGN    8
N_SFET:
        DQ      2
        DB      "$@"
        ALIGN    8
SFET:
        DQ    SFET+HEADSIZE
        DQ    SFET+HEADSIZE
        DQ    0x0
        DQ    LBL
        DQ    N_SFET
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X0, [X3],#+(CW*(1))
        MOV   X1,X3
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   $!   *
;  **********
;
        ALIGN    8
N_SSTOR:
        DQ      2
        DB      "$!"
        ALIGN    8
SSTOR:
        DQ    DOCOL
        DQ    SSTOR+HEADSIZE
        DQ    0x0
        DQ    SFET
        DQ    N_SSTOR
        DQ    0

        DQ TDUP, STOR, CELLP, SWAP, LMOVE
        DQ SEMIS
;

; 

;  ***********
;  *   $+!   *
;  ***********
;
        ALIGN    8
N_SADD:
        DQ      3
        DB      "$+!"
        ALIGN    8
SADD:
        DQ    DOCOL
        DQ    SADD+HEADSIZE
        DQ    0x0
        DQ    SSTOR
        DQ    N_SADD
        DQ    0

        DQ   LDUP, FETCH, TOR ; Remember old count.
        DQ   TDUP, PSTOR
        DQ   CELLP, FROMR, PLUS, SWAP, LMOVE
        DQ SEMIS
;

;  ***********
;  *   $C+   *
;  ***********
;
        ALIGN    8
N_CHAPP:
        DQ      3
        DB      "$C+"
        ALIGN    8
CHAPP:
        DQ    DOCOL
        DQ    CHAPP+HEADSIZE
        DQ    0x0
        DQ    SADD
        DQ    N_CHAPP
        DQ    0

        DQ   LDUP, TOR
        DQ   LDUP, FETCH, PLUS, CELLP, CSTOR
        DQ   ONE, FROMR, PSTOR
        DQ SEMIS
;

;  **********
;  *   $,   *
;  **********
;
        ALIGN    8
N_SCOMMA:
        DQ      2
        DB      "$,"
        ALIGN    8
SCOMMA:
        DQ    DOCOL
        DQ    SCOMMA+HEADSIZE
        DQ    0x0
        DQ    CHAPP
        DQ    N_SCOMMA
        DQ    0

        DQ HERE, TOR
        DQ LDUP, CELLP, ALLOT
        DQ RR, SSTOR, FROMR
 DQ LALIGN
        DQ SEMIS
;

;  ***********
;  *   C/L   *
;  ***********
;
        ALIGN    8
N_CSLL:
        DQ      3
        DB      "C/L"
        ALIGN    8
CSLL:
        DQ    DOCON
        DQ    64
        DQ    0x0
        DQ    SCOMMA
        DQ    N_CSLL
        DQ    0

;


;  **************
;  *   _FIRST   *
;  **************
;
        ALIGN    8
N__FIRST:
        DQ      6
        DB      "_FIRST"
        ALIGN    8
_FIRST:
        DQ    DOCON
        DQ    BUF1
        DQ    0x0
        DQ    CSLL
        DQ    N__FIRST
        DQ    0

 ;  
;
; 
;

;  **************
;  *   _LIMIT   *
;  **************
;
        ALIGN    8
N__LIMIT:
        DQ      6
        DB      "_LIMIT"
        ALIGN    8
_LIMIT:
        DQ    DOCON
        DQ    BUF1+(KBBUF+2*CW)*NBUF
        DQ    0x0
        DQ    _FIRST
        DQ    N__LIMIT
        DQ    0

; THE END  OF THE MEMORY 

;  **********
;  *   EM   *
;  **********
;
        ALIGN    8
N_LEM:
        DQ      2
        DB      "EM"
        ALIGN    8
LEM:
        DQ    DOCON
        DQ    ACTUAL_EM
        DQ    0x0
        DQ    _LIMIT
        DQ    N_LEM
        DQ    0

;

;  **********
;  *   BM   *
;  **********
;
        ALIGN    8
N_LBM:
        DQ      2
        DB      "BM"
        ALIGN    8
LBM:
        DQ    DOCON
        DQ    _ORIG
        DQ    0x0
        DQ    LEM
        DQ    N_LBM
        DQ    0

;

;  *************
;  *   B/BUF   *
;  *************
;
        ALIGN    8
N_BBUF:
        DQ      5
        DB      "B/BUF"
        ALIGN    8
BBUF:
        DQ    DOCON
        DQ    KBBUF
        DQ    0x0
        DQ    LBM
        DQ    N_BBUF
        DQ    0

;
; All user variables are initialised 
; with the values from USINI.
; The implementation relies on the initialisation of 
; those with numbers (1..11), so change in concord with USINI.

;  **********
;  *   U0   *
;  **********
;
        ALIGN    8
N_UZERO:
        DQ      2
        DB      "U0"
        ALIGN    8
UZERO:
        DQ    DOUSE
        DQ    (CW*(1))
        DQ    0x0
        DQ    BBUF
        DQ    N_UZERO
        DQ    0

;

;  **********
;  *   S0   *
;  **********
;
        ALIGN    8
N_SZERO:
        DQ      2
        DB      "S0"
        ALIGN    8
SZERO:
        DQ    DOUSE
        DQ    (CW*(2))
        DQ    0x0
        DQ    UZERO
        DQ    N_SZERO
        DQ    0

;

;  **********
;  *   R0   *
;  **********
;
        ALIGN    8
N_RZERO:
        DQ      2
        DB      "R0"
        ALIGN    8
RZERO:
        DQ    DOUSE
        DQ    (CW*(3))
        DQ    0x0
        DQ    SZERO
        DQ    N_RZERO
        DQ    0

;

;  ***********
;  *   TIB   *
;  ***********
;
        ALIGN    8
N_TIB:
        DQ      3
        DB      "TIB"
        ALIGN    8
TIB:
        DQ    DOUSE
        DQ    (CW*(4))
        DQ    0x0
        DQ    RZERO
        DQ    N_TIB
        DQ    0

;

;  **************
;  *   RUBOUT   *
;  **************
;
        ALIGN    8
N_RUBOUT:
        DQ      6
        DB      "RUBOUT"
        ALIGN    8
RUBOUT:
        DQ    DOUSE
        DQ    (CW*(5))
        DQ    0x0
        DQ    TIB
        DQ    N_RUBOUT
        DQ    0

;

;  ***************
;  *   WARNING   *
;  ***************
;
        ALIGN    8
N_LWARN:
        DQ      7
        DB      "WARNING"
        ALIGN    8
LWARN:
        DQ    DOUSE
        DQ    (CW*(7))
        DQ    0x0
        DQ    RUBOUT
        DQ    N_LWARN
        DQ    0

;

;  *************
;  *   FENCE   *
;  *************
;
        ALIGN    8
N_FENCE:
        DQ      5
        DB      "FENCE"
        ALIGN    8
FENCE:
        DQ    DOUSE
        DQ    (CW*(8))
        DQ    0x0
        DQ    LWARN
        DQ    N_FENCE
        DQ    0

;

;  **********
;  *   DP   *
;  **********
;
        ALIGN    8
N_LDP:
        DQ      2
        DB      "DP"
        ALIGN    8
LDP:
        DQ    DOUSE
        DQ    (CW*(9))
        DQ    0x0
        DQ    FENCE
        DQ    N_LDP
        DQ    0

;

;  ****************
;  *   VOC-LINK   *
;  ****************
;
        ALIGN    8
N_VOCL:
        DQ      8
        DB      "VOC-LINK"
        ALIGN    8
VOCL:
        DQ    DOUSE
        DQ    (CW*(10))
        DQ    0x0
        DQ    LDP
        DQ    N_VOCL
        DQ    0

;

;  **************
;  *   OFFSET   *
;  **************
;
        ALIGN    8
N_LOFFSET:
        DQ      6
        DB      "OFFSET"
        ALIGN    8
LOFFSET:
        DQ    DOUSE
        DQ    (CW*(37))
        DQ    0x0
        DQ    VOCL
        DQ    N_LOFFSET
        DQ    0

;

;  ****************
;  *   MAX-USER   *
;  ****************
;
        ALIGN    8
N_MAXUSER:
        DQ      8
        DB      "MAX-USER"
        ALIGN    8
MAXUSER:
        DQ    DOUSE
        DQ    (CW*(11))
        DQ    0x0
        DQ    LOFFSET
        DQ    N_MAXUSER
        DQ    0

;
; End of user variables with fixed place.
;
;

;  *************
;  *   WHERE   *
;  *************
;
        ALIGN    8
N_LWHERE:
        DQ      5
        DB      "WHERE"
        ALIGN    8
LWHERE:
        DQ    DOUSE
        DQ    (CW*(12))
        DQ    0x0
        DQ    MAXUSER
        DQ    N_LWHERE
        DQ    0
    ;  Occupies two CELLS! 
;

;  ***********
;  *   SCR   *
;  ***********
;
        ALIGN    8
N_SCR:
        DQ      3
        DB      "SCR"
        ALIGN    8
SCR:
        DQ    DOUSE
        DQ    (CW*(33))
        DQ    0x0
        DQ    LWHERE
        DQ    N_SCR
        DQ    0

;

;  *************
;  *   STATE   *
;  *************
;
        ALIGN    8
N_STATE:
        DQ      5
        DB      "STATE"
        ALIGN    8
STATE:
        DQ    DOUSE
        DQ    (CW*(18))
        DQ    0x0
        DQ    SCR
        DQ    N_STATE
        DQ    0

;

;  ************
;  *   BASE   *
;  ************
;
        ALIGN    8
N_BASE:
        DQ      4
        DB      "BASE"
        ALIGN    8
BASE:
        DQ    DOUSE
        DQ    (CW*(19))
        DQ    0x0
        DQ    STATE
        DQ    N_BASE
        DQ    0

;

;  ***********
;  *   DPL   *
;  ***********
;
        ALIGN    8
N_DPL:
        DQ      3
        DB      "DPL"
        ALIGN    8
DPL:
        DQ    DOUSE
        DQ    (CW*(20))
        DQ    0x0
        DQ    BASE
        DQ    N_DPL
        DQ    0

;

;  ***********
;  *   ECL   *
;  ***********
;
        ALIGN    8
N_ECL:
        DQ      3
        DB      "ECL"
        ALIGN    8
ECL:
        DQ    DOUSE
        DQ    (CW*(38))
        DQ    0x0
        DQ    DPL
        DQ    N_ECL
        DQ    0

;

;  ***********
;  *   FLD   *
;  ***********
;
        ALIGN    8
N_LFLD:
        DQ      3
        DB      "FLD"
        ALIGN    8
LFLD:
        DQ    DOUSE
        DQ    (CW*(21))
        DQ    0x0
        DQ    ECL
        DQ    N_LFLD
        DQ    0

;


;  ***********
;  *   CSP   *
;  ***********
;
        ALIGN    8
N_LCSP:
        DQ      3
        DB      "CSP"
        ALIGN    8
LCSP:
        DQ    DOUSE
        DQ    (CW*(22))
        DQ    0x0
        DQ    LFLD
        DQ    N_LCSP
        DQ    0

;
;

;  **********
;  *   R#   *
;  **********
;
        ALIGN    8
N_RNUM:
        DQ      2
        DB      "R#"
        ALIGN    8
RNUM:
        DQ    DOUSE
        DQ    (CW*(23))
        DQ    0x0
        DQ    LCSP
        DQ    N_RNUM
        DQ    0

;

;  ***********
;  *   HLD   *
;  ***********
;
        ALIGN    8
N_HLD:
        DQ      3
        DB      "HLD"
        ALIGN    8
HLD:
        DQ    DOUSE
        DQ    (CW*(24))
        DQ    0x0
        DQ    RNUM
        DQ    N_HLD
        DQ    0

;

;  ***********
;  *   OUT   *
;  ***********
;
        ALIGN    8
N_LOUT:
        DQ      3
        DB      "OUT"
        ALIGN    8
LOUT:
        DQ    DOUSE
        DQ    (CW*(25))
        DQ    0x0
        DQ    HLD
        DQ    N_LOUT
        DQ    0

;

;  *************
;  *   (BLK)   *
;  *************
;
        ALIGN    8
N_PBLK:
        DQ      5
        DB      "(BLK)"
        ALIGN    8
PBLK:
        DQ    DOUSE
        DQ    (CW*(26))
        DQ    0x0
        DQ    LOUT
        DQ    N_PBLK
        DQ    0

;

;  **********
;  *   PP   *
;  **********
;
        ALIGN    8
N_PP:
        DQ      2
        DB      "PP"
        ALIGN    8
PP:
        DQ    DOUSE
        DQ    (CW*(29))
        DQ    0x0
        DQ    PBLK
        DQ    N_PP
        DQ    0

;

;  *************
;  *   (>IN)   *
;  *************
;
        ALIGN    8
N_PIIN:
        DQ      5
        DB      "(>IN)"
        ALIGN    8
PIIN:
        DQ    DOUSE
        DQ    (CW*(30))
        DQ    0x0
        DQ    PP
        DQ    N_PIIN
        DQ    0

;

;  ************
;  *   ARGS   *
;  ************
;
        ALIGN    8
N_ARGS:
        DQ      4
        DB      "ARGS"
        ALIGN    8
ARGS:
        DQ    DOUSE
        DQ    (CW*(31))
        DQ    0x0
        DQ    PIIN
        DQ    N_ARGS
        DQ    0

;

;  ***************
;  *   HANDLER   *
;  ***************
;
        ALIGN    8
N_HANDLER:
        DQ      7
        DB      "HANDLER"
        ALIGN    8
HANDLER:
        DQ    DOUSE
        DQ    (CW*(32))
        DQ    0x0
        DQ    ARGS
        DQ    N_HANDLER
        DQ    0

;

;  ***************
;  *   CURRENT   *
;  ***************
;
        ALIGN    8
N_CURR:
        DQ      7
        DB      "CURRENT"
        ALIGN    8
CURR:
        DQ    DOUSE
        DQ    (CW*(34))
        DQ    0x0
        DQ    HANDLER
        DQ    N_CURR
        DQ    0

;

;  *****************
;  *   REMAINDER   *
;  *****************
;
        ALIGN    8
N_REMAIND:
        DQ      9
        DB      "REMAINDER"
        ALIGN    8
REMAIND:
        DQ    DOUSE
        DQ    (CW*(14))
        DQ    0x0
        DQ    CURR
        DQ    N_REMAIND
        DQ    0

;      IMPORTANT
; REQUIRES ONE MORE CELL!
;

;  ***************
;  *   CONTEXT   *
;  ***************
;
        ALIGN    8
N_CONTEXT:
        DQ      7
        DB      "CONTEXT"
        ALIGN    8
CONTEXT:
        DQ    DOUSE
        DQ    (CW*(39))
        DQ    0x0
        DQ    REMAIND
        DQ    N_CONTEXT
        DQ    0
 ; Up to  39+16
;      IMPORTANT
;     16 USER SPACE CELLS MUST BE KEPT FREE
;     IN ADDITION TO THE ONE FOR CONTEXT
;
;========== END USER VARIABLES =============
;

;  **********
;  *   1+   *
;  **********
;
        ALIGN    8
N_ONEP:
        DQ      2
        DB      "1+"
        ALIGN    8
ONEP:
        DQ    DOCOL
        DQ    ONEP+HEADSIZE
        DQ    0x0
        DQ    CONTEXT
        DQ    N_ONEP
        DQ    0

        DQ      ONE
        DQ      PLUS
        DQ      SEMIS
;

;  *************
;  *   CELL+   *
;  *************
;
        ALIGN    8
N_CELLP:
        DQ      5
        DB      "CELL+"
        ALIGN    8
CELLP:
        DQ    DOCOL
        DQ    CELLP+HEADSIZE
        DQ    0x0
        DQ    ONEP
        DQ    N_CELLP
        DQ    0

        DQ      LIT, CW
        DQ      PLUS
        DQ      SEMIS
;

;
;  *************
;  *   CELLS   *
;  *************
;
        ALIGN    8
N_LCELLS:
        DQ      5
        DB      "CELLS"
        ALIGN    8
LCELLS:
        DQ    DOCOL
        DQ    LCELLS+HEADSIZE
        DQ    0x0
        DQ    CELLP
        DQ    N_LCELLS
        DQ    0

        DQ        LIT, 3
        DQ      LSHIFT
        DQ      SEMIS
;

;  *************
;  *   CHAR+   *
;  *************
;
        ALIGN    8
N_CHARP:
        DQ      5
        DB      "CHAR+"
        ALIGN    8
CHARP:
        DQ    DOCOL
        DQ    (ONEP+HEADSIZE)
        DQ    0x0
        DQ    LCELLS
        DQ    N_CHARP
        DQ    0

;

;  *************
;  *   CHARS   *
;  *************
;
        ALIGN    8
N_CHARS:
        DQ      5
        DB      "CHARS"
        ALIGN    8
CHARS:
        DQ    DOCOL 
        DQ    CHARS+HEADSIZE
        DQ    0x0
        DQ    CHARP
        DQ    N_CHARS
        DQ    0

        DQ      SEMIS
;

;  *************
;  *   ALIGN   *
;  *************
;
        ALIGN    8
N_LALIGN:
        DQ      5
        DB      "ALIGN"
        ALIGN    8
LALIGN:
        DQ    DOCOL 
        DQ    LALIGN+HEADSIZE
        DQ    0x0
        DQ    CHARS
        DQ    N_LALIGN
        DQ    0


        DQ      LDP, FETCH
        DQ      ALIGNED
        DQ      LDP, STOR
;
        DQ      SEMIS
;

;  ***************
;  *   ALIGNED   *
;  ***************
;
        ALIGN    8
N_ALIGNED :
        DQ      7
        DB      "ALIGNED"
        ALIGN    8
ALIGNED :
        DQ    ALIGNED +HEADSIZE
        DQ    ALIGNED +HEADSIZE
        DQ    0x0
        DQ    LALIGN
        DQ    N_ALIGNED 
        DQ    0


        LDR     X0, [X13],#+(CW*(1))
        SUB     X0,X0, #1
        
        ORR     X0, X0, #(CW-1) 
        ADD     X0,X0, #1
        STR     X0, [X13,#-(CW*(1))]!
;
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   HERE   *
;  ************
;
        ALIGN    8
N_HERE:
        DQ      4
        DB      "HERE"
        ALIGN    8
HERE:
        DQ    DOCOL
        DQ    HERE+HEADSIZE
        DQ    0x0
        DQ    ALIGNED 
        DQ    N_HERE
        DQ    0

        DQ      LDP
        DQ      FETCH
        DQ      SEMIS
;

;  *************
;  *   ALLOT   *
;  *************
;
        ALIGN    8
N_ALLOT:
        DQ      5
        DB      "ALLOT"
        ALIGN    8
ALLOT:
        DQ    DOCOL
        DQ    ALLOT+HEADSIZE
        DQ    0x0
        DQ    HERE
        DQ    N_ALLOT
        DQ    0

        DQ      LDP
        DQ      PSTOR
        DQ      SEMIS
;

;  *********
;  *   ,   *
;  *********
;
        ALIGN    8
N_COMMA:
        DQ      1
        DB      ","
        ALIGN    8
COMMA:
        DQ    DOCOL
        DQ    COMMA+HEADSIZE
        DQ    0x0
        DQ    ALLOT
        DQ    N_COMMA
        DQ    0

        DQ      HERE
        DQ      LIT
        DQ      CW
        DQ      ALLOT
        DQ      STOR
        DQ      SEMIS
;

;  **********
;  *   C,   *
;  **********
;
        ALIGN    8
N_CCOMM:
        DQ      2
        DB      "C,"
        ALIGN    8
CCOMM:
        DQ    DOCOL
        DQ    CCOMM+HEADSIZE
        DQ    0x0
        DQ    COMMA
        DQ    N_CCOMM
        DQ    0

        DQ      HERE
        DQ      ONE
        DQ      ALLOT
        DQ      CSTOR
        DQ      SEMIS
;

;  **********
;  *   2,   *
;  **********
;
        ALIGN    8
N_TCOMM:
        DQ      2
        DB      "2,"
        ALIGN    8
TCOMM:
        DQ    DOCOL
        DQ    TCOMM+HEADSIZE
        DQ    0x0
        DQ    CCOMM
        DQ    N_TCOMM
        DQ    0

        DQ      HERE
        DQ      LIT
        DQ      CW*2
        DQ      ALLOT
        DQ      TSTOR
        DQ      SEMIS
;

;  *********
;  *   -   *
;  *********
;
        ALIGN    8
N_LSUB:
        DQ      1
        DB      "-"
        ALIGN    8
LSUB:
        DQ    LSUB+HEADSIZE
        DQ    LSUB+HEADSIZE
        DQ    0x0
        DQ    TCOMM
        DQ    N_LSUB
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;S1
        LDR     X0, [X13],#+(CW*(1))
        SUB     X0,X0,X1
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1     ;S1 = S2 - S1
;

;  *********
;  *   =   *
;  *********
;
        ALIGN    8
N_EQUAL:
        DQ      1
        DB      "="
        ALIGN    8
EQUAL:
        DQ    DOCOL
        DQ    EQUAL+HEADSIZE
        DQ    0x0
        DQ    LSUB
        DQ    N_EQUAL
        DQ    0

        DQ      LSUB
        DQ      ZEQU
        DQ      SEMIS
;

;  *********
;  *   <   *
;  *********
;
        ALIGN    8
N_LESS:
        DQ      1
        DB      "<"
        ALIGN    8
LESS:
        DQ    LESS+HEADSIZE
        DQ    LESS+HEADSIZE
        DQ    0x0
        DQ    EQUAL
        DQ    N_LESS
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;S1
        LDR     X3, [X13],#+(CW*(1))      ;S2
        MOV    X0, #0   ;0 default RESULT
        CMP     X3,X1
        BGE     LES1
        SUB     X0,X0, #1
LES1:   STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   U<   *
;  **********
;
        ALIGN    8
N_ULESS:
        DQ      2
        DB      "U<"
        ALIGN    8
ULESS:
        DQ    ULESS+HEADSIZE
        DQ    ULESS+HEADSIZE
        DQ    0x0
        DQ    LESS
        DQ    N_ULESS
        DQ    0

        LDR     X0, [X13],#+(CW*(1))
        LDR     X1, [X13],#+(CW*(1))
        SUBS    X1,X1,X0
        SBC     X0,X0,X0
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *********
;  *   >   *
;  *********
;
        ALIGN    8
N_GREAT:
        DQ      1
        DB      ">"
        ALIGN    8
GREAT:
        DQ    DOCOL
        DQ    GREAT+HEADSIZE
        DQ    0x0
        DQ    ULESS
        DQ    N_GREAT
        DQ    0

        DQ      SWAP
        DQ      LESS
        DQ      SEMIS
;

;  **********
;  *   <>   *
;  **********
;
        ALIGN    8
N_UNEQ:
        DQ      2
        DB      "<>"
        ALIGN    8
UNEQ:
        DQ    DOCOL
        DQ    UNEQ+HEADSIZE
        DQ    0x0
        DQ    GREAT
        DQ    N_UNEQ
        DQ    0

        DQ      LSUB
        DQ      ZEQU
        DQ      ZEQU
        DQ      SEMIS
;

;  ***********
;  *   ROT   *
;  ***********
;
        ALIGN    8
N_ROT:
        DQ      3
        DB      "ROT"
        ALIGN    8
ROT:
        DQ    ROT+HEADSIZE
        DQ    ROT+HEADSIZE
        DQ    0x0
        DQ    UNEQ
        DQ    N_ROT
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;S1
        LDR     X3, [X13],#+(CW*(1))      ;S2
        LDR     X0, [X13],#+(CW*(1))      ;S3
        STR     X3, [X13,#-(CW*(1))]!
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  *************
;  *   SPACE   *
;  *************
;
        ALIGN    8
N_SPACE:
        DQ      5
        DB      "SPACE"
        ALIGN    8
SPACE:
        DQ    DOCOL
        DQ    SPACE+HEADSIZE
        DQ    0x0
        DQ    ROT
        DQ    N_SPACE
        DQ    0

        DQ      LBL
        DQ      EMIT
        DQ      SEMIS
;

;  ************
;  *   ?DUP   *
;  ************
;
        ALIGN    8
N_QDUP:
        DQ      4
        DB      "?DUP"
        ALIGN    8
QDUP:
        DQ    DOCOL
        DQ    QDUP+HEADSIZE
        DQ    0x0
        DQ    SPACE
        DQ    N_QDUP
        DQ    0

        DQ      LDUP
        DQ      ZBRAN
        DQ      QDUP1-$-CW ; IF
        DQ      LDUP    ;THEN
QDUP1:  DQ      SEMIS
;

;  **************
;  *   LATEST   *
;  **************
;
        ALIGN    8
N_LATEST:
        DQ      6
        DB      "LATEST"
        ALIGN    8
LATEST:
        DQ    DOCOL
        DQ    LATEST+HEADSIZE
        DQ    0x0
        DQ    QDUP
        DQ    N_LATEST
        DQ    0

        DQ      CURR
        DQ      FETCH
        DQ      TLFA
        DQ      FETCH
        DQ      SEMIS
;

;  ************
;  *   >CFA   *
;  ************
;
        ALIGN    8
N_TCFA:
        DQ      4
        DB      ">CFA"
        ALIGN    8
TCFA:
        DQ    DOCOL
        DQ    TCFA+HEADSIZE
        DQ    0x0
        DQ    LATEST
        DQ    N_TCFA
        DQ    0

        DQ      LIT, (CW*(C_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;

;  ************
;  *   >DFA   *
;  ************
;
        ALIGN    8
N_TDFA:
        DQ      4
        DB      ">DFA"
        ALIGN    8
TDFA:
        DQ    DOCOL
        DQ    TDFA+HEADSIZE
        DQ    0x0
        DQ    TCFA
        DQ    N_TDFA
        DQ    0

        DQ      LIT, (CW*(D_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;

;  ************
;  *   >FFA   *
;  ************
;
        ALIGN    8
N_TFFA:
        DQ      4
        DB      ">FFA"
        ALIGN    8
TFFA:
        DQ    DOCOL
        DQ    TFFA+HEADSIZE
        DQ    0x0
        DQ    TDFA
        DQ    N_TFFA
        DQ    0

        DQ      LIT, (CW*(F_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;

;  ************
;  *   >LFA   *
;  ************
;
        ALIGN    8
N_TLFA:
        DQ      4
        DB      ">LFA"
        ALIGN    8
TLFA:
        DQ    DOCOL
        DQ    TLFA+HEADSIZE
        DQ    0x0
        DQ    TFFA
        DQ    N_TLFA
        DQ    0

        DQ      LIT, (CW*(L_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;

;  ************
;  *   >NFA   *
;  ************
;
        ALIGN    8
N_TNFA:
        DQ      4
        DB      ">NFA"
        ALIGN    8
TNFA:
        DQ    DOCOL
        DQ    TNFA+HEADSIZE
        DQ    0x0
        DQ    TLFA
        DQ    N_TNFA
        DQ    0

        DQ      LIT,(CW*(N_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;


;
;  ************
;  *   >SFA   *
;  ************
;
        ALIGN    8
N_TSFA:
        DQ      4
        DB      ">SFA"
        ALIGN    8
TSFA:
        DQ    DOCOL
        DQ    TSFA+HEADSIZE
        DQ    0x0
        DQ    TNFA
        DQ    N_TSFA
        DQ    0

        DQ      LIT,(CW*(S_HOFFSET))
        DQ      PLUS
        DQ      SEMIS
;
;
;

;  ************
;  *   >PHA   *
;  ************
;
        ALIGN    8
N_TPHA:
        DQ      4
        DB      ">PHA"
        ALIGN    8
TPHA:
        DQ    DOCOL
        DQ    TPHA+HEADSIZE
        DQ    0x0
        DQ    TSFA
        DQ    N_TPHA
        DQ    0

        DQ      LIT,(CW*(PH_OFFSET))
        DQ      PLUS
        DQ      SEMIS
;

;  *************
;  *   >BODY   *
;  *************
;
        ALIGN    8
N_TOBODY:
        DQ      5
        DB      ">BODY"
        ALIGN    8
TOBODY:
        DQ    DOCOL
        DQ    TOBODY+HEADSIZE
        DQ    0x0
        DQ    TPHA
        DQ    N_TOBODY
        DQ    0

        DQ      TDFA, FETCH
        DQ      CELLP           ; Skip DOES> pointer.
        DQ      SEMIS
;

;  *************
;  *   BODY>   *
;  *************
;
        ALIGN    8
N_BODYF:
        DQ      5
        DB      "BODY>"
        ALIGN    8
BODYF:
        DQ    DOCOL
        DQ    BODYF+HEADSIZE
        DQ    0x0
        DQ    TOBODY
        DQ    N_BODYF
        DQ    0

        DQ      LIT,(CW*(BD_OFFSET))
        DQ      LSUB
        DQ      SEMIS
;

;  ************
;  *   >WID   *
;  ************
;
        ALIGN    8
N_TWID:
        DQ      4
        DB      ">WID"
        ALIGN    8
TWID:
        DQ    DOCOL
        DQ    TWID+HEADSIZE
        DQ    0x0
        DQ    BODYF
        DQ    N_TWID
        DQ    0

        DQ      TOBODY
        DQ      CELLP ; Skip vfa link.
        DQ      SEMIS
;

;  ************
;  *   >VFA   *
;  ************
;
        ALIGN    8
N_TVFA:
        DQ      4
        DB      ">VFA"
        ALIGN    8
TVFA:
        DQ    DOCOL
        DQ    TVFA+HEADSIZE
        DQ    0x0
        DQ    TWID
        DQ    N_TVFA
        DQ    0

        DQ      TOBODY
        DQ      SEMIS
;


;  ************
;  *   !CSP   *
;  ************
;
        ALIGN    8
N_SCSP:
        DQ      4
        DB      "!CSP"
        ALIGN    8
SCSP:
        DQ    DOCOL
        DQ    SCSP+HEADSIZE
        DQ    0x0
        DQ    TVFA
        DQ    N_SCSP
        DQ    0

        DQ      SPFET
        DQ      LCSP
        DQ      STOR
        DQ      SEMIS
;
;

;  **************
;  *   ?ERROR   *
;  **************
;
        ALIGN    8
N_QERR:
        DQ      6
        DB      "?ERROR"
        ALIGN    8
QERR:
        DQ    DOCOL
        DQ    QERR+HEADSIZE
        DQ    0x0
        DQ    SCSP
        DQ    N_QERR
        DQ    0

        DQ      SWAP
        DQ      ZBRAN
        DQ      QERR1-$-CW ;IF
        DQ      PP, FETCH
        DQ      SRC, FETCH
        DQ      LWHERE, TSTOR
        DQ      THROW
        DQ      BRAN
        DQ      QERR2-$-CW  ;ELSE
QERR1:  DQ      DROP    ;THEN
QERR2:  DQ      SEMIS
;

;  **************
;  *   ?ERRUR   *
;  **************
;
        ALIGN    8
N_QERRUR:
        DQ      6
        DB      "?ERRUR"
        ALIGN    8
QERRUR:
        DQ    DOCOL
        DQ    QERRUR+HEADSIZE
        DQ    0x0
        DQ    QERR
        DQ    N_QERRUR
        DQ    0

        DQ      ZERO, MIN, LDUP, QERR
        DQ      SEMIS
;


;  **************
;  *   ?DELIM   *
;  **************
;
        ALIGN    8
N_QDELIM:
        DQ      6
        DB      "?DELIM"
        ALIGN    8
QDELIM:
        DQ    DOCOL
        DQ    QDELIM+HEADSIZE
        DQ    0x0
        DQ    QERRUR
        DQ    N_QDELIM
        DQ    0

        DQ      PPFET
        DQ      QBL
        DQ      ZEQU
        DQ      LIT, 10, QERR
        DQ      DROP
        DQ      SEMIS
;

;  ************
;  *   ?CSP   *
;  ************
;
        ALIGN    8
N_QCSP:
        DQ      4
        DB      "?CSP"
        ALIGN    8
QCSP:
        DQ    DOCOL
        DQ    QCSP+HEADSIZE
        DQ    0x0
        DQ    QDELIM
        DQ    N_QCSP
        DQ    0

        DQ      SPFET
        DQ      LCSP
        DQ      FETCH
        DQ      LSUB
        DQ      LIT, 20, QERR
        DQ      SEMIS
;

;  *************
;  *   ?COMP   *
;  *************
;
        ALIGN    8
N_QCOMP:
        DQ      5
        DB      "?COMP"
        ALIGN    8
QCOMP:
        DQ    DOCOL
        DQ    QCOMP+HEADSIZE
        DQ    0x0
        DQ    QCSP
        DQ    N_QCOMP
        DQ    0

        DQ      STATE
        DQ      FETCH
        DQ      ZEQU
        DQ      LIT, 17, QERR
        DQ      SEMIS
;

; 

;  **************
;  *   ?PAIRS   *
;  **************
;
        ALIGN    8
N_QPAIR:
        DQ      6
        DB      "?PAIRS"
        ALIGN    8
QPAIR:
        DQ    DOCOL
        DQ    QPAIR+HEADSIZE
        DQ    0x0
        DQ    QCOMP
        DQ    N_QPAIR
        DQ    0

        DQ      LSUB
        DQ      LIT, 19, QERR
        DQ      SEMIS
;

; 
;
;
;

;  *********
;  *   [   *
;  *********
;
        ALIGN    8
N_LBRAC:
        DQ      1
        DB      "["
        ALIGN    8
LBRAC:
        DQ    DOCOL
        DQ    LBRAC+HEADSIZE
        DQ    B_IMMED
        DQ    QPAIR
        DQ    N_LBRAC
        DQ    0

        DQ      ZERO
        DQ      STATE
        DQ      STOR
        DQ      SEMIS
;

;  *********
;  *   ]   *
;  *********
;
        ALIGN    8
N_RBRAC:
        DQ      1
        DB      "]"
        ALIGN    8
RBRAC:
        DQ    DOCOL
        DQ    RBRAC+HEADSIZE
        DQ    0x0
        DQ    LBRAC
        DQ    N_RBRAC
        DQ    0

        DQ      ONE
        DQ      STATE
        DQ      STOR
        DQ      SEMIS
;

;  **************
;  *   HIDDEN   *
;  **************
;
        ALIGN    8
N_HIDDEN:
        DQ      6
        DB      "HIDDEN"
        ALIGN    8
HIDDEN:
        DQ    DOCOL
        DQ    HIDDEN+HEADSIZE
        DQ    0x0
        DQ    RBRAC
        DQ    N_HIDDEN
        DQ    0

        DQ      TFFA
        DQ      LIT,B_INVIS
        DQ      TOGGL
        DQ      SEMIS
;

;  ***********
;  *   HEX   *
;  ***********
;
        ALIGN    8
N_HEX:
        DQ      3
        DB      "HEX"
        ALIGN    8
HEX:
        DQ    DOCOL
        DQ    HEX+HEADSIZE
        DQ    0x0
        DQ    HIDDEN
        DQ    N_HEX
        DQ    0

        DQ      LIT,16
        DQ      BASE
        DQ      STOR
        DQ      SEMIS
;

;  ***************
;  *   DECIMAL   *
;  ***************
;
        ALIGN    8
N_DECA:
        DQ      7
        DB      "DECIMAL"
        ALIGN    8
DECA:
        DQ    DOCOL
        DQ    DECA+HEADSIZE
        DQ    0x0
        DQ    HEX
        DQ    N_DECA
        DQ    0

        DQ      LIT,10
        DQ      BASE
        DQ      STOR
        DQ      SEMIS
;

;  ***************
;  *   (;CODE)   *
;  ***************
;
        ALIGN    8
N_PSCOD:
        DQ      7
        DB      "(;CODE)"
        ALIGN    8
PSCOD:
        DQ    DOCOL
        DQ    PSCOD+HEADSIZE
        DQ    0x0
        DQ    DECA
        DQ    N_PSCOD
        DQ    0

        DQ      FROMR
        DQ      LATEST
        DQ      TCFA
        DQ      STOR
        DQ      SEMIS
;

;

;  **************
;  *   CREATE   *
;  **************
;
        ALIGN    8
N_LCREATE:
        DQ      6
        DB      "CREATE"
        ALIGN    8
LCREATE:
        DQ    DOCOL
        DQ    LCREATE+HEADSIZE
        DQ    0x0
        DQ    PSCOD
        DQ    N_LCREATE
        DQ    0

        DQ      LNAME
        DQ      PCREAT
        DQ      LIT, HLNOOP, COMMA
        DQ      PSCOD
DODOE:  STR     X11, [X12,#-(CW*(1))]!
        LDR     X11,[X0,#+(CW*((D_HOFFSET)))] ;NEW IP 
        MOV     X0, X11
        ADD     X0,X0,#(CW*(1))
        LDR     X11,[X11]
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
HLNOOP: DQ      SEMIS
;

;  *************
;  *   DOES>   *
;  *************
;
        ALIGN    8
N_DOES:
        DQ      5
        DB      "DOES>"
        ALIGN    8
DOES:
        DQ    DOCOL
        DQ    DOES+HEADSIZE
        DQ    0x0
        DQ    LCREATE
        DQ    N_DOES
        DQ    0

        DQ      FROMR
        DQ      LATEST
        DQ      TDFA
        DQ      FETCH
        DQ      STOR
        DQ      SEMIS
;

;  *************
;  *   COUNT   *
;  *************
;
        ALIGN    8
N_COUNT:
        DQ      5
        DB      "COUNT"
        ALIGN    8
COUNT:
        DQ    DOCOL
        DQ    COUNT+HEADSIZE
        DQ    0x0
        DQ    DOES
        DQ    N_COUNT
        DQ    0

        DQ      LDUP
        DQ      ONEP
        DQ      SWAP
        DQ      CFET
        DQ      SEMIS
;

;  *****************
;  *   -TRAILING   *
;  *****************
;
        ALIGN    8
N_DTRAI:
        DQ      9
        DB      "-TRAILING"
        ALIGN    8
DTRAI:
        DQ    DOCOL
        DQ    DTRAI+HEADSIZE
        DQ    0x0
        DQ    COUNT
        DQ    N_DTRAI
        DQ    0

        DQ      LDUP
        DQ      ZERO
        DQ     XQDO
        DQ      DTRA4-$-CW
DTRA1:  DQ      OVER
        DQ      OVER
        DQ      PLUS
        DQ      ONE
        DQ      LSUB
        DQ      CFET
        DQ      QBL
        DQ      ZEQU
        DQ      ZBRAN
        DQ      DTRA2-$-CW ;IF
        DQ      LLEAV
DTRA2:  DQ      ONE
        DQ      LSUB    ; THEN
        DQ     ONE, XPLOOP, ZBRAN
        DQ      DTRA1-$-CW
	DQ      UNLOOP    ; LOOP
DTRA4:
        DQ      SEMIS
;

;  **********
;  *   S"   *
;  **********
;
        ALIGN    8
N_SQUOT:
        DQ      2
        DB      'S"'
        ALIGN    8
SQUOT:
        DQ    DOCOL
        DQ    SQUOT+HEADSIZE
        DQ    B_IMMED
        DQ    DTRAI
        DQ    N_SQUOT
        DQ    0

        DQ      DENQ
        DQ      SEMIS
;

;  **********
;  *   ."   *
;  **********
;
        ALIGN    8
N_DOTQ:
        DQ      2
        DB      '."'
        ALIGN    8
DOTQ:
        DQ    DOCOL
        DQ    DOTQ+HEADSIZE
        DQ    B_IMMED
        DQ    SQUOT
        DQ    N_DOTQ
        DQ    0

        DQ      DENQ
        DQ      STATE
        DQ      FETCH
        DQ      ZBRAN
        DQ      DOTQ1-$-CW ; IF
        DQ      LIT, LTYPE, COMMA
        DQ      BRAN
        DQ      DOTQ2-$-CW
DOTQ1:
        DQ      LTYPE
DOTQ2:
        DQ      SEMIS   ; THEN
;

;  **********
;  *   .(   *
;  **********
;
        ALIGN    8
N_DOTP:
        DQ      2
        DB      ".("
        ALIGN    8
DOTP:
        DQ    DOCOL
        DQ    DOTP+HEADSIZE
        DQ    B_IMMED
        DQ    DOTQ
        DQ    N_DOTP
        DQ    0

        DQ      LIT, ')'
        DQ      PARSE
        DQ      LTYPE
        DQ      SEMIS
;

;  ***************
;  *   SET-SRC   *
;  ***************
;
        ALIGN    8
N_SETSRC:
        DQ      7
        DB      "SET-SRC"
        ALIGN    8
SETSRC:
        DQ    DOCOL
        DQ    SETSRC+HEADSIZE
        DQ    0x0
        DQ    DOTP
        DQ    N_SETSRC
        DQ    0

        DQ      OVER, PLUS
        DQ      OVER, SRC, TSTOR
        DQ      PP, STOR ;  IN

;       DC      DOTS
        DQ      SEMIS
;

;  ****************
;  *   EVALUATE   *
;  ****************
;
        ALIGN    8
N_EVALUATE:
        DQ      8
        DB      "EVALUATE"
        ALIGN    8
EVALUATE:
        DQ    DOCOL
        DQ    EVALUATE+HEADSIZE
        DQ    0x0
        DQ    SETSRC
        DQ    N_EVALUATE
        DQ    0

        DQ      SAVE
        DQ      SETSRC
        DQ      LIT, INTER, CATCH
        DQ      RESTO
        DQ      THROW
        DQ      SEMIS
;

;  ************
;  *   FILL   *
;  ************
;
        ALIGN    8
N_FILL:
        DQ      4
        DB      "FILL"
        ALIGN    8
FILL:
        DQ    FILL+HEADSIZE
        DQ    FILL+HEADSIZE
        DQ    0x0
        DQ    EVALUATE
        DQ    N_FILL
        DQ    0

        LDR     X0, [X13],#+(CW*(1))      ; FILL CHAR
        LDR     X2, [X13],#+(CW*(1))      ; FILL COUNT
        LDR     X1, [X13],#+(CW*(1))      ; BEGIN ADDR
        ANDS X2,X2, X2
        BEQ FILL2    ; COUNT=0? 
FILL1:  STRB    W0, [X1],#+1
        SUBS  X2,X2,#1
        BNE FILL1
FILL2:  LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ************
;  *   CORA   *
;  ************
;
        ALIGN    8
N_CORA:
        DQ      4
        DB      "CORA"
        ALIGN    8
CORA:
        DQ    CORA+HEADSIZE
        DQ    CORA+HEADSIZE
        DQ    0x0
        DQ    FILL
        DQ    N_CORA
        DQ    0

        
        MOV    X0,#0   ; Result, assume all equal
        LDR     X3, [X13],#+(CW*(1))      ; count
        LDR     X4, [X13],#+(CW*(1))      ; addr2
        LDR     X5, [X13],#+(CW*(1))      ; addr1
        MOV     X1,#0
        MOV     X2,#0
CORA1:
        SUBS     X3,X3,#1
        BMI      CORA3        ; Stop looping if R3 < 0  and return 0 
        LDRB    W1, [X5],#+1          ; load byte at R5 in W1, auto-increment R5 
        LDRB    W2, [X4],#+1          ; load byte at R4 in W2, auto-increment R4 
        CMP      W1,W2
        BEQ      CORA1        ; chars equal, keep looping 
        MOV     X0, #1
        BHI      CORA3        ; R1/W1 is lexicographic later, return 1 
        
        MOV    X0,#-1  ;TRUE FLAG  
CORA3:
        
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   $^   *
;  **********
;
        ALIGN    8
N_SINDEX:
        DQ      2
        DB      "$^"
        ALIGN    8
SINDEX:
        DQ    SINDEX+HEADSIZE
        DQ    SINDEX+HEADSIZE
        DQ    0x0
        DQ    CORA
        DQ    N_SINDEX
        DQ    0

;     search address of given byte starting from R4 
        LDR     X0, [X13],#+(CW*(1))      ; char
        LDR     X2, [X13],#+(CW*(1))      ; count
        LDR     X4, [X13],#+(CW*(1))      ; addr
        BICS    X1,X1,X1    ;Clear R1 and zero flag
SINDEX1:
        ANDS    X2,X2,X2
        BEQ     SINDEX3
        LDRB    W1, [X4],#+1         ; load next byte from R4 and auto-increment R4
        CMP     W0,W1       ; Compare BYTE and set flags
        BEQ     SINDEX2
        SUB     X2,X2,#1
        B     SINDEX1
SINDEX3:
        MOV    X4,#0       ; Not found: 0 
        ADD     X4,X4, #1
SINDEX2:
        SUB     X4,X4, #1
        STR     X4, [X13,#-(CW*(1))]!            ; Return address, or 0 
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   $/   *
;  **********
;
        ALIGN    8
N_SSLAS:
        DQ      2
        DB      "$/"
        ALIGN    8
SSLAS:
        DQ    SSLAS+HEADSIZE
        DQ    SSLAS+HEADSIZE
        DQ    0x0
        DQ    SINDEX
        DQ    N_SSLAS
        DQ    0

        
        LDR     X0, [X13],#+(CW*(1))          ; char
        LDR     X2, [X13],#+(CW*(1))          ; count
        MOV     X5,X2
        LDR     X4, [X13],#+(CW*(1))          ; addr
        MOV     X1,X4   ; Copy
        ANDS    X4,X4,X4    ; test R4 for nullpointer 
        BEQ     SSLAS3   ; Nil pointer 
SSLAS1:
        LDRB    W3, [X4],#+1         ; load byte at R4 in W3, then auto-increment R4
        CMP     W0, W3
        BEQ     SSLAS2
        SUBS    X2,X2, #1
        BNE     SSLAS1

; Not present.
        STR     X2, [X13,#-(CW*(1))]!    ; Nil pointer.
        B     SSLAS4
SSLAS2:
        SUB     X5,X5,X2
        SUB     X2,X2,#1 ;Delimiter is not part of first string.
SSLAS3:
        STR     X4, [X13,#-(CW*(1))]!
SSLAS4:
        STR     X2, [X13,#-(CW*(1))]!   ;Remaining length
        STR     X1, [X13,#-(CW*(1))]!   ;Start of first string.
        STR     X5, [X13,#-(CW*(1))]!   ;Skipped length.
        
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
;  **********
;  *   $\   *
;  **********
;
        ALIGN    8
N_BSLAS:
        DQ      2
        DB      "$\"
        ALIGN    8
BSLAS:
        DQ    BSLAS+HEADSIZE
        DQ    BSLAS+HEADSIZE
        DQ    0x0
        DQ    SSLAS
        DQ    N_BSLAS
        DQ    0
   ;_dummy
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  

;  *************
;  *   ERASE   *
;  *************
;
        ALIGN    8
N_LERASE:
        DQ      5
        DB      "ERASE"
        ALIGN    8
LERASE:
        DQ    DOCOL
        DQ    LERASE+HEADSIZE
        DQ    0x0
        DQ    BSLAS
        DQ    N_LERASE
        DQ    0

        DQ      ZERO
        DQ      FILL
        DQ      SEMIS
;

;  *************
;  *   BLANK   *
;  *************
;
        ALIGN    8
N_BLANK:
        DQ      5
        DB      "BLANK"
        ALIGN    8
BLANK:
        DQ    DOCOL
        DQ    BLANK+HEADSIZE
        DQ    0x0
        DQ    LERASE
        DQ    N_BLANK
        DQ    0

        DQ      LBL
        DQ      FILL
        DQ      SEMIS
;

;  ************
;  *   HOLD   *
;  ************
;
        ALIGN    8
N_HOLD:
        DQ      4
        DB      "HOLD"
        ALIGN    8
HOLD:
        DQ    DOCOL
        DQ    HOLD+HEADSIZE
        DQ    0x0
        DQ    BLANK
        DQ    N_HOLD
        DQ    0

        DQ      LIT,-1
        DQ      HLD
        DQ      PSTOR
        DQ      HLD
        DQ      FETCH
        DQ      CSTOR
        DQ      SEMIS
;

;  ***********
;  *   PAD   *
;  ***********
;
        ALIGN    8
N_PAD:
        DQ      3
        DB      "PAD"
        ALIGN    8
PAD:
        DQ    DOCOL
        DQ    PAD+HEADSIZE
        DQ    0x0
        DQ    HOLD
        DQ    N_PAD
        DQ    0

        DQ      HERE
; Allow for a one line name, a double binary number and some hold char's
        DQ      LIT,84+128+64
        DQ      PLUS
        DQ      SEMIS
;

; 
;

;  ************
;  *   CHAR   *
;  ************
;
        ALIGN    8
N_LCHAR:
        DQ      4
        DB      "CHAR"
        ALIGN    8
LCHAR:
        DQ    DOCOL
        DQ    LCHAR+HEADSIZE
        DQ    0x0
        DQ    PAD
        DQ    N_LCHAR
        DQ    0

        DQ      LNAME, DROP, CFET
        DQ      SEMIS
;

;  **************
;  *   [CHAR]   *
;  **************
;
        ALIGN    8
N_BCHAR:
        DQ      6
        DB      "[CHAR]"
        ALIGN    8
BCHAR:
        DQ    DOCOL
        DQ    BCHAR+HEADSIZE
        DQ    B_IMMED
        DQ    LCHAR
        DQ    N_BCHAR
        DQ    0

        DQ      LCHAR, LITER
        DQ      SEMIS
;

;  ****************
;  *   (NUMBER)   *
;  ****************
;
        ALIGN    8
N_PNUMB:
        DQ      8
        DB      "(NUMBER)"
        ALIGN    8
PNUMB:
        DQ    DOCOL
        DQ    PNUMB+HEADSIZE
        DQ    0x0
        DQ    BCHAR
        DQ    N_PNUMB
        DQ    0

        DQ      ZERO, ZERO
        DQ      ZERO, DPL, STOR
NPNUM1:  DQ      PPFET   ; BEGIN
        DQ      LDUP, LIT, ADOT, EQUAL
        DQ      ZBRAN
        DQ      NPNUM2-$-CW ; case 1 
        DQ      DROP, DPL, STOR
        DQ      BRAN
        DQ      NPNUM1-$-CW ; continue 
NPNUM2:
        DQ      LDUP, LIT, ',', EQUAL
        DQ      ZBRAN
        DQ      NPNUM3-$-CW ; case 2 
        DQ      TDROP
        DQ      BRAN
        DQ      NPNUM1-$-CW ; continue 
NPNUM3:
        DQ      LDUP, QBL, ZEQU
        DQ      ZBRAN
        DQ      NPNUM5-$-CW  ; while 
        DQ      NIP
        DQ      BASE, FETCH, DIGIT
        DQ      ZEQU
        DQ      LIT, 10, QERR

        DQ      SWAP
        DQ      BASE
        DQ      FETCH
        DQ      USTAR
        DQ      DROP
        DQ      ROT
        DQ      BASE
        DQ      FETCH
        DQ      USTAR
        DQ      DPLUS
        DQ      BRAN
        DQ      NPNUM1-$-CW ; repeat 
NPNUM5:
        DQ      DROP, PP, STOR
        DQ      SEMIS
;

;  **************
;  *   NUMBER   *
;  **************
;
        ALIGN    8
N_NUMB:
        DQ      6
        DB      "NUMBER"
        ALIGN    8
NUMB:
        DQ    DOCOL
        DQ    NUMB+HEADSIZE
        DQ    0x0
        DQ    PNUMB
        DQ    N_NUMB
        DQ    0

LNUMB:
        DQ      LIT, -1, PP, PSTOR
        DQ      PNUMB, SDLITE
        DQ      SEMIS
;

;  ***************
;  *   >NUMBER   *
;  ***************
;
        ALIGN    8
N_TONUMB:
        DQ      7
        DB      ">NUMBER"
        ALIGN    8
TONUMB:
        DQ    DOCOL
        DQ    TONUMB+HEADSIZE
        DQ    0x0
        DQ    NUMB
        DQ    N_TONUMB
        DQ    0

        DQ      TDUP, PLUS, TOR     ; End available on return stack.
        DQ      ZERO
        DQ     XQDO
        DQ      TONUM9-$-CW
TONUM1:
        DQ      LDUP, CFET, BASE, FETCH, DIGIT
        DQ      ZEQU
        DQ      ZBRAN
        DQ      TONUM4-$-CW ; IF
        DQ      DROP
        DQ      LLEAV
TONUM4:
        DQ      SWAP, TOR ; Address out of the way.
        DQ      SWAP
        DQ      BASE
        DQ      FETCH
        DQ      USTAR
        DQ      DROP
        DQ      ROT
        DQ      BASE
        DQ      FETCH
        DQ      USTAR
        DQ      DPLUS
        DQ      FROMR, ONEP     ; Address back.
        DQ     ONE, XPLOOP, ZBRAN
        DQ      TONUM1-$-CW
	DQ      UNLOOP
TONUM9:
        DQ      FROMR
        DQ      OVER, LSUB
        DQ      SEMIS
;

;  *************
;  *   FOUND   *
;  *************
;
        ALIGN    8
N_FOUND:
        DQ      5
        DB      "FOUND"
        ALIGN    8
FOUND:
        DQ    DOCOL
        DQ    FOUND+HEADSIZE
        DQ    0x0
        DQ    TONUMB
        DQ    N_FOUND
        DQ    0

        DQ      CONTEXT, TOR
FOUND1: DQ      RR, FETCH
;        DC      DOTS
        DQ      PFIND, LDUP, ZEQU
        DQ      ZBRAN
        DQ      FOUND3-$-CW
        DQ      DROP
        DQ      RR, FETCH, LIT, ONLYBODY, LSUB ;Was this ONLY?
        DQ      ZBRAN
        DQ      FOUND2-$-CW
        DQ      FROMR, CELLP, TOR
        DQ      BRAN
        DQ      FOUND1-$-CW
FOUND2: DQ      ZERO
FOUND3: DQ      RDROP
        DQ      NIP,NIP
        DQ      SEMIS
;

;  ***************
;  *   PRESENT   *
;  ***************
;
        ALIGN    8
N_PRESENT:
        DQ      7
        DB      "PRESENT"
        ALIGN    8
PRESENT:
        DQ    DOCOL
        DQ    PRESENT+HEADSIZE
        DQ    0x0
        DQ    FOUND
        DQ    N_PRESENT
        DQ    0

        DQ      LDUP, TOR
        DQ      FOUND
        DQ      LDUP
        DQ      ZBRAN
        DQ      PRES1-$-CW
        DQ      LDUP
        DQ      TNFA, FETCH, FETCH ;  Get precise length.
        DQ      RR, EQUAL
        DQ      LAND
PRES1:
        DQ      RDROP
        DQ      SEMIS
;


; 
;

;

;  **************
;  *   (FIND)   *
;  **************
;
        ALIGN    8
N_PFIND:
        DQ      6
        DB      "(FIND)"
        ALIGN    8
PFIND:
        DQ    DOCOL
        DQ    PFIND+HEADSIZE
        DQ    0x0
        DQ    PRESENT
        DQ    N_PFIND
        DQ    0

; The idea is to have a fast slim loop.
; The optimisation tries to skip, using the extra links.
; At all labels the stack is (sc, dea).
; PFIND0 : nothing known yet about dea
; PFCONTINUE : dea is disapproved, go on 
; PFIND1 : dea is a match, inasfar name and prefix
; PFEXIT : dea is the match, or null.

PFIND0:                       ;Start of fast loop.
         DQ      NMATCH
         DQ      ZEQU
        DQ      ZBRAN
        DQ      PFCONTINUE-$-CW
; dea's name and sc have equal length
        DQ      TDUP, TNFA, FETCH, FETCH
        DQ      EQUAL 
; dea is a prefix 
        DQ      OVER, TFFA, FETCH
        DQ      LIT, B_DENOT, LAND
        DQ      LOR             ; One of those ,ist apply.
        DQ      ZBRAN
        DQ      PFCONTINUE-$-CW
; Discard still, if invisible ("smudged") or dummy.
        DQ      LDUP, TFFA, FETCH
        DQ      LIT, B_INVIS | B_DUMMY, LAND
        DQ      ZEQU
        DQ      ZBRAN
        DQ      PFCONTINUE-$-CW
        DQ      BRAN
        DQ      PFEXIT-$-CW		; Match! 
PFCONTINUE:
        DQ	TLFA, FETCH
        DQ      LDUP, ZEQU
        DQ      ZBRAN
        DQ      PFIND0-$-CW
PFEXIT:   
        DQ      SEMIS
;

;  *************
;  *   ERROR   *
;  *************
;
        ALIGN    8
N_LERROR:
        DQ      5
        DB      "ERROR"
        ALIGN    8
LERROR:
        DQ    DOCOL
        DQ    LERROR+HEADSIZE
        DQ    0x0
        DQ    PFIND
        DQ    N_LERROR
        DQ    0

; Make sure the whole word is shown in case of denotations. 
        DQ      LWHERE, TFET
        DQ      OVER
        DQ      ZBRAN
        DQ      ERR2-$-CW      ; Virgin: print empty string.
        DQ      OVER, LIT, 20, LSUB
        DQ      MAX
        DQ      SWAP
        DQ      ONEM         ;Backup for if we are at next word.
ERR1:   DQ      LDUP, CFET, QBL, ZEQU  ;Show whole word!
        DQ      ZBRAN
        DQ      ERR2 -$-CW
        DQ      ONEP
        DQ      BRAN
        DQ      ERR1 -$-CW
ERR2:
        DQ      OVER, LSUB
        DQ      ETYPE
        DQ      SKIP
         DQ      19
SB1: DB      " ? ciforth ERROR # "
       ALIGN    8
        DQ      LIT, SB1
        DQ      LIT, 19
        DQ      ETYPE
        DQ      BASE, FETCH
        DQ      DECA
        DQ      OVER
        DQ      STOD, ZERO, PDDOTR      ;This is about (.) 
        DQ      ETYPE
        DQ      BASE, STOR
        DQ      LIT, -256, MAX
        DQ      LIT, 63, MIN
        DQ      LIT, MESS, CATCH, DROP  ;Prevent error loops 
        DQ      SEMIS
;

;  *************
;  *   CATCH   *
;  *************
;
        ALIGN    8
N_CATCH:
        DQ      5
        DB      "CATCH"
        ALIGN    8
CATCH:
        DQ    DOCOL
        DQ    CATCH+HEADSIZE
        DQ    0x0
        DQ    LERROR
        DQ    N_CATCH
        DQ    0

        DQ      SPFET, CELLP, TOR
        DQ      HANDLER, FETCH, TOR
        DQ      RPFET, HANDLER, STOR
        DQ      EXEC
        DQ      FROMR, HANDLER, STOR
        DQ      RDROP, ZERO
        DQ      SEMIS
;

;  *************
;  *   THROW   *
;  *************
;
        ALIGN    8
N_THROW:
        DQ      5
        DB      "THROW"
        ALIGN    8
THROW:
        DQ    DOCOL
        DQ    THROW+HEADSIZE
        DQ    0x0
        DQ    CATCH
        DQ    N_THROW
        DQ    0

        DQ      LDUP
        DQ      ZBRAN
        DQ      THROW1-$-CW
        DQ      HANDLER, FETCH, ZEQU
        DQ      ZBRAN
        DQ      THROW2-$-CW
        DQ      LERROR
        DQ      CLS
        DQ      QUIT
THROW2:
        DQ      HANDLER, FETCH, RPSTO
        DQ      FROMR, HANDLER, STOR
        DQ      FROMR, SWAP, TOR
        DQ      SPSTO
        DQ      FROMR
        DQ      X
THROW1:
        DQ      DROP
        DQ      SEMIS
;

; 

; 
;

;  ***********
;  *   ID.   *
;  ***********
;
        ALIGN    8
N_IDDOT:
        DQ      3
        DB      "ID."
        ALIGN    8
IDDOT:
        DQ    DOCOL
        DQ    IDDOT+HEADSIZE
        DQ    0x0
        DQ    THROW
        DQ    N_IDDOT
        DQ    0

        DQ      LDUP, TFFA, FETCH
        DQ      LIT, B_DUMMY, LAND
        DQ      ZBRAN
        DQ      IDDOT1-$-CW
        DQ      DROP
        DQ      BRAN
        DQ      IDDOT2-$-CW
IDDOT1:
        DQ      TNFA
        DQ      FETCH
        DQ      SFET
        DQ      LTYPE
        DQ      SPACE
        DQ      SPACE
        DQ      SPACE
IDDOT2:
        DQ      SEMIS
;

;  **************
;  *   HEADER   *
;  **************
;
        ALIGN    8
N_LHEAD:
        DQ      6
        DB      "HEADER"
        ALIGN    8
LHEAD:
        DQ    DOCOL
        DQ    LHEAD+HEADSIZE
        DQ    0x0
        DQ    IDDOT
        DQ    N_LHEAD
        DQ    0

        DQ      LDUP
        DQ      ZEQU
        DQ      LIT, 5, QERR
        DQ      TDUP
        DQ      PRESENT
        DQ      LDUP
        DQ      ZBRAN
        DQ      CREA1-$-CW ;IF
        DQ      TNFA, FETCH, SFET
        DQ      ETYPE
        DQ      LIT,4
        DQ      MESS
        DQ      X       ;THEN
CREA1:  DQ      DROP
 DQ      LALIGN
        DQ      SCOMMA
 DQ      LALIGN
        DQ      HERE,TOR

        DQ      RR, TPHA, COMMA         ;Code field.
        DQ      RR, TPHA, COMMA         ;Data field.
        DQ      ZERO, COMMA ;Flag field.
        DQ      ZERO, COMMA ;Link field.
        DQ      COMMA   ;Name field.


        DQ      BLK, FETCH, LDUP, ZEQU
        DQ      ZBRAN
        DQ      CREA2-$-CW
        DQ      DROP, PP, FETCH
CREA2:  DQ      COMMA  ;Source field.

                DQ      FROMR
        DQ      SEMIS
;

;  ************
;  *   LINK   *
;  ************
;
        ALIGN    8
N_LLINK:
        DQ      4
        DB      "LINK"
        ALIGN    8
LLINK:
        DQ    DOCOL
        DQ    LLINK+HEADSIZE
        DQ    0x0
        DQ    LHEAD
        DQ    N_LLINK
        DQ    0

        DQ      TLFA
        DQ      TDUP
        DQ      FETCH
        DQ      SWAP, TLFA, STOR
        DQ      STOR
        DQ      SEMIS
;

;  ****************
;  *   (CREATE)   *
;  ****************
;
        ALIGN    8
N_PCREAT:
        DQ      8
        DB      "(CREATE)"
        ALIGN    8
PCREAT:
        DQ    DOCOL
        DQ    PCREAT+HEADSIZE
        DQ    0x0
        DQ    LLINK
        DQ    N_PCREAT
        DQ    0

        DQ      LHEAD
        DQ      CURR, FETCH
        DQ      LLINK
        DQ      SEMIS
;

;  *****************
;  *   [COMPILE]   *
;  *****************
;
        ALIGN    8
N_BCOMP:
        DQ      9
        DB      "[COMPILE]"
        ALIGN    8
BCOMP:
        DQ    DOCOL
        DQ    BCOMP+HEADSIZE
        DQ    B_IMMED
        DQ    PCREAT
        DQ    N_BCOMP
        DQ    0

        DQ      LNAME
        DQ      PRESENT
        DQ      LDUP
        DQ      ZEQU
        DQ      LIT, 16, QERR
        DQ      TCFA
        DQ      COMMA
        DQ      SEMIS
;

;  ****************
;  *   POSTPONE   *
;  ****************
;
        ALIGN    8
N_POSTP:
        DQ      8
        DB      "POSTPONE"
        ALIGN    8
POSTP:
        DQ    DOCOL
        DQ    POSTP+HEADSIZE
        DQ    B_IMMED
        DQ    BCOMP
        DQ    N_POSTP
        DQ    0

        DQ      LNAME
        DQ      PRESENT
        DQ      LDUP
        DQ      ZEQU
        DQ      LIT, 15, QERR
        DQ      LDUP, TFFA, FETCH
        DQ      LIT, B_IMMED, LAND, ZEQU
        DQ      ZBRAN
        DQ      POSTP1-$-CW
         DQ      LIT, LIT, COMMA
         DQ      COMMA
         DQ      LIT, COMMA
POSTP1:
         DQ      COMMA
        DQ      SEMIS
;

;  ***************
;  *   LITERAL   *
;  ***************
;
        ALIGN    8
N_LITER:
        DQ      7
        DB      "LITERAL"
        ALIGN    8
LITER:
        DQ    DOCOL
        DQ    LITER+HEADSIZE
        DQ    B_IMMED
        DQ    POSTP
        DQ    N_LITER
        DQ    0

        DQ      STATE
        DQ      FETCH
        DQ      ZBRAN
        DQ      LITE1-$-CW ;IF
        DQ      LIT, LIT, COMMA
        DQ      COMMA   ;THEN
LITE1:  DQ      SEMIS
;

;  ****************
;  *   DLITERAL   *
;  ****************
;
        ALIGN    8
N_DLITE:
        DQ      8
        DB      "DLITERAL"
        ALIGN    8
DLITE:
        DQ    DOCOL
        DQ    DLITE+HEADSIZE
        DQ    B_IMMED
        DQ    LITER
        DQ    N_DLITE
        DQ    0

        DQ      STATE
        DQ      FETCH
        DQ      ZBRAN
        DQ      DLIT1-$-CW ; IF
        DQ      SWAP
        DQ      LITER
        DQ      LITER   ; THEN
DLIT1:  DQ      SEMIS
;
;

;  *****************
;  *   SDLITERAL   *
;  *****************
;
        ALIGN    8
N_SDLITE:
        DQ      9
        DB      "SDLITERAL"
        ALIGN    8
SDLITE:
        DQ    DOCOL
        DQ    SDLITE+HEADSIZE
        DQ    B_IMMED
        DQ    DLITE
        DQ    N_SDLITE
        DQ    0

        DQ      DPL
        DQ      FETCH
        DQ      ZBRAN
        DQ      SDLIT1-$-CW ; IF
        DQ      DLITE
        DQ      BRAN
        DQ      SDLIT2-$-CW ; IF
SDLIT1:
        DQ      DROP, LITER
SDLIT2:
        DQ      SEMIS
;


;  **************
;  *   ?STACK   *
;  **************
;
        ALIGN    8
N_QSTAC:
        DQ      6
        DB      "?STACK"
        ALIGN    8
QSTAC:
        DQ    DOCOL
        DQ    QSTAC+HEADSIZE
        DQ    0x0
        DQ    SDLITE
        DQ    N_QSTAC
        DQ    0

        DQ      SPFET
        DQ      SZERO
        DQ      FETCH
        DQ      SWAP
        DQ      ULESS
        DQ      ONE, QERR
        DQ      SPFET
        DQ      HERE
        DQ      LIT,0x80
        DQ      PLUS
        DQ      ULESS
        DQ      LIT, 7, QERR
        DQ      SEMIS
        ;
;
;

;  *****************
;  *   INTERPRET   *
;  *****************
;
        ALIGN    8
N_INTER:
        DQ      9
        DB      "INTERPRET"
        ALIGN    8
INTER:
        DQ    DOCOL
        DQ    INTER+HEADSIZE
        DQ    0x0
        DQ    QSTAC
        DQ    N_INTER
        DQ    0

INTE1:
        DQ      LNAME
        DQ      LDUP      ; Zero length.
        DQ      ZBRAN
        DQ      INTE8-$-CW ;WHILE
;       DC      DOTS
;       DC      TDUP, LTYPE
        DQ      OVER, TOR       ; Save old parse pointer.
        DQ      FOUND
        DQ      LDUP, ZEQU
        DQ      LIT, 12, QERR
        DQ      LDUP, TFFA, FETCH
        DQ      LDUP, LIT, B_DENOT, LAND ;Retain copy of flags.
        DQ      ZBRAN
        DQ      INTE3B-$-CW ;IF
        DQ      OVER, TNFA, FETCH, FETCH
        DQ      RR, PLUS, PP, STOR  ;Skip over prefix.
INTE3B:                  ;THEN 
        DQ      RDROP           ; Drop old parse pointer.
        DQ      LIT, B_IMMED, LAND
        DQ      STATE, FETCH, ZEQU, LOR
        DQ      ZBRAN
        DQ      INTE3-$-CW ;IF
        DQ      EXEC
        DQ      BRAN
        DQ      INTE4-$-CW ;IF
INTE3:
        DQ      COMMA
                        ;THEN
INTE4:
        DQ      QSTAC
        DQ      BRAN
        DQ      INTE1-$-CW  ;AGAIN
INTE8:  DQ      DROP, DROP
        DQ      SEMIS
;

;  *****************
;  *   IMMEDIATE   *
;  *****************
;
        ALIGN    8
N_IMMED:
        DQ      9
        DB      "IMMEDIATE"
        ALIGN    8
IMMED:
        DQ    DOCOL
        DQ    IMMED+HEADSIZE
        DQ    0x0
        DQ    INTER
        DQ    N_IMMED
        DQ    0

        DQ      LATEST
        DQ      TFFA
        DQ      LIT, B_IMMED
        DQ      TOGGL
        DQ      SEMIS
;

;  **************
;  *   PREFIX   *
;  **************
;
        ALIGN    8
N_PREFX:
        DQ      6
        DB      "PREFIX"
        ALIGN    8
PREFX:
        DQ    DOCOL
        DQ    PREFX+HEADSIZE
        DQ    0x0
        DQ    IMMED
        DQ    N_PREFX
        DQ    0

        DQ      LATEST
        DQ      TFFA
        DQ      LIT, B_DENOT
        DQ      TOGGL
        DQ      SEMIS
;

;  *****************
;  *   NAMESPACE   *
;  *****************
;
        ALIGN    8
N_VOCAB:
        DQ      9
        DB      "NAMESPACE"
        ALIGN    8
VOCAB:
        DQ    DOCOL
        DQ    VOCAB+HEADSIZE
        DQ    0x0
        DQ    PREFX
        DQ    N_VOCAB
        DQ    0

        DQ      LCREATE
        DQ      LATEST   ; Link this DEA into VOC-LINK chain.
        DQ      VOCL
        DQ      FETCH
        DQ      COMMA
        DQ      VOCL
        DQ      STOR
        DQ      WORDLIST, DROP 
        DQ      DOES
DOVOC:
        DQ      ALSO
        DQ      CELLP   ; Make it a WID. 
        DQ      CONTEXT
        DQ      STOR
        DQ      SEMIS
        ;
;
;   The link to task is a cold start value only.
;   It is updated each time a definition is
;   appended to the 'FORTH' namespace.
;

;  ****************
;  *   WORDLIST   *
;  ****************
;
        ALIGN    8
N_WORDLIST:
        DQ      8
        DB      "WORDLIST"
        ALIGN    8
WORDLIST:
        DQ    DOCOL
        DQ    WORDLIST+HEADSIZE
        DQ    0x0
        DQ    VOCAB
        DQ    N_WORDLIST
        DQ    0

        DQ      HERE
        DQ      ZERO, COMMA   ; Dummy code field
        DQ      ZERO, COMMA   ; Dummy data field
        DQ      LIT, B_DUMMY, COMMA ; Dummy flag field
        DQ      ZERO, COMMA ;Link field: empty chain
        DQ      LIT, (ZERO+(CW*(D_HOFFSET))), COMMA ;Empty string for name.
        DQ      ZERO, COMMA
                DQ      SEMIS
        ;
;

;

;  *******************
;  *   DEFINITIONS   *
;  *******************
;
        ALIGN    8
N_DEFIN:
        DQ      11
        DB      "DEFINITIONS"
        ALIGN    8
DEFIN:
        DQ    DOCOL
        DQ    DEFIN+HEADSIZE
        DQ    0x0
        DQ    WORDLIST
        DQ    N_DEFIN
        DQ    0

        DQ      CONTEXT
        DQ      FETCH
        DQ      CURR
        DQ      STOR
        DQ      SEMIS
;

;  ************
;  *   ALSO   *
;  ************
;
        ALIGN    8
N_ALSO:
        DQ      4
        DB      "ALSO"
        ALIGN    8
ALSO:
        DQ    DOCOL
        DQ    ALSO+HEADSIZE
        DQ    0x0
        DQ    DEFIN
        DQ    N_ALSO
        DQ    0

        DQ      CONTEXT, LDUP, CELLP
        DQ      LIT, (CW*(16-1))
        DQ      LMOVE
        DQ      LIT, ONLYBODY  ;End sentinel for array of word lists.
        DQ      CONTEXT, LIT, (CW*(16)), PLUS
        DQ      STOR ;Trim sets of wordset.
        DQ      SEMIS
;

;  ****************
;  *   PREVIOUS   *
;  ****************
;
        ALIGN    8
N_PREVI:
        DQ      8
        DB      "PREVIOUS"
        ALIGN    8
PREVI:
        DQ    DOCOL
        DQ    PREVI+HEADSIZE
        DQ    0x0
        DQ    ALSO
        DQ    N_PREVI
        DQ    0

        DQ      CONTEXT, LDUP, CELLP, SWAP
        DQ      LIT, (CW*(16))
        DQ      LMOVE
        DQ      SEMIS
;

;  *********
;  *   (   *
;  *********
;
        ALIGN    8
N_PAREN:
        DQ      1
        DB      "("
        ALIGN    8
PAREN:
        DQ    DOCOL
        DQ    PAREN+HEADSIZE
        DQ    B_IMMED
        DQ    PREVI
        DQ    N_PAREN
        DQ    0

        DQ      LIT,')'
        DQ      PARSE
        DQ      TDROP
        DQ      SEMIS
;
;

;  *********
;  *   \   *
;  *********
;
        ALIGN    8
N_BACKS:
        DQ      1
        DB      "\"
        ALIGN    8
BACKS:
        DQ    DOCOL
        DQ    BACKS+HEADSIZE
        DQ    B_IMMED
        DQ    PAREN
        DQ    N_BACKS
        DQ    0

; Backup one character, just in case we are at the end of a line.
        DQ      LIT, -1, PP, PSTOR
        DQ      LIT,ALF
        DQ      PARSE
        DQ      TDROP
        DQ      SEMIS
;

;  ************
;  *   QUIT   *
;  ************
;
        ALIGN    8
N_QUIT:
        DQ      4
        DB      "QUIT"
        ALIGN    8
QUIT:
        DQ    DOCOL
        DQ    QUIT+HEADSIZE
        DQ    0x0
        DQ    BACKS
        DQ    N_QUIT
        DQ    0

        DQ      LBRAC
QUIT1:                  ;BEGIN
        DQ      RZERO
        DQ      FETCH
        DQ      RPSTO

; debugging 
;        DC      REFTIB  
; end debugging 

        DQ      LIT
        DQ      PACCEP
        DQ      CATCH
        DQ      LDUP, LIT, -EPIPE, EQUAL
        DQ      ZBRAN
        DQ      ENDIF7-$-CW
        DQ      BYE     ;End of input, no error!
ENDIF7:
        DQ      QERRUR
        DQ      ONEP
; debugging        DC      TDUP, LTYPE, CR 
        DQ      SETSRC
        DQ      INTER
        DQ      OK
        DQ      BRAN
        DQ      QUIT1-$-CW  ;AGAIN
        DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
;

;  **********
;  *   OK   *
;  **********
;
        ALIGN    8
N_OK:
        DQ      2
        DB      "OK"
        ALIGN    8
OK:
        DQ    DOCOL
        DQ    OK+HEADSIZE
        DQ    0x0
        DQ    QUIT
        DQ    N_OK
        DQ    0

        DQ      STATE
        DQ      FETCH
        DQ      ZEQU
        DQ      ZBRAN
        DQ      OK2-$-CW ;IF
        DQ      SKIP
         DQ      3
SB2: DB      " OK"
       ALIGN    8
        DQ      LIT, SB2
        DQ      LIT, 3
        DQ      LTYPE
        DQ      CR
OK2:
        DQ      SEMIS
;


;  ***************
;  *   OPTIONS   *
;  ***************
;
        ALIGN    8
N_OPTIONS:
        DQ      7
        DB      "OPTIONS"
        ALIGN    8
OPTIONS:
        DQ    DOCOL
        DQ    OPTIONS+HEADSIZE
        DQ    0x0
        DQ    OK
        DQ    N_OPTIONS
        DQ    0

;       Execute option.
        DQ      ARGS, FETCH, CELLP, CELLP, FETCH, LDUP
        DQ      ZBRAN
        DQ      OPT4-$-CW ; No options  




;Supposedly on the stack is now a pointer to the option.
        DQ      COUNT

        DQ      LIT, '-', UNEQ
        DQ      ZBRAN
        DQ      OPT3-$-CW
        DQ      LIT, 3, LDUP, LERROR
        DQ      XCODE, STOR, BYE
OPT3:
        DQ      CFET
        DQ      LIT, 0x1F, LAND
        DQ      LLOAD       ;Supposedly stack-neutral.
        DQ      ZERO, SWAP ; Sign on suppressed.
OPT4:
        DQ      DROP
        DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
;


;  *************
;  *   _INIT   *
;  *************
;
        ALIGN    8
N_INIT:
        DQ      5
        DB      "_INIT"
        ALIGN    8
INIT:
        DQ    DOCOL
        DQ    INIT+HEADSIZE
        DQ    0x0
        DQ    OPTIONS
        DQ    N_INIT
        DQ    0

        DQ      SZERO, FETCH, SPSTO
        DQ      ZERO, HANDLER, STOR
        DQ      DECA
        DQ      ONLY, FORTH, DEFIN

        DQ      LIT, 0, BLINI  ;Default, open library file read-only!;

        DQ      SEMIS
;


;  ***********
;  *   S>D   *
;  ***********
;
        ALIGN    8
N_STOD:
        DQ      3
        DB      "S>D"
        ALIGN    8
STOD:
        DQ    STOD+HEADSIZE
        DQ    STOD+HEADSIZE
        DQ    0x0
        DQ    INIT
        DQ    N_STOD
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;S1
        MOV    X0,#0
        ANDS    X1,X1,X1
        BPL     STOD1   ;POS
        SUB     X0,X0, #1      ;NEG
STOD1:
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   ABS   *
;  ***********
;
        ALIGN    8
N_LABS:
        DQ      3
        DB      "ABS"
        ALIGN    8
LABS:
        DQ    DOCOL
        DQ    LABS+HEADSIZE
        DQ    0x0
        DQ    STOD
        DQ    N_LABS
        DQ    0

        DQ      LDUP
        DQ      ZLESS
        DQ      ZBRAN
        DQ      PM1-$-CW   ;IF
        DQ      NEGATE   ;THEN
PM1:
        DQ      SEMIS
;

;  ************
;  *   DABS   *
;  ************
;
        ALIGN    8
N_DABS:
        DQ      4
        DB      "DABS"
        ALIGN    8
DABS:
        DQ    DOCOL
        DQ    DABS+HEADSIZE
        DQ    0x0
        DQ    LABS
        DQ    N_DABS
        DQ    0

        DQ      LDUP
        DQ      ZLESS
        DQ      ZBRAN
        DQ      DPM1-$-CW  ;IF
        DQ      DNEGA   ;THEN
DPM1:
        DQ      SEMIS
;

;  ***********
;  *   MIN   *
;  ***********
;
        ALIGN    8
N_MIN:
        DQ      3
        DB      "MIN"
        ALIGN    8
MIN:
        DQ    DOCOL
        DQ    MIN+HEADSIZE
        DQ    0x0
        DQ    DABS
        DQ    N_MIN
        DQ    0

        DQ      TDUP
        DQ      GREAT
        DQ      ZBRAN
        DQ      MIN1-$-CW  ;IF
        DQ      SWAP    ;THEN
MIN1:   DQ      DROP
        DQ      SEMIS
;

;  ***********
;  *   MAX   *
;  ***********
;
        ALIGN    8
N_MAX:
        DQ      3
        DB      "MAX"
        ALIGN    8
MAX:
        DQ    DOCOL
        DQ    MAX+HEADSIZE
        DQ    0x0
        DQ    MIN
        DQ    N_MAX
        DQ    0

        DQ      TDUP
        DQ      LESS
        DQ      ZBRAN
        DQ      MAX1-$-CW  ;IF
        DQ      SWAP    ;THEN
MAX1:   DQ      DROP
        DQ      SEMIS
;

;  **************
;  *   LSHIFT   *
;  **************
;
        ALIGN    8
N_LSHIFT:
        DQ      6
        DB      "LSHIFT"
        ALIGN    8
LSHIFT:
        DQ    LSHIFT+HEADSIZE
        DQ    LSHIFT+HEADSIZE
        DQ    0x0
        DQ    MAX
        DQ    N_LSHIFT
        DQ    0

        LDR     X2, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
         LSL X0,X0, X2
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **************
;  *   RSHIFT   *
;  **************
;
        ALIGN    8
N_RSHIFT:
        DQ      6
        DB      "RSHIFT"
        ALIGN    8
RSHIFT:
        DQ    RSHIFT+HEADSIZE
        DQ    RSHIFT+HEADSIZE
        DQ    0x0
        DQ    LSHIFT
        DQ    N_RSHIFT
        DQ    0

        LDR     X2, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        LSR X0,X0, X2
         STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***************
;  *   ARSHIFT   *
;  ***************
;
        ALIGN    8
N_ARSHIFT:
        DQ      7
        DB      "ARSHIFT"
        ALIGN    8
ARSHIFT:
        DQ    ARSHIFT+HEADSIZE
        DQ    ARSHIFT+HEADSIZE
        DQ    0x0
        DQ    RSHIFT
        DQ    N_ARSHIFT
        DQ    0

        LDR     X2, [X13],#+(CW*(1))
        LDR     X0, [X13],#+(CW*(1))
        ASR X0,X0, X2
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  **********
;  *   M*   *
;  **********
;
        ALIGN    8
N_MSTAR:
        DQ      2
        DB      "M*"
        ALIGN    8
MSTAR:
        DQ    MSTAR+HEADSIZE
        DQ    MSTAR+HEADSIZE
        DQ    0x0
        DQ    ARSHIFT
        DQ    N_MSTAR
        DQ    0

        LDR     X3, [X13],#+(CW*(1))
        LDR     X2, [X13],#+(CW*(1))
 ; end thumb  )

        MUL      X1,X2,X3          ;Signed *, 64 bits result 
        SMULH    X0,X2,X3          ;Signed *, 64 bits result 
        STR     X1, [X13,#-(CW*(1))]!
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ;STORE DOUBLE CELL
;

; High level implementation from FIG-forth 
;  **************
;  *   SM/REM   *
;  **************
;
        ALIGN    8
N_MSLAS:
        DQ      6
        DB      "SM/REM"
        ALIGN    8
MSLAS:
        DQ    DOCOL
        DQ    MSLAS+HEADSIZE
        DQ    0x0
        DQ    MSTAR
        DQ    N_MSLAS
        DQ    0

        DQ      OVER, TOR, TOR, DABS  ;   OVER >R >R DABS 
        DQ      RR, LABS, USLAS       ;   R ABS UM/MOD 
        DQ      FROMR, RR, LXOR       ;   R> R XOR 
        DQ      ZLESS                 ;   <0 IF NEGATE THEN 
        DQ      ZBRAN
        DQ      MSLAS1-$-CW
        DQ      NEGATE
MSLAS1:
        DQ      SWAP, FROMR           ;   SWAP R> 
        DQ      ZLESS                 ;   <0 IF NEGATE THEN 
        DQ      ZBRAN
        DQ      MSLAS2-$-CW
        DQ      NEGATE
MSLAS2:
        DQ      SWAP                  ;   SWAP ; 
        DQ      SEMIS
;
_COMMENTED({
CODE_HEADER({SM/REM},{MSLAS})
        POP(R8)      _C{DIVISOR}
        POP(R10)     _C{MSW OF DIVIDEND}
        POP(R0)      _C{LSW OF DIVIDEND}
        BL           IDIV _C{ not yet 64 bit/32 bit signed}
        _2PUSH_R10          _C{STORE QUOT/REM}
_C


_C{ Divide routine 32/32 signed from Henry Thomas (http://me.henri.net/fp-div.html)}
_C{ Entry  R0: numerator (lo) must be signed positive}
_C{        R8: denominator (den aka divisor) must be non-zero and signed negative}
IDIV:

        lo .req R0
        hi .req R10
        den .req R8
        MOVI  R5, #0   _C{ R5 is sign-memory of den}
        CMP den, #0
        BMI IDIV1
        MVN den,den  _C{ den  <-- two-complement of den}
        ADD den, den, #1
        MOVI  R5, #-1
IDIV1:  MOVI  R6, #0   _C{ R6 is sign-memory of lo}
        CMP lo,#0
        BPL IDIV2
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
        MOVI  R6,#-1

_C{the original routine:}
_C{with positive numerator and negative denominator}

IDIV2:  MOVI  hi, #0 _C{ hi = 0}
        ADDS lo, lo, lo
        .rept 32 _C{ repeat 32 times}
          ADCS hi, den, hi, lsl #1
          SUBCC hi, hi, den
          ADCS lo,lo
        .endr
_C{end of original routine}

        _C{ remainder has sign of numerator}
        CMP R6, #0
        BEQ IDIV3
        MVN hi, hi  _C{ hi  <-- two-complement of remainder}
        ADD hi, hi, #1

IDIV3:
        _C{ quotient has both signs of numerator and denominator}
        CMP R5,R6
        BNE IDIV4
        MVN lo,lo    _C{ lo <-- two-complement of lo}
        ADD lo,lo,#1
IDIV4:
        MOV PC, LR _C{ return}
_C{Exit   R0:  quotient (lo)}
_C{       R10: remainder (hi)}
}) 


;  **********
;  *   2/   *
;  **********
;
        ALIGN    8
N_TWOSL:
        DQ      2
        DB      "2/"
        ALIGN    8
TWOSL:
        DQ    DOCOL
        DQ    TWOSL+HEADSIZE
        DQ    0x0
        DQ    MSLAS
        DQ    N_TWOSL
        DQ    0

        DQ      STOD, TWO, FMSLAS
        DQ      NIP
        DQ      SEMIS
;

;  **********
;  *   2*   *
;  **********
;
        ALIGN    8
N_TWOST:
        DQ      2
        DB      "2*"
        ALIGN    8
TWOST:
        DQ    DOCOL
        DQ    TWOST+HEADSIZE
        DQ    0x0
        DQ    TWOSL
        DQ    N_TWOST
        DQ    0

        DQ      TWO, STAR
        DQ      SEMIS
;

;  **********
;  *   1-   *
;  **********
;
        ALIGN    8
N_ONEM:
        DQ      2
        DB      "1-"
        ALIGN    8
ONEM:
        DQ    DOCOL
        DQ    ONEM+HEADSIZE
        DQ    0x0
        DQ    TWOST
        DQ    N_ONEM
        DQ    0

        DQ      ONE, LSUB
        DQ      SEMIS
;

;  **************
;  *   FM/MOD   *
;  **************
;
        ALIGN    8
N_FMSLAS:
        DQ      6
        DB      "FM/MOD"
        ALIGN    8
FMSLAS:
        DQ    DOCOL
        DQ    FMSLAS+HEADSIZE
        DQ    0x0
        DQ    ONEM
        DQ    N_FMSLAS
        DQ    0

        DQ      LDUP, TOR
        DQ      TDUP, LXOR, TOR
        DQ      MSLAS
        DQ      FROMR, ZLESS
        DQ      ZBRAN
        DQ      FMMOD1-$-CW
        DQ      OVER
        DQ      ZBRAN
        DQ      FMMOD1-$-CW
        DQ      ONE, LSUB
        DQ      SWAP, FROMR, PLUS, SWAP
        DQ      BRAN
        DQ      FMMOD2-$-CW
FMMOD1:
        DQ      RDROP
FMMOD2:
        DQ      SEMIS
;

;  *********
;  *   *   *
;  *********
;
        ALIGN    8
N_STAR:
        DQ      1
        DB      "*"
        ALIGN    8
STAR:
        DQ    DOCOL
        DQ    STAR+HEADSIZE
        DQ    0x0
        DQ    FMSLAS
        DQ    N_STAR
        DQ    0

        DQ      MSTAR
        DQ      DROP
        DQ      SEMIS
;

;  ************
;  *   /MOD   *
;  ************
;
        ALIGN    8
N_SLMOD:
        DQ      4
        DB      "/MOD"
        ALIGN    8
SLMOD:
        DQ    DOCOL
        DQ    SLMOD+HEADSIZE
        DQ    0x0
        DQ    STAR
        DQ    N_SLMOD
        DQ    0

        DQ      TOR
        DQ      STOD
        DQ      FROMR
        DQ      MSLAS
        DQ      SEMIS
;

;  *********
;  *   /   *
;  *********
;
        ALIGN    8
N_SLASH:
        DQ      1
        DB      "/"
        ALIGN    8
SLASH:
        DQ    DOCOL
        DQ    SLASH+HEADSIZE
        DQ    0x0
        DQ    SLMOD
        DQ    N_SLASH
        DQ    0

        DQ      SLMOD
        DQ      NIP
        DQ      SEMIS
;

;  ***********
;  *   MOD   *
;  ***********
;
        ALIGN    8
N_LMOD:
        DQ      3
        DB      "MOD"
        ALIGN    8
LMOD:
        DQ    DOCOL
        DQ    LMOD+HEADSIZE
        DQ    0x0
        DQ    SLASH
        DQ    N_LMOD
        DQ    0

        DQ      SLMOD
        DQ      DROP
        DQ      SEMIS
;

;  *************
;  *   */MOD   *
;  *************
;
        ALIGN    8
N_SSMOD:
        DQ      5
        DB      "*/MOD"
        ALIGN    8
SSMOD:
        DQ    DOCOL
        DQ    SSMOD+HEADSIZE
        DQ    0x0
        DQ    LMOD
        DQ    N_SSMOD
        DQ    0

        DQ      TOR
        DQ      MSTAR
        DQ      FROMR
        DQ      MSLAS
        DQ      SEMIS
;

;  **********
;  *   */   *
;  **********
;
        ALIGN    8
N_SSLA:
        DQ      2
        DB      "*/"
        ALIGN    8
SSLA:
        DQ    DOCOL
        DQ    SSLA+HEADSIZE
        DQ    0x0
        DQ    SSMOD
        DQ    N_SSLA
        DQ    0

        DQ      SSMOD
        DQ      NIP
        DQ      SEMIS
;

;  *************
;  *   M/MOD   *
;  *************
;
        ALIGN    8
N_MSMOD1:
        DQ      5
        DB      "M/MOD"
        ALIGN    8
MSMOD1:
        DQ    DOCOL
        DQ    MSMOD1+HEADSIZE
        DQ    0x0
        DQ    SSLA
        DQ    N_MSMOD1
        DQ    0

;  ***************
;  *   UDM/MOD   *
;  ***************
;
        ALIGN    8
N_MSMOD:
        DQ      7
        DB      "UDM/MOD"
        ALIGN    8
MSMOD:
        DQ    DOCOL
        DQ    MSMOD+HEADSIZE
        DQ    0x0
        DQ    MSMOD1
        DQ    N_MSMOD
        DQ    0

        DQ      TOR
        DQ      ZERO
        DQ      RR
        DQ      USLAS
        DQ      FROMR
        DQ      SWAP
        DQ      TOR
        DQ      USLAS
        DQ      FROMR
        DQ      SEMIS
;

;  **************
;  *   (LINE)   *
;  **************
;
        ALIGN    8
N_PLINE:
        DQ      6
        DB      "(LINE)"
        ALIGN    8
PLINE:
        DQ    DOCOL
        DQ    PLINE+HEADSIZE
        DQ    0x0
        DQ    MSMOD
        DQ    N_PLINE
        DQ    0

        DQ      TOR
        DQ      LIT,64
        DQ      MSTAR
        DQ      BBUF
        DQ      FMSLAS
        DQ      FROMR ; This blocks, so is screens.
        DQ      PLUS
        DQ      BLOCK
        DQ      PLUS
        DQ      LIT,63
        DQ      SEMIS
;

;  **************
;  *   ERRSCR   *
;  **************
;
        ALIGN    8
N_ERRSCR:
        DQ      6
        DB      "ERRSCR"
        ALIGN    8
ERRSCR:
        DQ    DOVAR
        DQ    ERRSCR+HEADSIZE
        DQ    0x0
        DQ    PLINE
        DQ    N_ERRSCR
        DQ    0

        DQ ERRORSCREEN
;

;  ***************
;  *   MESSAGE   *
;  ***************
;
        ALIGN    8
N_MESS:
        DQ      7
        DB      "MESSAGE"
        ALIGN    8
MESS:
        DQ    DOCOL
        DQ    MESS+HEADSIZE
        DQ    0x0
        DQ    ERRSCR
        DQ    N_MESS
        DQ    0

        DQ      LWARN
        DQ      FETCH
        DQ      ZBRAN
        DQ      MESS1-$-CW ;IF
        DQ      ERRSCR, FETCH
        DQ      PLINE, ONEP     ; Also print the '\n' !
        DQ      ETYPE
        DQ      X
MESS1:                  ;THEN
        DQ      DROP
        DQ      SEMIS
;
;
;

;
;
_COMMENTED(
 Dit blok  PC@,PC!,PW@,PW! eerst maar even uitgeschakeld voor ARM !
 Dit moet waarschijnlijk helemaal anders als voor Intel
) 
;  ***********
;  *   PC@   *
;  ***********
;
        ALIGN    8
N_PCFET:
        DQ      3
        DB      "PC@"
        ALIGN    8
PCFET:
        DQ    PCFET+HEADSIZE
        DQ    PCFET+HEADSIZE
        DQ    0x0
        DQ    MESS
        DQ    N_PCFET
        DQ    0

; FETCH CHARACTER (BYTE) FROM PORT
        LDR     X1, [X13],#+(CW*(1))      ; PORT ADDR
        MOV     X0,#0
;todo        IN      AL,_DX16   ; BYTE INPUT
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   PC!   *
;  ***********
;
        ALIGN    8
N_PCSTO:
        DQ      3
        DB      "PC!"
        ALIGN    8
PCSTO:
        DQ    PCSTO+HEADSIZE
        DQ    PCSTO+HEADSIZE
        DQ    0x0
        DQ    PCFET
        DQ    N_PCSTO
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;PORT ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
;todo        OUT     _DX16,AL   ; BYTE OUTPUT
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   PW@   *
;  ***********
;
        ALIGN    8
N_PFET:
        DQ      3
        DB      "PW@"
        ALIGN    8
PFET:
        DQ    PFET+HEADSIZE
        DQ    PFET+HEADSIZE
        DQ    0x0
        DQ    PCSTO
        DQ    N_PFET
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;PORT ADDR
;todo        IN      _AX16,_DX16  ;WORD INPUT
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;

;  ***********
;  *   PW!   *
;  ***********
;
        ALIGN    8
N_PSTO:
        DQ      3
        DB      "PW!"
        ALIGN    8
PSTO:
        DQ    PSTO+HEADSIZE
        DQ    PSTO+HEADSIZE
        DQ    0x0
        DQ    PFET
        DQ    N_PSTO
        DQ    0

        LDR     X1, [X13],#+(CW*(1))      ;PORT ADDR
        LDR     X0, [X13],#+(CW*(1))      ;DATA
;todo        OUT     _DX16,_AX16   ;WORD OUTPUT
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1  
;
;
;

;  *************
;  *   _PREV   *
;  *************
;
        ALIGN    8
N__PREV:
        DQ      5
        DB      "_PREV"
        ALIGN    8
_PREV:
        DQ    DOVAR
        DQ    _PREV+HEADSIZE
        DQ    0x0
        DQ    PSTO
        DQ    N__PREV
        DQ    0

        DQ      BUF1

;
;

;  *************
;  *   #BUFF   *
;  *************
;
        ALIGN    8
N_NOBUF:
        DQ      5
        DB      "#BUFF"
        ALIGN    8
NOBUF:
        DQ    DOCON
        DQ    NBUF
        DQ    0x0
        DQ    _PREV
        DQ    N_NOBUF
        DQ    0

;

;  ************
;  *   +BUF   *
;  ************
;
        ALIGN    8
N_PBUF:
        DQ      4
        DB      "+BUF"
        ALIGN    8
PBUF:
        DQ    DOCOL
        DQ    PBUF+HEADSIZE
        DQ    0x0
        DQ    NOBUF
        DQ    N_PBUF
        DQ    0

        DQ      BBUF, CELLP, CELLP
        DQ      PLUS,LDUP
        DQ      _LIMIT,EQUAL
        DQ      ZBRAN
        DQ      PBUF1-$-CW
        DQ      DROP,_FIRST
PBUF1:  DQ      LDUP, _PREV, FETCH, LSUB
        DQ      SEMIS
;

;  **************
;  *   UPDATE   *
;  **************
;
        ALIGN    8
N_UPDAT:
        DQ      6
        DB      "UPDATE"
        ALIGN    8
UPDAT:
        DQ    DOCOL
        DQ    UPDAT+HEADSIZE
        DQ    0x0
        DQ    PBUF
        DQ    N_UPDAT
        DQ    0

        DQ      _PREV, FETCH
        DQ      LDUP, CELLP,CELLP
        DQ      SWAP, FETCH
        DQ      LOFFSET,  FETCH, PLUS
        DQ      BLWRITE
        DQ      SEMIS
;

;  *********************
;  *   EMPTY-BUFFERS   *
;  *********************
;
        ALIGN    8
N_MTBUF:
        DQ      13
        DB      "EMPTY-BUFFERS"
        ALIGN    8
MTBUF:
        DQ    DOCOL
        DQ    MTBUF+HEADSIZE
        DQ    0x0
        DQ    UPDAT
        DQ    N_MTBUF
        DQ    0

        DQ      _FIRST
        DQ      _LIMIT,OVER
        DQ      LSUB,LERASE
        DQ      _FIRST, _PREV, STOR
        DQ      SEMIS
        ;
;

;  ****************
;  *   (BUFFER)   *
;  ****************
;
        ALIGN    8
N_BUFFER:
        DQ      8
        DB      "(BUFFER)"
        ALIGN    8
BUFFER:
        DQ    DOCOL
        DQ    BUFFER+HEADSIZE
        DQ    0x0
        DQ    MTBUF
        DQ    N_BUFFER
        DQ    0

; Find the buffer, if it is already here.
    DQ      _PREV, FETCH
BUFFER1:
    DQ          TOR, RR, FETCH, OVER, EQUAL
    DQ      ZBRAN
        DQ      BUFFER3-$-CW
    DQ        DROP, FROMR, EXIT
BUFFER3:
    DQ          FROMR
    DQ      PBUF, ZEQU
    DQ      ZBRAN
        DQ      BUFFER1-$-CW
; We are left with the previous buffer.
BUFFER2:
    DQ       PBUF, ZEQU, LIT, 48, QERR  ; No free buffers available.
    DQ       LDUP, CELLP, FETCH
    DQ       LIT, -1, GREAT        ; Unlocked buffer?
    DQ      ZBRAN
        DQ      BUFFER2-$-CW
    DQ       TOR
; Fill in the house keeping.
    DQ       RR, STOR
    DQ       ZERO, RR, CELLP, STOR

    DQ       RR, _PREV, STOR
    DQ       FROMR
    DQ  SEMIS
;


;  *************
;  *   BLOCK   *
;  *************
;
        ALIGN    8
N_BLOCK:
        DQ      5
        DB      "BLOCK"
        ALIGN    8
BLOCK:
        DQ    DOCOL
        DQ    BLOCK+HEADSIZE
        DQ    0x0
        DQ    BUFFER
        DQ    N_BLOCK
        DQ    0


        DQ      BUFFER
        DQ      LDUP, CELLP, FETCH, ONE, LAND, ZEQU
        DQ      ZBRAN
        DQ      BLOCK1-$-CW
        DQ      LDUP, CELLP, CELLP
        DQ      OVER, FETCH
        DQ      LOFFSET,  FETCH, PLUS
        DQ      BLREAD
        DQ      LDUP, CELLP, ONE, TOGGL
BLOCK1:
        DQ      LDUP, _PREV, STOR
        DQ      CELLP, CELLP
        DQ      SEMIS
;

;  *************
;  *   FLUSH   *
;  *************
;
        ALIGN    8
N_FLUSH:
        DQ      5
        DB      "FLUSH"
        ALIGN    8
FLUSH:
        DQ    DOCOL
        DQ    (MTBUF+HEADSIZE)
        DQ    0x0
        DQ    BLOCK
        DQ    N_FLUSH
        DQ    0


; Unlock all buffers
        DQ      _LIMIT
        DQ      _FIRST, CELLP
        DQ     XDO
        DQ      FLUS2-$-CW
FLUS1:  DQ      ZERO, IDO, STOR
        DQ      LIT,(KBBUF+2*CW)
        DQ      PLOOP
        DQ      (FLUS1-$)
FLUS2:
        DQ      SEMIS
;

;  ************
;  *   SAVE   *
;  ************
;
        ALIGN    8
N_SAVE:
        DQ      4
        DB      "SAVE"
        ALIGN    8
SAVE:
        DQ    DOCOL
        DQ    SAVE+HEADSIZE
        DQ    0x0
        DQ    FLUSH
        DQ    N_SAVE
        DQ    0

        DQ      FROMR
        DQ      SRC, TFET
        DQ      PP, FETCH
        DQ      TOR, TOR, TOR
        DQ      TOR
        DQ SEMIS
;

;  ***************
;  *   RESTORE   *
;  ***************
;
        ALIGN    8
N_RESTO:
        DQ      7
        DB      "RESTORE"
        ALIGN    8
RESTO:
        DQ    DOCOL
        DQ    RESTO+HEADSIZE
        DQ    0x0
        DQ    SAVE
        DQ    N_RESTO
        DQ    0

        DQ      FROMR
        DQ      FROMR, FROMR, FROMR
        DQ      PP, STOR
        DQ      SRC, TSTOR
        DQ      TOR
        DQ SEMIS
;

; 
;

; 
;

;  ************
;  *   LOCK   *
;  ************
;
        ALIGN    8
N_LLOCK:
        DQ      4
        DB      "LOCK"
        ALIGN    8
LLOCK:
        DQ    DOCOL
        DQ    LLOCK+HEADSIZE
        DQ    0x0
        DQ    RESTO
        DQ    N_LLOCK
        DQ    0

        DQ      BUFFER
        DQ      CELLP
        DQ      LIT, -2, SWAP, PSTOR
        DQ      SEMIS
;

;  **************
;  *   UNLOCK   *
;  **************
;
        ALIGN    8
N_LUNLOCK:
        DQ      6
        DB      "UNLOCK"
        ALIGN    8
LUNLOCK:
        DQ    DOCOL
        DQ    LUNLOCK+HEADSIZE
        DQ    0x0
        DQ    LLOCK
        DQ    N_LUNLOCK
        DQ    0

        DQ      BUFFER
        DQ      CELLP
        DQ      TWO, SWAP, PSTOR
        DQ      SEMIS
;

;  ************
;  *   LOAD   *
;  ************
;
        ALIGN    8
N_LLOAD:
        DQ      4
        DB      "LOAD"
        ALIGN    8
LLOAD:
        DQ    DOCOL
        DQ    LLOAD+HEADSIZE
        DQ    0x0
        DQ    LUNLOCK
        DQ    N_LLOAD
        DQ    0

        DQ      SAVE
        DQ      TOR
        DQ      RR, LIT, LLOCK, CATCH  ; Be careful about always unlocking.
        DQ      LDUP, ZEQU
        DQ      ZBRAN
        DQ      LOAD5-$-CW
        DQ      DROP
        DQ      RR, BLOCK     ; Can't fail anymore. 
        DQ      BBUF
        DQ      SETSRC
        DQ      LIT, INTER, CATCH
        DQ      RR, LUNLOCK
LOAD5:
        DQ      RDROP
        DQ      RESTO
        DQ      THROW
        DQ      SEMIS
;

;  ************
;  *   THRU   *
;  ************
;
        ALIGN    8
N_THRU:
        DQ      4
        DB      "THRU"
        ALIGN    8
THRU:
        DQ    DOCOL
        DQ    THRU+HEADSIZE
        DQ    0x0
        DQ    LLOAD
        DQ    N_THRU
        DQ    0

        DQ      ONEP, SWAP
        DQ     XDO
        DQ      THRU2-$-CW
THRU1:
        DQ      IDO, LLOAD
        DQ     ONE, XPLOOP, ZBRAN
        DQ      THRU1-$-CW
	DQ      UNLOOP
THRU2:
        DQ      SEMIS
;
;

;  ***********
;  *   BLK   *
;  ***********
;
        ALIGN    8
N_BLK:
        DQ      3
        DB      "BLK"
        ALIGN    8
BLK:
        DQ    DOCOL
        DQ    BLK+HEADSIZE
        DQ    0x0
        DQ    THRU
        DQ    N_BLK
        DQ    0

        DQ      PP, FETCH
        DQ      _FIRST, _LIMIT, WITHIN
        DQ      SRC, TFET, LSUB
        DQ      LIT, 1024, EQUAL, LAND
        DQ      ZBRAN
        DQ      BLK1-$-CW
        DQ      SRC, FETCH, TWO, LCELLS, LSUB, FETCH
        DQ      BRAN
        DQ      BLK2-$-CW
BLK1:
        DQ      ZERO
BLK2:
        DQ      PBLK, STOR
        DQ      PBLK
        DQ      SEMIS
;

;



;  ***********
;  *   XOS   *
;  ***********
;
        ALIGN    8
N_XOS:
        DQ      3
        DB      "XOS"
        ALIGN    8
XOS:
        DQ    XOS+HEADSIZE
        DQ    XOS+HEADSIZE
        DQ    0x0
        DQ    BLK
        DQ    N_XOS
        DQ    0

;


        LDR     X8, [X13],#+(CW*(1))       ; Function number              
        LDR     X2, [X13],#+(CW*(1))       ; Third parameter, or dummy    
        LDR     X1, [X13],#+(CW*(1))       ; Second parameter, or dummy   
        LDR     X0, [X13],#+(CW*(1))       ; First parameter.             
        SVC     #0          ;  Generic call on LINUX       
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1       ; Positive means okay. Negative means -errno.
;
;
 ; 




;


;  ************
;  *   XOS5   *
;  ************
;
        ALIGN    8
N_XOS5:
        DQ      4
        DB      "XOS5"
        ALIGN    8
XOS5:
        DQ    XOS5+HEADSIZE
        DQ    XOS5+HEADSIZE
        DQ    0x0
        DQ    XOS
        DQ    N_XOS5
        DQ    0

        LDR     X8, [X13],#+(CW*(1))        ; Function number            
        LDR     X4, [X13],#+(CW*(1))        ; 5th parameter, or dummy
        LDR     X3, [X13],#+(CW*(1))        ; 4th parameter, or dummy
        LDR     X2, [X13],#+(CW*(1))        ; Third parameter, or dummy
        LDR     X1, [X13],#+(CW*(1))        ; Second parameter, or dummy
        LDR     X0, [X13],#+(CW*(1))        ; First parameter.
        SVC     #0           ; Generic call on LINUX      
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ; Positive means okay. Negative means -errno.
;


;

;  ************
;  *   XOS7   *
;  ************
;
        ALIGN    8
N_XOS7:
        DQ      4
        DB      "XOS7"
        ALIGN    8
XOS7:
        DQ    XOS7+HEADSIZE
        DQ    XOS7+HEADSIZE
        DQ    0x0
        DQ    XOS5
        DQ    N_XOS7
        DQ    0

        LDR     X8, [X13],#+(CW*(1))        ; Function number            
        LDR     X6, [X13],#+(CW*(1))        ; 7th parameter, or dummy
        LDR     X5, [X13],#+(CW*(1))        ; 6th parameter, or dummy
        LDR     X4, [X13],#+(CW*(1))        ; 5th parameter, or dummy
        LDR     X3, [X13],#+(CW*(1))        ; 4th parameter, or dummy
        LDR     X2, [X13],#+(CW*(1))        ; Third parameter, or dummy
        LDR     X1, [X13],#+(CW*(1))        ; Second parameter, or dummy
        LDR     X0, [X13],#+(CW*(1))        ; First parameter.
        SVC     #0           ; Generic call on LINUX      
        STR     X0, [X13,#-(CW*(1))]!
        LDR     X0, [X11],#+(CW*(1))
         LDR     X1,[X0]
         BR      X1            ; Positive means okay. Negative means -errno.

;

;
 ; 


;  **********
;  *   MS   *
;  **********
;
        ALIGN    8
N_MS:
        DQ      2
        DB      "MS"
        ALIGN    8
MS:
        DQ    DOCOL
        DQ    MS+HEADSIZE
        DQ    0x0
        DQ    XOS7
        DQ    N_MS
        DQ    0

; Use select because nanosleep has a 20 MS resolution. 
; 

        DQ      LIT, 1000000, LIT, 1000000000, SSMOD ; (ms -- ns s) 
        DQ      LIT, TV, TSTOR
        DQ      ZERO, ZERO
        DQ      ZERO, ZERO
        DQ      LIT, TV
        DQ      ZERO, ZERO
        DQ      LIT, select, XOS7
; 

        DQ      QERRUR
        DQ      SEMIS
TV:     RESB    (CW*(2))    ; A time_t and a long, both cells.
;
; 

        ;
;------------------------------------
;       CONSOLE RELEATED I/O         
;------------------------------------







;  *****************
;  *   RW-BUFFER   *
;  *****************
;
        ALIGN    8
N_RWBUF:
        DQ      9
        DB      "RW-BUFFER"
        ALIGN    8
RWBUF:
        DQ    DOVAR
        DQ    RWBUF+HEADSIZE
        DQ    0x0
        DQ    MS
        DQ    N_RWBUF
        DQ    0

        RESB    0x1000
; 

;


;  ***********
;  *   ZEN   *
;  ***********
;
        ALIGN    8
N_ZEN:
        DQ      3
        DB      "ZEN"
        ALIGN    8
ZEN:
        DQ    DOCOL
        DQ    ZEN+HEADSIZE
        DQ    0x0
        DQ    RWBUF
        DQ    N_ZEN
        DQ    0

        DQ      RWBUF, SSTOR
        DQ      ZERO, RWBUF, CHAPP
        DQ      RWBUF, CELLP

        DQ      SEMIS
; 



;


;  *****************
;  *   OPEN-FILE   *
;  *****************
;
        ALIGN    8
N_OFILE:
        DQ      9
        DB      "OPEN-FILE"
        ALIGN    8
OFILE:
        DQ    DOCOL
        DQ    OFILE+HEADSIZE
        DQ    0x0
        DQ    ZEN
        DQ    N_OFILE
        DQ    0


      DQ      TOR, ZEN
       DQ       LIT, AT_FDCWD, SWAP, FROMR
       DQ      LIT, openat, XOS
       DQ      LDUP, ZERO, MIN
       DQ      SEMIS 
;


;


;  ******************
;  *   CLOSE-FILE   *
;  ******************
;
        ALIGN    8
N_CFILE:
        DQ      10
        DB      "CLOSE-FILE"
        ALIGN    8
CFILE:
        DQ    DOCOL
        DQ    CFILE+HEADSIZE
        DQ    0x0
        DQ    OFILE
        DQ    N_CFILE
        DQ    0

        DQ      X, X
        DQ      LIT, close, XOS
        DQ      SEMIS
;


;



;  *******************
;  *   CREATE-FILE   *
;  *******************
;
        ALIGN    8
N_CREATEF:
        DQ      11
        DB      "CREATE-FILE"
        ALIGN    8
CREATEF:
        DQ    DOCOL
        DQ    CREATEF+HEADSIZE
        DQ    0x0
        DQ    CFILE
        DQ    N_CREATEF
        DQ    0


        DQ      TOR, TDUP, DFILE, DROP
      DQ      ZEN, LIT, AT_FDCWD, SWAP
        DQ     LIT, O_CREAT + O_TRUNC + O_WRONLY
        DQ      FROMR
        DQ      ZERO      ; fifth wheel
        DQ      LIT, openat, XOS5
        DQ      LDUP, ZERO, MIN
        DQ      SEMIS 

;


;


;  *******************
;  *   DELETE-FILE   *
;  *******************
;
        ALIGN    8
N_DFILE:
        DQ      11
        DB      "DELETE-FILE"
        ALIGN    8
DFILE:
        DQ    DOCOL
        DQ    DFILE+HEADSIZE
        DQ    0x0
        DQ    CREATEF
        DQ    N_DFILE
        DQ    0


      DQ      ZEN
     DQ     LIT, AT_FDCWD, SWAP, ZERO
        DQ      LIT, unlinkat, XOS
        DQ      SEMIS 
;


;


;  *****************
;  *   READ-FILE   *
;  *****************
;
        ALIGN    8
N_RFILE:
        DQ      9
        DB      "READ-FILE"
        ALIGN    8
RFILE:
        DQ    DOCOL
        DQ    RFILE+HEADSIZE
        DQ    0x0
        DQ    DFILE
        DQ    N_RFILE
        DQ    0

        DQ      ROT, ROT
        DQ      LIT, read, XOS
        DQ      LDUP, ZERO, MAX, SWAP, ZERO, MIN
        DQ      SEMIS
; 
;

;

;  *********************
;  *   FILE-POSITION   *
;  *********************
;
        ALIGN    8
N_FPFILE:
        DQ      13
        DB      "FILE-POSITION"
        ALIGN    8
FPFILE:
        DQ    DOCOL
        DQ    FPFILE+HEADSIZE
        DQ    0x0
        DQ    RFILE
        DQ    N_FPFILE
        DQ    0

        DQ      ZERO
        DQ      LIT, SEEK_CUR
        DQ      LIT, lseek, XOS
        DQ      ZERO     ;Double precision return result
        DQ      OVER, LDUP
        DQ      NEGATE, LIT, 256, ULESS, LAND ; Heuristic 
        DQ      SEMIS
 ; 

;


;  ***********************
;  *   REPOSITION-FILE   *
;  ***********************
;
        ALIGN    8
N_PFILE:
        DQ      15
        DB      "REPOSITION-FILE"
        ALIGN    8
PFILE:
        DQ    DOCOL
        DQ    PFILE+HEADSIZE
        DQ    0x0
        DQ    FPFILE
        DQ    N_PFILE
        DQ    0

        DQ      TOR, DROP, FROMR   ;Convert to single precision.
        DQ      SWAP
        DQ      LIT, SEEK_SET
        DQ      LIT, lseek, XOS
        DQ      ZERO, MIN
        DQ      SEMIS
 ; 



;


;  ******************
;  *   WRITE-FILE   *
;  ******************
;
        ALIGN    8
N_WFILE:
        DQ      10
        DB      "WRITE-FILE"
        ALIGN    8
WFILE:
        DQ    DOCOL
        DQ    WFILE+HEADSIZE
        DQ    0x0
        DQ    PFILE
        DQ    N_WFILE
        DQ    0

        DQ      ROT, ROT
        DQ      LIT, write, XOS
        DQ      ZERO, MIN
        DQ      SEMIS
; 



;


;  ****************
;  *   GET-FILE   *
;  ****************
;
        ALIGN    8
N_GETFILE:
        DQ      8
        DB      "GET-FILE"
        ALIGN    8
GETFILE:
        DQ    DOCOL
        DQ    GETFILE+HEADSIZE
        DQ    0x0
        DQ    WFILE
        DQ    N_GETFILE
        DQ    0

     DQ LALIGN
        DQ      TDUP, SCOMMA, DROP
        DQ      LIT, 'F'+(0x100*('i'+0x100*('L'+0x100*'e'))), COMMA ;Magic number.
        DQ      ZERO, OFILE, THROW, TOR
        DQ      HERE, LDUP
        DQ      LEM, LIT, 6, SLASH, LDUP, ALLOT
        DQ      LIT, 1000, LSUB
        DQ      RR, RFILE, THROW
        DQ      FROMR, CFILE, THROW
        DQ      TDUP, PLUS, LDP, STOR        ; No allocation if it fails.
        DQ      SEMIS
;

;  ****************
;  *   PUT-FILE   *
;  ****************
;
        ALIGN    8
N_PUTFILE:
        DQ      8
        DB      "PUT-FILE"
        ALIGN    8
PUTFILE:
        DQ    DOCOL
        DQ    PUTFILE+HEADSIZE
        DQ    0x0
        DQ    GETFILE
        DQ    N_PUTFILE
        DQ    0

        DQ      LIT, 0755
        DQ      CREATEF, THROW
        DQ      LDUP, TOR
        DQ      WFILE, THROW
        DQ      FROMR, CFILE, THROW
        DQ      SEMIS
;

;  ****************
;  *   INCLUDED   *
;  ****************
;
        ALIGN    8
N_INCLUDED:
        DQ      8
        DB      "INCLUDED"
        ALIGN    8
INCLUDED:
        DQ    DOCOL
        DQ    INCLUDED+HEADSIZE
        DQ    0x0
        DQ    PUTFILE
        DQ    N_INCLUDED
        DQ    0

        DQ      HERE, TOR
        DQ      LIT, GETFILE, CATCH
        DQ      LDUP
        DQ      ZBRAN
        DQ      INCLUD1-$-CW
        DQ      FROMR, LDP, STOR
        DQ      THROW
        DQ      BRAN
        DQ      INCLUD2-$-CW
INCLUD1:
        DQ      RDROP, DROP
INCLUD2:
        DQ      EVALUATE
        DQ      SEMIS
;

;  ***************
;  *   INCLUDE   *
;  ***************
;
        ALIGN    8
N_LINCLUD:
        DQ      7
        DB      "INCLUDE"
        ALIGN    8
LINCLUD:
        DQ    DOCOL
        DQ    LINCLUD+HEADSIZE
        DQ    0x0
        DQ    INCLUDED
        DQ    N_LINCLUD
        DQ    0

        DQ      LNAME, INCLUDED
        DQ      SEMIS
;

;  ******************
;  *   REFILL-TIB   *
;  ******************
;
        ALIGN    8
N_REFTIB:
        DQ      10
        DB      "REFILL-TIB"
        ALIGN    8
REFTIB:
        DQ    DOCOL
        DQ    REFTIB+HEADSIZE
        DQ    0x0
        DQ    LINCLUD
        DQ    N_REFTIB
        DQ    0

        DQ      REMAIND, FETCH, TOR
        DQ      TIB, FETCH, RR, PLUS
        DQ      LIT, TIBSIZE, RR, LSUB
        DQ      ZERO
        DQ      RFILE
        DQ      QERRUR
        DQ      LDUP, ZEQU, LIT, -EPIPE, LAND ; Presumably end of pipe.
        DQ      QERRUR
        DQ      TIB, FETCH, SWAP, FROMR, PLUS
        DQ      REMAIND, TSTOR
        DQ      SEMIS
;
;

;

; All I/O goes through the modern device (Unix-standard)
; device I/O. Code fields are filled in during bootup. 
; Lower case labels starting with "c_.." are c-supplied facilities.

;  **************
;  *   ACCEPT   *
;  **************
;
        ALIGN    8
N_ACCEP:
        DQ      6
        DB      "ACCEPT"
        ALIGN    8
ACCEP:
        DQ    DOCOL
        DQ    ACCEP+HEADSIZE
        DQ    0x0
        DQ    REFTIB
        DQ    N_ACCEP
        DQ    0

        DQ      PACCEP
        DQ      TSWAP, ROT, MIN
        DQ      LDUP, TOR, LMOVE, FROMR
        DQ      SEMIS
;

;  ****************
;  *   (ACCEPT)   *
;  ****************
;
        ALIGN    8
N_PACCEP:
        DQ      8
        DB      "(ACCEPT)"
        ALIGN    8
PACCEP:
        DQ    DOCOL
        DQ    PACCEP+HEADSIZE
        DQ    0x0
        DQ    ACCEP
        DQ    N_PACCEP
        DQ    0

PACCEP2:
        DQ      REMAIND
        DQ      TFET
        DQ      LIT, ALF, SINDEX
        DQ      ZEQU
        DQ      ZBRAN
        DQ      PACCEP1-$-CW
        DQ      REMAIND, TFET
        DQ      TIB, FETCH
        DQ      SWAP, LMOVE
        DQ      TIB, FETCH
        DQ      REMAIND, CELLP, STOR
        DQ      REFTIB
        DQ      BRAN
        DQ      PACCEP2-$-CW
PACCEP1:
        DQ      REMAIND, TFET
        DQ      LIT, ALF, SSLAS
        DQ      TSWAP, REMAIND, TSTOR
        DQ      SEMIS


;  ***********
;  *   KEY   *
;  ***********
;
        ALIGN    8
N_KEY:
        DQ      3
        DB      "KEY"
        ALIGN    8
KEY:
        DQ    DOCOL
        DQ    KEY+HEADSIZE
        DQ    0x0
        DQ    PACCEP
        DQ    N_KEY
        DQ    0

        DQ      ONE, LIT, RAWIO, STTERM ; Do not wait for CR
        DQ      ZERO, SPFET     ; Expect single key on stack
        DQ      ZERO            ; Standard input
        DQ      SWAP, ONE       ; Buffer and length
        DQ      LIT, read, XOS
        DQ      LDUP, QERRUR
        DQ      ZEQU, LIT, -EPIPE, LAND, QERRUR ; Presumably end of pipe.
        DQ      ONE, LIT, RAWIO, STTERM       ; Toggle back
        DQ      SEMIS
;

;  ************
;  *   KEY?   *
;  ************
;
        ALIGN    8
N_KEYQ:
        DQ      4
        DB      "KEY?"
        ALIGN    8
KEYQ:
        DQ    DOCOL
        DQ    KEYQ+HEADSIZE
        DQ    0x0
        DQ    KEY
        DQ    N_KEYQ
        DQ    0

        DQ      ZERO, LIT, RAWIO, STTERM
        DQ      ZERO, ZERO     ; Time out of zero. See MS.
        DQ      LIT, TV, TSTOR
        DQ      ONE ; Waiting for standard in.
        DQ      LIT, RFDS, LSTOR
        DQ      ONE     ; # of highest channel to watch.
        DQ      LIT, RFDS
        DQ      ZERO, ZERO  ; Not output, not events.
        DQ      LIT, TV, ZERO, ZERO, LIT, select, XOS7
        
        DQ      LDUP,  QERRUR
        DQ      NEGATE        ; 1 -> flag.
        DQ      ZERO, LIT, RAWIO, STTERM
        DQ      SEMIS

        ALIGN    8
RFDS:   RESB    (CW*(32))     ;32 channels maximum.

;

;  ************
;  *   TYPE   *
;  ************
;
        ALIGN    8
N_LTYPE:
        DQ      4
        DB      "TYPE"
        ALIGN    8
LTYPE:
        DQ    DOCOL
        DQ    LTYPE+HEADSIZE
        DQ    0x0
        DQ    KEYQ
        DQ    N_LTYPE
        DQ    0

        DQ      LDUP, LOUT, PSTOR
        DQ      ONE, WFILE, THROW
        DQ      SEMIS
;

;  *************
;  *   ETYPE   *
;  *************
;
        ALIGN    8
N_ETYPE:
        DQ      5
        DB      "ETYPE"
        ALIGN    8
ETYPE:
        DQ    DOCOL
        DQ    ETYPE+HEADSIZE
        DQ    0x0
        DQ    LTYPE
        DQ    N_ETYPE
        DQ    0

        DQ      TWO, WFILE, THROW
        DQ      SEMIS
;

;  ************
;  *   EMIT   *
;  ************
;
        ALIGN    8
N_EMIT:
        DQ      4
        DB      "EMIT"
        ALIGN    8
EMIT:
        DQ    DOCOL
        DQ    EMIT+HEADSIZE
        DQ    0x0
        DQ    ETYPE
        DQ    N_EMIT
        DQ    0

        DQ      SPFET, ONE, LTYPE
        DQ      DROP
        DQ      SEMIS
;


;  **************
;  *   TERMIO   *
;  **************
;
        ALIGN    8
N_TERMIO:
        DQ      6
        DB      "TERMIO"
        ALIGN    8
TERMIO:
        DQ    DOVAR
        DQ    TERMIO+HEADSIZE
        DQ    0x0
        DQ    EMIT
        DQ    N_TERMIO
        DQ    0

        RESB    SIZE_TERMIO     ; Filled in during cold boot.
;

;  ****************
;  *   SET-TERM   *
;  ****************
;
        ALIGN    8
N_STTERM:
        DQ      8
        DB      "SET-TERM"
        ALIGN    8
STTERM:
        DQ    DOCOL
        DQ    STTERM+HEADSIZE
        DQ    0x0
        DQ    TERMIO
        DQ    N_STTERM
        DQ    0

SIZEL   EQU 4     ; Also for 64 bits. 
        DQ      LIT, (TERMIO+HEADSIZE)+SIZEL*3
        DQ      SWAP, TOGGL
        DQ      LIT, (TERMIO+HEADSIZE)+SIZEL*4+7
        DQ      CSTOR
        DQ      ZERO, LIT, TCSETS, TERMIO
        DQ      LIT, ioctl, XOS, QERRUR
        DQ      SEMIS
;
; 






;

        ;
;------------------------------------
;       SYSTEM DEPENDANT DISK I/O
;------------------------------------

;  ******************
;  *   DISK-ERROR   *
;  ******************
;
        ALIGN    8
N_DERR:
        DQ      10
        DB      "DISK-ERROR"
        ALIGN    8
DERR:
        DQ    DOVAR
        DQ    DERR+HEADSIZE
        DQ    0x0
        DQ    STTERM
        DQ    N_DERR
        DQ    0

        DQ      -1
;


;  ******************
;  *   BLOCK-FILE   *
;  ******************
;
        ALIGN    8
N_BLFL:
        DQ      10
        DB      "BLOCK-FILE"
        ALIGN    8
BLFL:
        DQ    DOVAR
        DQ    BLFL+HEADSIZE
        DQ    0x0
        DQ    DERR
        DQ    N_BLFL
        DQ    0

        DQ      9
        DB      "forth.lab"
        RESB    252-9               ; Allow for some path

;  ********************
;  *   BLOCK-HANDLE   *
;  ********************
;
        ALIGN    8
N_BHAN:
        DQ      12
        DB      "BLOCK-HANDLE"
        ALIGN    8
BHAN:
        DQ    DOVAR
        DQ    BHAN+HEADSIZE
        DQ    0x0
        DQ    BLFL
        DQ    N_BHAN
        DQ    0

        DQ      -1
;

;  *******************
;  *   ?DISK-ERROR   *
;  *******************
;
        ALIGN    8
N_QDSKER:
        DQ      11
        DB      "?DISK-ERROR"
        ALIGN    8
QDSKER:
        DQ    DOCOL
        DQ    QDSKER+HEADSIZE
        DQ    0x0
        DQ    BHAN
        DQ    N_QDSKER
        DQ    0

        DQ      LIT, 8, QERR
        DQ      SEMIS
;
;


;  ******************
;  *   BLOCK-INIT   *
;  ******************
;
        ALIGN    8
N_BLINI:
        DQ      10
        DB      "BLOCK-INIT"
        ALIGN    8
BLINI:
        DQ    DOCOL
        DQ    BLINI+HEADSIZE
        DQ    0x0
        DQ    QDSKER
        DQ    N_BLINI
        DQ    0

        DQ      BLFL, SFET, DTRAI
        DQ      ROT
        DQ      OFILE
        DQ      ZEQU, NEGATE  ; 0 if disk problems, 1 if not.
        DQ      LWARN, FETCH, MIN ; AND but WARNING is 0/1.
        DQ      LWARN, STOR
        DQ      BHAN, STOR
        DQ      MTBUF
        DQ      SEMIS
;

;  ******************
;  *   BLOCK-EXIT   *
;  ******************
;
        ALIGN    8
N_BLEXI:
        DQ      10
        DB      "BLOCK-EXIT"
        ALIGN    8
BLEXI:
        DQ    DOCOL
        DQ    BLEXI+HEADSIZE
        DQ    0x0
        DQ    BLINI
        DQ    N_BLEXI
        DQ    0

        DQ      FLUSH
        DQ      BHAN, FETCH
        DQ      CFILE
        DQ      ZERO, LWARN, STOR
        DQ      LIT, -1, BHAN, STOR    ;Regardless of close errors.
        DQ      QDSKER
        DQ      SEMIS
;

;  **************
;  *   BLSEEK   *
;  **************
;
        ALIGN    8
N_BLSEEK:
        DQ      6
        DB      "BLSEEK"
        ALIGN    8
BLSEEK:
        DQ    DOCOL
        DQ    BLSEEK+HEADSIZE
        DQ    0x0
        DQ    BLEXI
        DQ    N_BLSEEK
        DQ    0

        DQ      BBUF
        DQ      USTAR
        DQ      BHAN, FETCH
        DQ      PFILE           ; Disk position.
        DQ      QDSKER
        DQ      SEMIS
;

;  ******************
;  *   BLOCK-READ   *
;  ******************
;
        ALIGN    8
N_BLREAD:
        DQ      10
        DB      "BLOCK-READ"
        ALIGN    8
BLREAD:
        DQ    DOCOL
        DQ    BLREAD+HEADSIZE
        DQ    0x0
        DQ    BLSEEK
        DQ    N_BLREAD
        DQ    0

        DQ      BLSEEK ; That's done
        DQ      BBUF
        DQ      BHAN, FETCH
        DQ      RFILE           ; Disk read 
        DQ      SWAP, BBUF, UNEQ
        DQ      LOR, QDSKER
        DQ      SEMIS
;

;  *******************
;  *   BLOCK-WRITE   *
;  *******************
;
        ALIGN    8
N_BLWRITE:
        DQ      11
        DB      "BLOCK-WRITE"
        ALIGN    8
BLWRITE:
        DQ    DOCOL
        DQ    BLWRITE+HEADSIZE
        DQ    0x0
        DQ    BLREAD
        DQ    N_BLWRITE
        DQ    0

        DQ      BLSEEK ; That's done
        DQ      BBUF
        DQ      BHAN, FETCH
        DQ      WFILE           ; Disk write
        DQ      QDSKER
        DQ      SEMIS
;
;

;  *************
;  *   SHELL   *
;  *************
;
        ALIGN    8
N_SHELL:
        DQ      5
        DB      "SHELL"
        ALIGN    8
SHELL:
        DQ    DOVAR
        DQ    SHELL+HEADSIZE
        DQ    0x0
        DQ    BLWRITE
        DQ    N_SHELL
        DQ    0

                 DQ      7
        DB      "/bin/sh"

        RESB    252-9               ; Allow for some path

;


;  **************
;  *   SYSTEM   *
;  **************
;
        ALIGN    8
N_SYSTEM:
        DQ      6
        DB      "SYSTEM"
        ALIGN    8
SYSTEM:
        DQ    DOCOL
        DQ    SYSTEM+HEADSIZE
        DQ    0x0
        DQ    SHELL
        DQ    N_SYSTEM
        DQ    0

          DQ      LIT, COMBUF, SSTOR, ZERO, LIT, COMBUF, CHAPP
          DQ      ZERO, SHELL, CHAPP, LIT, -1, SHELL, PSTOR

         DQ      ZERO, ZERO, ZERO, ZERO, ZERO, LIT, clone, XOS5
          DQ      LDUP, QERRUR
          DQ      LDUP, ZEQU
          DQ      ZBRAN
        DQ      SYSTEM1-$-CW
        DQ      SHELL, CELLP, LIT, SYSARGS
;  Get environment vector
      DQ      ARGS, FETCH, SFET, ONEP, LCELLS, PLUS
        DQ      LIT, execve, XOS
        DQ      LIT, 49, LERROR, BYE   ; Returned, this is always wrong.
SYSTEM1:
; int waitid(idtype_t idtype, id_t id, siginfo_t * infop , int options ) 

        DQ      LDUP   ; pid of child
        DQ      LIT, P_PID, SWAP
        DQ      LIT, SIGINFOT
        DQ      LIT, WEXITED, ZERO
        DQ      LIT, waitid, XOS5
        DQ      LDUP
        DQ      LIT, EINTR, EQUAL
        DQ      BRAN
        DQ      SYSTEM2-$-CW
        DQ      DROP
        DQ      ZBRAN
        DQ      SYSTEM1-$-CW
SYSTEM2:
        DQ      QERRUR
        DQ      TDROP
        DQ      SEMIS
;
        ALIGN    8
SIGINFOT: RESB    CW*9
SYSARG2: DB     "-c"
         DB      0
        ALIGN    8
COMBUF:  RESB    CW+256  ;One cell for high level string manipulation.
        ALIGN    8
SYSARGS:DQ   (SHELL+HEADSIZE)+CW, SYSARG2, COMBUF + CW, 0
;
 ; 



;



;


;

;  *********
;  *   '   *
;  *********
;
        ALIGN    8
N_ITICK:
        DQ      1
        DB      "'"
        ALIGN    8
ITICK:
        DQ    DOCOL 
        DQ    ITICK+HEADSIZE
        DQ    0x0
        DQ    SYSTEM
        DQ    N_ITICK
        DQ    0

        DQ      LNAME, PRESENT
        DQ      LDUP, ZEQU
        DQ      LIT, 11, QERR
        DQ      SEMIS
;

;  ***********
;  *   [']   *
;  ***********
;
        ALIGN    8
N_BTICK:
        DQ      3
        DB      "[']"
        ALIGN    8
BTICK:
        DQ    DOCOL
        DQ    (TICK+HEADSIZE)
        DQ    B_IMMED
        DQ    ITICK
        DQ    N_BTICK
        DQ    0

;

;  ******************
;  *   FORGET-VOC   *
;  ******************
;
        ALIGN    8
N_FORGV:
        DQ      10
        DB      "FORGET-VOC"
        ALIGN    8
FORGV:
        DQ    DOCOL
        DQ    FORGV+HEADSIZE
        DQ    0x0
        DQ    BTICK
        DQ    N_FORGV
        DQ    0

        DQ      TDUP
        DQ      SWAP
        DQ      ULESS
        DQ      ZBRAN
        DQ      FORGV1-$-CW
;  Forget part of contents.
        DQ      SWAP
        DQ      TOR
        DQ      TWID
FORGV0:
        DQ      LDUP
        DQ      LDUP
FORGV3:
        DQ      TLFA,FETCH
        DQ      LDUP
        DQ      RR
        DQ      ULESS
        DQ      ZBRAN
        DQ      FORGV3-$-CW
        DQ      SWAP            ;  Not to be forgotten entry found.
        DQ      TLFA
        DQ      STOR           ;  Short other entries out.
        DQ      TLFA,FETCH
        DQ      LDUP
        DQ      ZEQU
        DQ      ZBRAN
        DQ      FORGV0-$-CW        ;  Repeat until end of wordlist.
        DQ      DROP
        DQ      FROMR
        DQ      BRAN
        DQ      FORGV2-$-CW
FORGV1:
;       { Vocabulary itself is also forgotten.}
        DQ      TVFA
        DQ      FETCH     ; Unlink by linking next namespace.
        DQ      VOCL
        DQ      STOR
        DQ      ONLY, FORTH
        DQ      DEFIN
FORGV2: DQ      SEMIS
;

;  **************
;  *   FORGET   *
;  **************
;
        ALIGN    8
N_FORG:
        DQ      6
        DB      "FORGET"
        ALIGN    8
FORG:
        DQ    DOCOL
        DQ    FORG+HEADSIZE
        DQ    0x0
        DQ    FORGV
        DQ    N_FORG
        DQ    0

        DQ      TICK
        DQ      LDUP
        DQ      FENCE
        DQ      FETCH
        DQ      LESS
        DQ      LIT, 21, QERR
        DQ      LIT,FORGV
        DQ      FORV
        DQ      TNFA, FETCH, LDP, STOR
        DQ      SEMIS
;

;  *************
;  *   (BACK   *
;  *************
;
        ALIGN    8
N_PBACK:
        DQ      5
        DB      "(BACK"
        ALIGN    8
PBACK:
        DQ    DOCOL
        DQ    PBACK+HEADSIZE
        DQ    0x0
        DQ    FORG
        DQ    N_PBACK
        DQ    0

        DQ      HERE
        DQ      SEMIS
;

;  *************
;  *   BACK)   *
;  *************
;
        ALIGN    8
N_BACKP:
        DQ      5
        DB      "BACK)"
        ALIGN    8
BACKP:
        DQ    DOCOL
        DQ    BACKP+HEADSIZE
        DQ    0x0
        DQ    PBACK
        DQ    N_BACKP
        DQ    0

        DQ      HERE
        DQ      CELLP
        DQ      LSUB
        DQ      COMMA
        DQ      SEMIS
;

;  ****************
;  *   (FORWARD   *
;  ****************
;
        ALIGN    8
N_PFORWARD:
        DQ      8
        DB      "(FORWARD"
        ALIGN    8
PFORWARD:
        DQ    DOCOL
        DQ    PFORWARD+HEADSIZE
        DQ    0x0
        DQ    BACKP
        DQ    N_PFORWARD
        DQ    0

        DQ      HERE
        DQ      X
        DQ      COMMA
        DQ      SEMIS
;

;  ****************
;  *   FORWARD)   *
;  ****************
;
        ALIGN    8
N_FORWARDP:
        DQ      8
        DB      "FORWARD)"
        ALIGN    8
FORWARDP:
        DQ    DOCOL
        DQ    FORWARDP+HEADSIZE
        DQ    0x0
        DQ    PFORWARD
        DQ    N_FORWARDP
        DQ    0

        DQ      HERE
        DQ      OVER
        DQ      CELLP
        DQ      LSUB
        DQ      SWAP
        DQ      STOR
        DQ      SEMIS
;

;  *************
;  *   BEGIN   *
;  *************
;
        ALIGN    8
N_BEGIN:
        DQ      5
        DB      "BEGIN"
        ALIGN    8
BEGIN:
        DQ    DOCOL
        DQ    BEGIN+HEADSIZE
        DQ    B_IMMED
        DQ    FORWARDP
        DQ    N_BEGIN
        DQ    0

        DQ      PBACK
        DQ      QCOMP, ONE
        DQ      SEMIS
;

;  ************
;  *   THEN   *
;  ************
;
        ALIGN    8
N_THEN:
        DQ      4
        DB      "THEN"
        ALIGN    8
THEN:
        DQ    DOCOL
        DQ    THEN+HEADSIZE
        DQ    B_IMMED
        DQ    BEGIN
        DQ    N_THEN
        DQ    0

        DQ      QCOMP, TWO, QPAIR
        DQ      FORWARDP
        DQ      SEMIS
;

;  **********
;  *   DO   *
;  **********
;
        ALIGN    8
N_DO:
        DQ      2
        DB      "DO"
        ALIGN    8
DO:
        DQ    DOCOL
        DQ    DO+HEADSIZE
        DQ    B_IMMED
        DQ    THEN
        DQ    N_DO
        DQ    0

         DQ      LIT, XDO, COMMA, PFORWARD, PBACK
        DQ      QCOMP, LIT,3    ; Magic number
        DQ      SEMIS
;

;  ***********
;  *   ?DO   *
;  ***********
;
        ALIGN    8
N_QDO:
        DQ      3
        DB      "?DO"
        ALIGN    8
QDO:
        DQ    DOCOL
        DQ    QDO+HEADSIZE
        DQ    B_IMMED
        DQ    DO
        DQ    N_QDO
        DQ    0

         DQ      LIT, XQDO, COMMA, PFORWARD, PBACK
        DQ      QCOMP, LIT,3    ; Magic number
        DQ      SEMIS
;

;  ************
;  *   LOOP   *
;  ************
;
        ALIGN    8
N_LLOOP:
        DQ      4
        DB      "LOOP"
        ALIGN    8
LLOOP:
        DQ    DOCOL
        DQ    LLOOP+HEADSIZE
        DQ    B_IMMED
        DQ    QDO
        DQ    N_LLOOP
        DQ    0

        DQ      LIT, ONE, COMMA ; POSTPONE 1 
        DQ      PLOOP
        DQ      SEMIS
;

;  *************
;  *   +LOOP   *
;  *************
;
        ALIGN    8
N_PLOOP:
        DQ      5
        DB      "+LOOP"
        ALIGN    8
PLOOP:
        DQ    DOCOL
        DQ    PLOOP+HEADSIZE
        DQ    B_IMMED
        DQ    LLOOP
        DQ    N_PLOOP
        DQ    0

        DQ      QCOMP, LIT, 3, QPAIR
        DQ      LIT, XPLOOP, COMMA
        DQ      LIT, ZBRAN, COMMA, BACKP
        DQ      LIT, UNLOOP, COMMA
        DQ      FORWARDP ; For DO to push the leave address.
        DQ      SEMIS
;

;  *************
;  *   UNTIL   *
;  *************
;
        ALIGN    8
N_UNTIL:
        DQ      5
        DB      "UNTIL"
        ALIGN    8
UNTIL:
        DQ    DOCOL
        DQ    UNTIL+HEADSIZE
        DQ    B_IMMED
        DQ    PLOOP
        DQ    N_UNTIL
        DQ    0

        DQ      QCOMP, ONE, QPAIR
        DQ      LIT, ZBRAN, COMMA, BACKP
        DQ      SEMIS
;

;  *************
;  *   AGAIN   *
;  *************
;
        ALIGN    8
N_AGAIN:
        DQ      5
        DB      "AGAIN"
        ALIGN    8
AGAIN:
        DQ    DOCOL
        DQ    AGAIN+HEADSIZE
        DQ    B_IMMED
        DQ    UNTIL
        DQ    N_AGAIN
        DQ    0

        DQ      QCOMP, ONE, QPAIR
        DQ      LIT, BRAN, COMMA, BACKP
        DQ      SEMIS
;

;  **************
;  *   REPEAT   *
;  **************
;
        ALIGN    8
N_REPEA:
        DQ      6
        DB      "REPEAT"
        ALIGN    8
REPEA:
        DQ    DOCOL
        DQ    REPEA+HEADSIZE
        DQ    B_IMMED
        DQ    AGAIN
        DQ    N_REPEA
        DQ    0

        DQ      QCOMP, ONE, QPAIR   ; Matches BEGIN ?
        DQ      LIT, BRAN, COMMA, BACKP
        DQ      LIT, 4, QPAIR ; Matches WHILE ?
        DQ      FORWARDP ; WHILE target. 
        DQ      SEMIS
;

;  **********
;  *   IF   *
;  **********
;
        ALIGN    8
N_LIF:
        DQ      2
        DB      "IF"
        ALIGN    8
LIF:
        DQ    DOCOL
        DQ    LIF+HEADSIZE
        DQ    B_IMMED
        DQ    REPEA
        DQ    N_LIF
        DQ    0

        DQ      LIT, ZBRAN, COMMA, PFORWARD
        DQ      QCOMP, TWO     ; Magic number
        DQ      SEMIS
;

;  ************
;  *   ELSE   *
;  ************
;
        ALIGN    8
N_LELSE:
        DQ      4
        DB      "ELSE"
        ALIGN    8
LELSE:
        DQ    DOCOL
        DQ    LELSE+HEADSIZE
        DQ    B_IMMED
        DQ    LIF
        DQ    N_LELSE
        DQ    0

        DQ      QCOMP, TWO, QPAIR
        DQ      LIT, BRAN, COMMA, PFORWARD
        DQ      SWAP
        DQ      FORWARDP
        DQ      TWO     ; Magic number
        DQ      SEMIS
;

;  *************
;  *   WHILE   *
;  *************
;
        ALIGN    8
N_LWHILE:
        DQ      5
        DB      "WHILE"
        ALIGN    8
LWHILE:
        DQ    DOCOL
        DQ    LWHILE+HEADSIZE
        DQ    B_IMMED
        DQ    LELSE
        DQ    N_LWHILE
        DQ    0

        DQ      QCOMP, LDUP, ONE, QPAIR
        DQ      TOR    ;  Save backward target. 
        DQ      TOR
        DQ      LIT, ZBRAN, COMMA, PFORWARD
        DQ      LIT, 4 ; Magic number
        DQ      FROMR
        DQ      FROMR
        DQ      SEMIS
;

;  **************
;  *   SPACES   *
;  **************
;
        ALIGN    8
N_SPACES:
        DQ      6
        DB      "SPACES"
        ALIGN    8
SPACES:
        DQ    DOCOL
        DQ    SPACES+HEADSIZE
        DQ    0x0
        DQ    LWHILE
        DQ    N_SPACES
        DQ    0

        DQ      ZERO
        DQ      MAX
        DQ      ZERO
        DQ     XQDO
        DQ      SPAX1-$-CW
SPAX2:  DQ      SPACE
        DQ     ONE, XPLOOP, ZBRAN
        DQ      SPAX2-$-CW
	DQ      UNLOOP    ;LOOP
SPAX1:
        DQ      SEMIS
;

;  **********
;  *   <#   *
;  **********
;
        ALIGN    8
N_BDIGS:
        DQ      2
        DB      "<#"
        ALIGN    8
BDIGS:
        DQ    DOCOL
        DQ    BDIGS+HEADSIZE
        DQ    0x0
        DQ    SPACES
        DQ    N_BDIGS
        DQ    0

        DQ      PAD
        DQ      HLD
        DQ      STOR
        DQ      SEMIS
;

;  **********
;  *   #>   *
;  **********
;
        ALIGN    8
N_EDIGS:
        DQ      2
        DB      "#>"
        ALIGN    8
EDIGS:
        DQ    DOCOL
        DQ    EDIGS+HEADSIZE
        DQ    0x0
        DQ    BDIGS
        DQ    N_EDIGS
        DQ    0

        DQ      DROP
        DQ      DROP
        DQ      HLD
        DQ      FETCH
        DQ      PAD
        DQ      OVER
        DQ      LSUB
        DQ      SEMIS
;

;  ************
;  *   SIGN   *
;  ************
;
        ALIGN    8
N_SIGN:
        DQ      4
        DB      "SIGN"
        ALIGN    8
SIGN:
        DQ    DOCOL
        DQ    SIGN+HEADSIZE
        DQ    0x0
        DQ    EDIGS
        DQ    N_SIGN
        DQ    0

        DQ      ZLESS
        DQ      ZBRAN
        DQ      SIGN1-$-CW ;IF
        DQ      LIT, AMS
        DQ      HOLD    ;THEN
SIGN1:  DQ      SEMIS
;

;  *********
;  *   #   *
;  *********
;
        ALIGN    8
N_DIG:
        DQ      1
        DB      "#"
        ALIGN    8
DIG:
        DQ    DOCOL
        DQ    DIG+HEADSIZE
        DQ    0x0
        DQ    SIGN
        DQ    N_DIG
        DQ    0

        DQ      BASE
        DQ      FETCH
        DQ      MSMOD
        DQ      ROT
        DQ      LIT,9
        DQ      OVER
        DQ      LESS
        DQ      ZBRAN
        DQ      DIG1-$-CW  ;IF
        DQ      LIT,7
        DQ      PLUS    ;THEN
DIG1:   DQ      LIT,0x30
        DQ      PLUS
        DQ      HOLD
        DQ      SEMIS
;

;  **********
;  *   #S   *
;  **********
;
        ALIGN    8
N_DIGS:
        DQ      2
        DB      "#S"
        ALIGN    8
DIGS:
        DQ    DOCOL
        DQ    DIGS+HEADSIZE
        DQ    0x0
        DQ    DIG
        DQ    N_DIGS
        DQ    0

DIGS1:  DQ      DIG     ;BEGIN
        DQ      OVER
        DQ      OVER
        DQ      LOR
        DQ      ZEQU
        DQ      ZBRAN
        DQ      DIGS1-$-CW ;UNTIL
        DQ      SEMIS
;
; 

;  *************
;  *   (D.R)   *
;  *************
;
        ALIGN    8
N_PDDOTR:
        DQ      5
        DB      "(D.R)"
        ALIGN    8
PDDOTR:
        DQ    DOCOL
        DQ    PDDOTR+HEADSIZE
        DQ    0x0
        DQ    DIGS
        DQ    N_PDDOTR
        DQ    0

        DQ      TOR
        DQ      SWAP
        DQ      OVER
        DQ      DABS
        DQ      BDIGS
        DQ      DIGS
        DQ      ROT
        DQ      SIGN
        DQ      EDIGS
        DQ      FROMR
        DQ      OVER
        DQ      LSUB, ZERO, MAX
        DQ      ZERO
        DQ     XQDO
        DQ      PDDOT1-$-CW
PDDOT2:  DQ      LBL, HOLD  ;WARNING: HOLD outside of #>.
        DQ     ONE, XPLOOP, ZBRAN
        DQ      PDDOT2-$-CW
	DQ      UNLOOP
PDDOT1:
        DQ      EDIGS  ;Drop string instead of number.
        DQ      SEMIS
;

;  ***********
;  *   D.R   *
;  ***********
;
        ALIGN    8
N_DDOTR:
        DQ      3
        DB      "D.R"
        ALIGN    8
DDOTR:
        DQ    DOCOL
        DQ    DDOTR+HEADSIZE
        DQ    0x0
        DQ    PDDOTR
        DQ    N_DDOTR
        DQ    0

        DQ      PDDOTR
        DQ      LTYPE
        DQ      SEMIS
;

;  **********
;  *   .R   *
;  **********
;
        ALIGN    8
N_DOTR:
        DQ      2
        DB      ".R"
        ALIGN    8
DOTR:
        DQ    DOCOL
        DQ    DOTR+HEADSIZE
        DQ    0x0
        DQ    DDOTR
        DQ    N_DOTR
        DQ    0

        DQ      TOR
        DQ      STOD
        DQ      FROMR
        DQ      DDOTR
        DQ      SEMIS
;

;  **********
;  *   D.   *
;  **********
;
        ALIGN    8
N_DDOT:
        DQ      2
        DB      "D."
        ALIGN    8
DDOT:
        DQ    DOCOL
        DQ    DDOT+HEADSIZE
        DQ    0x0
        DQ    DOTR
        DQ    N_DDOT
        DQ    0

        DQ      ZERO
        DQ      DDOTR
        DQ      SPACE
        DQ      SEMIS
;

;  *********
;  *   .   *
;  *********
;
        ALIGN    8
N_DOT:
        DQ      1
        DB      "."
        ALIGN    8
DOT:
        DQ    DOCOL
        DQ    DOT+HEADSIZE
        DQ    0x0
        DQ    DDOT
        DQ    N_DOT
        DQ    0

        DQ      STOD
        DQ      DDOT
        DQ      SEMIS
;

;  *********
;  *   ?   *
;  *********
;
        ALIGN    8
N_QUES:
        DQ      1
        DB      "?"
        ALIGN    8
QUES:
        DQ    DOCOL
        DQ    QUES+HEADSIZE
        DQ    0x0
        DQ    DOT
        DQ    N_QUES
        DQ    0

        DQ      FETCH
        DQ      DOT
        DQ      SEMIS
;

;  **********
;  *   U.   *
;  **********
;
        ALIGN    8
N_UDOT:
        DQ      2
        DB      "U."
        ALIGN    8
UDOT:
        DQ    DOCOL
        DQ    UDOT+HEADSIZE
        DQ    0x0
        DQ    QUES
        DQ    N_UDOT
        DQ    0

        DQ      ZERO
        DQ      DDOT
        DQ      SEMIS
;

;  *****************
;  *   FOR-WORDS   *
;  *****************
;
        ALIGN    8
N_FORW:
        DQ      9
        DB      "FOR-WORDS"
        ALIGN    8
FORW:
        DQ    DOCOL
        DQ    FORW+HEADSIZE
        DQ    0x0
        DQ    UDOT
        DQ    N_FORW
        DQ    0

        DQ      SWAP
        DQ      TOR
        DQ      TOR
FORW1:  DQ      FROMR
        DQ      RR
        DQ      OVER
        DQ      TLFA
        DQ      FETCH
        DQ      TOR
        DQ      EXEC
        DQ      RR
        DQ      ZEQU
        DQ      ZBRAN
        DQ      FORW1-$-CW
        DQ      RDROP
        DQ      RDROP
        DQ      SEMIS
;

;  ****************
;  *   FOR-VOCS   *
;  ****************
;
        ALIGN    8
N_FORV:
        DQ      8
        DB      "FOR-VOCS"
        ALIGN    8
FORV:
        DQ    DOCOL
        DQ    FORV+HEADSIZE
        DQ    0x0
        DQ    FORW
        DQ    N_FORV
        DQ    0

        DQ      TOR
        DQ      VOCL
        DQ      FETCH
        DQ      TOR
FORV1:  DQ      FROMR
        DQ      RR
        DQ      OVER
        DQ      TVFA
        DQ      FETCH
        DQ      TOR
        DQ      EXEC
        DQ      RR
        DQ      ZEQU
        DQ      ZBRAN
        DQ      FORV1-$-CW
        DQ      RDROP
        DQ      RDROP
        DQ      SEMIS
;

;  *************
;  *   WORDS   *
;  *************
;
        ALIGN    8
N_WORDS:
        DQ      5
        DB      "WORDS"
        ALIGN    8
WORDS:
        DQ    DOCOL
        DQ    WORDS+HEADSIZE
        DQ    0x0
        DQ    FORV
        DQ    N_WORDS
        DQ    0

        DQ      CSLL
        DQ      LOUT
        DQ      STOR
        DQ      ZERO            ; End sentinel. 
        DQ      LIT, NOOP
        DQ      CONTEXT
        DQ      FETCH
        DQ      FORW
WORDS1:
        DQ      LDUP
        DQ      ZBRAN
        DQ      WORDS2-$-CW
        DQ      IDDOT
        DQ      BRAN
        DQ      WORDS1-$-CW
WORDS2:
        DQ DROP
        DQ      SEMIS
;



;  *****************
;  *   EXIT-CODE   *
;  *****************
;
        ALIGN    8
N_XCODE:
        DQ      9
        DB      "EXIT-CODE"
        ALIGN    8
XCODE:
        DQ    DOVAR
        DQ    XCODE+HEADSIZE
        DQ    0x0
        DQ    WORDS
        DQ    N_XCODE
        DQ    0

        DQ      0
;


;  ***********
;  *   BYE   *
;  ***********
;
        ALIGN    8
N_BYE:
        DQ      3
        DB      "BYE"
        ALIGN    8
BYE:
        DQ    DOCOL
        DQ    BYE+HEADSIZE
        DQ    0x0
        DQ    XCODE
        DQ    N_BYE
        DQ    0

; Exit to linux, with status as stored. 
        DQ      XCODE, FETCH, X, X, LIT, exit, XOS
        DQ      SEMIS   ;Unnecessary, but helpful for decompilation.
; 
;

;  ************
;  *   LIST   *
;  ************
;
        ALIGN    8
N_LLIST:
        DQ      4
        DB      "LIST"
        ALIGN    8
LLIST:
        DQ    DOCOL
        DQ    LLIST+HEADSIZE
        DQ    0x0
        DQ    BYE
        DQ    N_LLIST
        DQ    0

        DQ      BASE, FETCH, TOR
        DQ      DECA
        DQ      SCR,STOR

        DQ      SKIP
         DQ      6
SB3: DB      "SCR # "
       ALIGN    8
        DQ      LIT, SB3
        DQ      LIT, 6
        DQ      LTYPE
        DQ      SCR, FETCH, DOT
        DQ      SCR, FETCH, BLOCK, BBUF

        DQ      LIT, 16, ZERO
        DQ     XDO
        DQ      LLIST9-$-CW
LLIST1: DQ      LIT, ALF, SSLAS
        DQ      CR, IDO, LIT, 2, DOTR
        DQ      SPACE, LTYPE
        DQ     ONE, XPLOOP, ZBRAN
        DQ      LLIST1-$-CW
	DQ      UNLOOP
LLIST9:
        DQ      TDROP
        DQ      FROMR, BASE, STOR
        DQ      SEMIS
;

;  *************
;  *   INDEX   *
;  *************
;
        ALIGN    8
N_INDEX:
        DQ      5
        DB      "INDEX"
        ALIGN    8
INDEX:
        DQ    DOCOL
        DQ    INDEX+HEADSIZE
        DQ    0x0
        DQ    LLIST
        DQ    N_INDEX
        DQ    0

        DQ      LIT,AFF
        DQ      EMIT,CR
        DQ      ONEP,SWAP
        DQ     XDO
        DQ      INDE9-$-CW
INDE1:  DQ      CR,IDO
        DQ      LIT,3
        DQ      DOTR,SPACE
        DQ      ZERO,IDO
        DQ      PLINE, LTYPE
        DQ     ONE, XPLOOP, ZBRAN
        DQ      INDE1-$-CW
	DQ      UNLOOP
INDE9:
        DQ      SEMIS
;

;  **********
;  *   .S   *
;  **********
;
        ALIGN    8
N_DOTS:
        DQ      2
        DB      ".S"
        ALIGN    8
DOTS:
        DQ    DOCOL
        DQ    DOTS+HEADSIZE
        DQ    0x0
        DQ    INDEX
        DQ    N_DOTS
        DQ    0

        DQ      CR
        DQ      LIT, 'S', EMIT
        DQ      LIT, ASO, EMIT
        DQ      SPACE
        DQ      SPFET, SZERO, FETCH
DOC2:   DQ      OVER, OVER,  LSUB, ZLESS
        DQ      ZBRAN
        DQ      DOC1-$-CW
        DQ      ZERO, CELLP, LSUB, LDUP, FETCH
        DQ      DOT
        DQ      BRAN
        DQ      DOC2-$-CW
DOC1:    DQ DROP, DROP
        DQ      LIT, ASC, EMIT
        DQ SEMIS
;

;  ********************
;  *   ENVIRONMENT?   *
;  ********************
;
        ALIGN    8
N_ENVQ:
        DQ      12
        DB      "ENVIRONMENT?"
        ALIGN    8
ENVQ:
        DQ    DOCOL
        DQ    ENVQ+HEADSIZE
        DQ    0x0
        DQ    DOTS
        DQ    N_ENVQ
        DQ    0

        DQ      LIT, ENV, TWID, PFIND
        DQ      TOR, TDROP, FROMR
        DQ      LDUP
        DQ      ZBRAN
        DQ      ENVQ1-$-CW
        DQ      EXEC
        DQ      LIT, -1
ENVQ1:
        DQ      SEMIS
;

; 
;
; This word is not even fig!

;  ***************
;  *   .SIGNON   *
;  ***************
;
        ALIGN    8
N_SIGNON:
        DQ      7
        DB      ".SIGNON"
        ALIGN    8
SIGNON:
        DQ    DOCOL
        DQ    SIGNON+HEADSIZE
        DQ    0x0
        DQ    ENVQ
        DQ    N_SIGNON
        DQ    0

; PRINT CPU TYPE (8088)
        DQ      CR
        DQ      BASE,FETCH
        DQ      LIT,36, BASE,STOR
        DQ      LCPU, DDOT
        DQ      BASE,STOR
;
        DQ      LENAME, LTYPE, SPACE
        DQ      LVERSION, LTYPE, SPACE
        DQ      CR
        DQ      SEMIS
;

;
;**** LAST DICTIONARY WORD OF FORTH ****

;  ************
;  *   TASK   *
;  ************
;
        ALIGN    8
N_TASK:
        DQ      4
        DB      "TASK"
        ALIGN    8
TASK:
        DQ    DOCOL
        DQ    TASK+HEADSIZE
        DQ    0x0
        DQ    SIGNON
        DQ    N_TASK
        DQ    0

        DQ      SEMIS
;
; End of ONLY namespace. Namespaces all end in a link to 0.

;  *********
;  *   '   *
;  *********
;
        ALIGN    8
N_TICK:
        DQ      1
        DB      "'"
        ALIGN    8
TICK:
        DQ    DOCOL
        DQ    TICK+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    0
        DQ    N_TICK
        DQ    0

        DQ      ITICK
        DQ      LITER
        DQ      SEMIS
;

;  *********
;  *   &   *
;  *********
;
        ALIGN    8
N_DCHAR:
        DQ      1
        DB      "&"
        ALIGN    8
DCHAR:
        DQ    DOCOL
        DQ    DCHAR+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    TICK
        DQ    N_DCHAR
        DQ    0

        DQ      PPFET
        DQ      NIP
        DQ      LDUP, QBL
        DQ      LIT, 10, QERR
        DQ      LITER
        DQ      QDELIM
        DQ      SEMIS

;

;  *********
;  *   ^   *
;  *********
;
        ALIGN    8
N_DCTL:
        DQ      1
        DB      "^"
        ALIGN    8
DCTL:
        DQ    DOCOL
        DQ    DCTL+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    DCHAR
        DQ    N_DCTL
        DQ    0

        DQ      PPFET
        DQ      NIP
        DQ      LDUP, QBL
        DQ      LIT, 10, QERR
        DQ      LIT, '@', LSUB
        DQ      LITER
        DQ      QDELIM
        DQ      SEMIS
;

;  *********
;  *   0   *
;  *********
;
        ALIGN    8
N_DEN0:
        DQ      1
        DB      "0"
        ALIGN    8
DEN0:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DCTL
        DQ    N_DEN0
        DQ    0

;  *********
;  *   1   *
;  *********
;
        ALIGN    8
N_DEN1:
        DQ      1
        DB      "1"
        ALIGN    8
DEN1:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN0
        DQ    N_DEN1
        DQ    0

;  *********
;  *   2   *
;  *********
;
        ALIGN    8
N_DEN2:
        DQ      1
        DB      "2"
        ALIGN    8
DEN2:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN1
        DQ    N_DEN2
        DQ    0

;  *********
;  *   3   *
;  *********
;
        ALIGN    8
N_DEN3:
        DQ      1
        DB      "3"
        ALIGN    8
DEN3:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN2
        DQ    N_DEN3
        DQ    0

;  *********
;  *   4   *
;  *********
;
        ALIGN    8
N_DEN4:
        DQ      1
        DB      "4"
        ALIGN    8
DEN4:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN3
        DQ    N_DEN4
        DQ    0

;  *********
;  *   5   *
;  *********
;
        ALIGN    8
N_DEN5:
        DQ      1
        DB      "5"
        ALIGN    8
DEN5:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN4
        DQ    N_DEN5
        DQ    0

;  *********
;  *   6   *
;  *********
;
        ALIGN    8
N_DEN6:
        DQ      1
        DB      "6"
        ALIGN    8
DEN6:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN5
        DQ    N_DEN6
        DQ    0


;  *********
;  *   7   *
;  *********
;
        ALIGN    8
N_DEN7:
        DQ      1
        DB      "7"
        ALIGN    8
DEN7:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN6
        DQ    N_DEN7
        DQ    0

;  *********
;  *   8   *
;  *********
;
        ALIGN    8
N_DEN8:
        DQ      1
        DB      "8"
        ALIGN    8
DEN8:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN7
        DQ    N_DEN8
        DQ    0

;  *********
;  *   9   *
;  *********
;
        ALIGN    8
N_DEN9:
        DQ      1
        DB      "9"
        ALIGN    8
DEN9:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN8
        DQ    N_DEN9
        DQ    0

;  *********
;  *   A   *
;  *********
;
        ALIGN    8
N_DENA:
        DQ      1
        DB      "A"
        ALIGN    8
DENA:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEN9
        DQ    N_DENA
        DQ    0

;  *********
;  *   B   *
;  *********
;
        ALIGN    8
N_DENB:
        DQ      1
        DB      "B"
        ALIGN    8
DENB:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DENA
        DQ    N_DENB
        DQ    0


;  *********
;  *   C   *
;  *********
;
        ALIGN    8
N_DENC:
        DQ      1
        DB      "C"
        ALIGN    8
DENC:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DENB
        DQ    N_DENC
        DQ    0

;  *********
;  *   D   *
;  *********
;
        ALIGN    8
N_DEND:
        DQ      1
        DB      "D"
        ALIGN    8
DEND:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DENC
        DQ    N_DEND
        DQ    0

;  *********
;  *   E   *
;  *********
;
        ALIGN    8
N_DENE:
        DQ      1
        DB      "E"
        ALIGN    8
DENE:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DEND
        DQ    N_DENE
        DQ    0

;  *********
;  *   F   *
;  *********
;
        ALIGN    8
N_DENF:
        DQ      1
        DB      "F"
        ALIGN    8
DENF:
        DQ    DOCOL
        DQ    LNUMB
        DQ    B_IMMED + B_DENOT
        DQ    DENE
        DQ    N_DENF
        DQ    0

;

;  *********
;  *   -   *
;  *********
;
        ALIGN    8
N_DENM:
        DQ      1
        DB      "-"
        ALIGN    8
DENM:
        DQ    DOCOL
        DQ    DENM+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    DENF
        DQ    N_DENM
        DQ    0

        DQ      PNUMB, DNEGA, SDLITE
        DQ      SEMIS
;

;  *********
;  *   +   *
;  *********
;
        ALIGN    8
N_DENP:
        DQ      1
        DB      "+"
        ALIGN    8
DENP:
        DQ    DOCOL
        DQ    DENP+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    DENM
        DQ    N_DENP
        DQ    0

        DQ      PNUMB, SDLITE
        DQ      SEMIS
;

;  *********
;  *   "   *
;  *********
;
        ALIGN    8
N_DENQ:
        DQ      1
        DB      '"'
        ALIGN    8
DENQ:
        DQ    DOCOL
        DQ    DENQ+HEADSIZE
        DQ    B_IMMED + B_DENOT
        DQ    DENP
        DQ    N_DENQ
        DQ    0

        DQ      LIT, SKIP, COMMA        ;  'SKIP , HERE >R 0 ,
        DQ      HERE, TOR, ZERO, COMMA
DENQ1:
        DQ      LIT, '"', PARSE         ;           BEGIN &" PARSE
        DQ      PPFET, LDUP, LIT, '"', EQUAL ;           PP@@ DUP &" =
        DQ      ZBRAN
        DQ      DENQ2-$-CW                 ;           WHILE
        DQ      TDROP, ONEP             ;           2DROP 1+ R@ $+!
        DQ      LDUP, ALLOT, RR, SADD
        DQ      BRAN
        DQ      DENQ1-$-CW                  ;           REPEAT
DENQ2:
        DQ      QBL, ZEQU
        DQ      LIT, 10, QERR           ;           ?BLANK 0= 10 ?ERROR
        DQ      DROP                    ;                DROP 
        DQ      LDUP, ALLOT, RR, SADD     ;         DUP ALLOT R@ $+!
    DQ LALIGN
        DQ      FROMR, SFET, DLITE      ;           R> $@ POSTPONE DLITERAL ;
        DQ      SEMIS
;

; The FORTH word is the only non-denotation in the ONLY namespace.
;  *************
;  *   FORTH   *
;  *************
;
        ALIGN    8
N_FORTH:
        DQ      5
        DB      "FORTH"
        ALIGN    8
FORTH:
        DQ    DODOE
        DQ    FORTH+HEADSIZE
        DQ    0x0
        DQ    DENQ
        DQ    N_FORTH
        DQ    0

        DQ      DOVOC
        DQ      0       ; END OF NAMESPACE LIST

        ALIGN    8

        DQ    0x0
        DQ    0
        DQ    B_DUMMY
        DQ    TASK
        DQ    (ZERO+(CW*(D_HOFFSET)))
        DQ    0

;
STARTOFONLY    EQU FORTH
;
;

;  ************
;  *   CORE   *
;  ************
;
        ALIGN    8
N_CORE:
        DQ      4
        DB      "CORE"
        ALIGN    8
CORE:
        DQ    DOCOL
        DQ    CORE+HEADSIZE
        DQ    0x0
        DQ    0
        DQ    N_CORE
        DQ    0

        DQ      ZERO    ; Not (fully) present.
        DQ      SEMIS
;

;  ***********
;  *   CPU   *
;  ***********
;
        ALIGN    8
N_LCPU:
        DQ      3
        DB      "CPU"
        ALIGN    8
LCPU:
        DQ    DOCOL
        DQ    LCPU+HEADSIZE
        DQ    0x0
        DQ    CORE
        DQ    N_LCPU
        DQ    0

             ; 'ARMV7'
       DQ      LIT, 0x0113F684, ZERO      ; 'ARMV8'
        DQ      SEMIS
;

;  ***************
;  *   VERSION   *
;  ***************
;
        ALIGN    8
N_LVERSION:
        DQ      7
        DB      "VERSION"
        ALIGN    8
LVERSION:
        DQ    DOCOL
        DQ    LVERSION+HEADSIZE
        DQ    0x0
        DQ    LCPU
        DQ    N_LVERSION
        DQ    0

_COMMENTED(
_C{       If this is there it is an official release}
        DC      SKIP
        DC      22
IBMPC:  DSS     "IBM-PC ciforth"
        DB      FIGREL+0x40,ADOT,FIGREV+0x30,ADOT,USRVER+0x30
        DC      LIT, IBMPC, LIT, 22
) ;       If M4_VERSION exists and contains a . it is an official release
        DQ      SKIP
         DQ      5
SB4: DB      "5.4.1"
       ALIGN    8
        DQ      LIT, SB4
        DQ      LIT, 5
        DQ      SEMIS
;

;  ************
;  *   NAME   *
;  ************
;
        ALIGN    8
N_LENAME:
        DQ      4
        DB      "NAME"
        ALIGN    8
LENAME:
        DQ    DOCOL
        DQ    LENAME+HEADSIZE
        DQ    0x0
        DQ    LVERSION
        DQ    N_LENAME
        DQ    0

        DQ      SKIP
         DQ      7
SB5: DB      "ciforth"
       ALIGN    8
        DQ      LIT, SB5
        DQ      LIT, 7
        DQ      SEMIS
;

;  ****************
;  *   SUPPLIER   *
;  ****************
;
        ALIGN    8
N_SUPPLIER:
        DQ      8
        DB      "SUPPLIER"
        ALIGN    8
SUPPLIER:
        DQ    DOCOL
        DQ    SUPPLIER+HEADSIZE
        DQ    0x0
        DQ    LENAME
        DQ    N_SUPPLIER
        DQ    0

        DQ      SKIP
         DQ      40
SB6: DB      "Albert van der Horst/Frans van der Markt"
       ALIGN    8
        DQ      LIT, SB6
        DQ      LIT, 40
        DQ      SEMIS
;
STARTOFENVIRONMENT    EQU   SUPPLIER
;
;
 ;  

_COMMENTED(

The remaining memory ( up to 'EM' ) is
used for:

        1. EXTENSION DICTIONARY
        2. PARAMETER STACK
        3. TERMINAL INPUT BUFFER
        4. RETURN STACK
        5. USER VARIABLE AREA
        6. DISK BUFFERS (UNLESS REQUIRED <1 MBYTE)


) 

;       This is the proper way to do it.
;       No memory addresses should be arrived at through equates.
;       However now we must teach the linker to keep the
;       two sections together.

FORTHSIZE       EQU     $ - _main
; We must use bss, because of the c-library.
; The dictionary is not a normal bss, because it is executable.
; The linker -N option takes care of that. 
       _BSS_

INITDP:                 ;  It may be that it is not consecutive with TASK.
                        ;  Because of bad linking, resulting in a unusable Forth.
        BUFFERSIZE      EQU  (KBBUF+2*CW)*NBUF

; For more task the TAS area is duplicated; the remainder is the same.
        RESB    EM - FORTHSIZE - TAS - BUFFERSIZE 
; INITR1:                         Return stack for next task
;          RESB    US              ;  For next task, if present. 
; The following framework may be repeated for more tasks.
; It is floating, i.e. it may be shifted up and down, in order
; to have INITR0 at a STACKSIZE boundary. 
        RESB    STACKSIZE - US    ; Data stack, only limited if more tasks present.
INITS0:                         ; Grows down.
STRTIB: RESB    TIBSIZE     ; Terminal input buffer.
        RESB    STACKSIZE       ; Return stack, grows down.
INITR0:                         ; Grows down
STRUSA: RESB    US              ; User area
BUF1:   RESB    BUFFERSIZE      ; FIRST DISK BUFFER
ACTUAL_EM:  ; Differs from EM if code is relocated.

        
; 
;


        ;
_COMMENTED(

  MISC. NOTES AND SCATTERED THOUGHTS

- Remember that all the FORTH words in this version are
  upper case letters.  Use <CAPS LOCK> when in FORTH.



- Subscribe to FORTH Dimensions.  It is a valuable source
  of system and application ideas.  Talking with fellow
  FORTH programmers is sure to stir up some exciting ideas.
  Consider joining a FIG chapter.  See the back of FORTH
  Dimensions for more info.

) 
; Define the entry point, not valid for auto booting.
        
        ; _ORIG


























;





